#!/bin/sh

cat <<EOF
This script will create the tables and load the small MovieLens 
data set that is available from www.grouplens.org to test if the 
cfengine is working properly with the text-based console client 
(bin/cf_client)

EOF

Usage="usage: $0 [-a jdbc|mysql]"

#
# Command line arguments
#
# -a jdbc|mysql (default is java) Method used to load data from files into DB
#

CF_PATH=`dirname $0`/../..
export CF_PATH
. $CF_PATH/config.sh
curdir=`pwd`

# The mysql load scripts currently don't work under Windows.
if [ "$WINDOWS" -eq 1 ]; then
	DB=jdbc
else
	DB=mysql
fi

# Process command line arguments
while getopts ha: opt
do
        case $opt in 
        a)      DB=$OPTARG;;
        h|\?)     echo $Usage
                exit 1;
        esac
done

tarfile=$CF_PATH/Data/MovieLens/ml-data.tar.gz
untarred_dir=ml-data

if test ! -f "$tarfile" ; then
cat <<EOF
You need to first download http://www.grouplens.org/data/ml-data.tar.gz
to the Data/MovieLens directory (which is most likely your current directory)
EOF
	exit 1;
fi

echo "Loading data from $tarfile"
echo

cat <<EOF
This action will destroy all cfengine related tables in the 
database '$SqlDbName', if such a database already exists. 

****Are you sure you want to continue? (Y/N)
EOF

read ans

case $ans in 
	Y|y|yes|Yes|YES) 
		echo "Proceeding..."
	 ;;
	*)
		exit 1;	
	;;
esac



echo "Extracting data files from $tarfile"

gunzip < $tarfile | ( cd $CF_PATH/Data/MovieLens && tar xf - )


#JAVA="$JAVA_HOME/bin/java"; export JAVA

CF_CLASSPATH="$CF_PATH/lib/cfengine.jar:$CF_PATH/lib/mysql.jar"
if [ -n "${CLASSPATH}" ]; then
        CF_CLASSPATH="$CF_CLASSPATH:$CLASSPATH"
fi
export CF_CLASSPATH

if [ $WINDOWS -eq 1 ]; then
	CF_CLASSPATH=`cygpath -wp $CF_CLASSPATH`
	w_CF_PATH=`cygpath -w $CF_PATH`
fi

ML_dir=$CF_PATH/Data/MovieLens/$untarred_dir

echo
echo "Converting ML data to correct input file format"
echo "   Extracting movie genres as categories"
$JAVA -cp $CF_CLASSPATH -Dcf.path=$w_CF_PATH org.recommender.util.ConvertMLItemData \
	$ML_dir

# Can't have types with id=0 - HACK
sed 's/|0/|20/' $ML_dir/u.genre > tmp$$
mv tmp$$ $ML_dir/u.genre
sed 's/\t0/\t20/' $ML_dir/itemtype.txt > tmp$$
mv tmp$$ $ML_dir/itemtype.txt

awk 'BEGIN { OFS="\t"; FS="|" } { print $2, $1 }' \
	< $ML_dir/u.genre \
	> $ML_dir/typedata.txt

# WE can't yet handle quotes in text statements, so this is a total hack
#sed s/\'//g < $ML_dir/typedata.txt > tmp$$
#mv tmp$$ $ML_dir/typedata.txt

echo "   Converting ratings"
awk 'BEGIN {OFS="\t"} { print $1, $2, $3 }' \
        < $ML_dir/u.data \
        > tmp$$
mv tmp$$ $ML_dir/u.data

echo "   Converting item name mappings"
awk 'BEGIN {OFS="\t"; FS="|" } { print $1, $2 }' \
        < $ML_dir/u.item \
        > $ML_dir/itemdata.txt

echo "   Creating dummy user name to user id mappings"
awk 'BEGIN { OFS="\t"; FS="|" } {  print $1, $1 }' \
	< $ML_dir/u.user \
	> $ML_dir/userdata.txt

echo 
echo "Running 'bin/initialize_tables.sh'"
# JDBC doesn't have an initialize script
$CF_PATH/bin/initialize_tables.sh -f -a mysql

echo 
echo "Running 'bin/load_data.sh'"
$CF_PATH/bin/load_data.sh -f $ML_dir/u.data \
	-t $ML_dir/itemtype.txt -a $DB 
echo
echo "Creating and loading DB tables needed for client only"

echo "   Creating and loading table mapping item names(text) to item ids"
$CF_PATH/bin/java_jdbc/make_item_db $ML_dir/itemdata.txt

echo "   Creating and loading table mapping user names(text) to user ids"
$CF_PATH/bin/java_jdbc/make_user_db $ML_dir/userdata.txt

echo "   Creating and loading table mapping item category names(text) to category ids"
$CF_PATH/bin/java_jdbc/make_type_db $ML_dir/typedata.txt

echo

echo "Removing extracted data files"
rm -rf ml-data
