#!/bin/sh

####################################################################
# shell command for building the mysql database.
####################################################################


Usage="usage: $0 [-u sql_user] [-p sql_password] [-s iuf|entropy|none] [-a jdbc|mysql] -f ratings_file -t itemtype_file"

#
# Command line arguments
#
# -u SQL_USER     	(default set in config.sh)
# -p SQL_PASSWORD 	(default set in config.sh)
# -s iuf|entropy|none  	(default set is iuf)           Method used to select users to load into server cache
# -a jdbc|mysql   	(default is java)              Method used to load data from files into DB tables
# -f ratings_file 	(mandatory)                    File containing ratings
# -t itemtype_file 	(mandatory)                    File describing what items belong to what categories (types)


# Table names are set in config.sh
CF_PATH=`dirname $0`/..
export CF_PATH
. $CF_PATH/config.sh

# Sampling mode default
MODE="iuf"

DB=jdbc

# Process command line arguments
while getopts hu:p:f:t:s:a: opt
do
        case $opt in 
       	u)      SqlUser=$OPTARG ; note=1 ;;
	p)      SqlPassword=$OPTARG; note=1 ;;
	s)	MODE=$OPTARG;;
	a)	DB=$OPTARG;;
	f)      RatingsDataFile=$OPTARG;;
	t)	ItemTypeDataFile=$OPTARG;;
        h|\?)     echo $Usage
                exit 1;
        esac
done

if [ -z "$RatingsDataFile" ]; then
	echo "You must specify the location of the file containing the ratings you wish to load"
	echo $Usage
	exit 1
fi

if [ -z "$ItemTypeDataFile" ]; then
	echo "NOTICE: You did not specify a itemtype_file, thus gettopn_by_type will not work"; 
	echo "        If you don't know what this means, you can safely ignore it";
	echo
fi

echo "This script:"; 
echo "     * will load data from flat files into your database"; 
echo "     * assumes that you have already created your database (but not the tables)"
echo "     * will append new ratings to any existing ratings in the table"
echo


case $DB in
	jdbc) 
		echo "Loading data using JAVA/JDBC - portable but slow";
		if [ "$note" = 1 ]; then
			echo "WARNING: -u and -p options ignored when using JDBC"
		fi 		
		$CF_PATH/bin/java_jdbc/make_rating_db $RatingsDataFile
		echo "###"
		if [ "$ItemTypeDataFile" != "" ]; then
			$CF_PATH/bin/java_jdbc/make_itemtype_db $ItemTypeDataFile
		fi
		;;
	mysql)
		echo "Loading data using MySQL LOAD LOCAL INFILE"; echo
		SUBARGS="-u $SqlUser -p $SqlPassword -f $RatingsDataFile"	
		if [ "$ItemTypeDataFile" != "" ]; then
			SUBARGS="$SUBARGS -t $ItemTypeDataFile"
		fi
		eval $CF_PATH/bin/mysql/build_db $SUBARGS 
                ;;
esac


#JAVA="$JAVA_HOME/bin/java"; export JAVA

CF_CLASSPATH="$CF_PATH/lib/cfengine.jar:$CF_PATH/lib/mysql.jar"
if [ -n "${CLASSPATH}" ]; then
        CF_CLASSPATH="$CF_CLASSPATH:$CLASSPATH"
fi
export CF_CLASSPATH

if [ $WINDOWS -eq 1 ]; then
	CF_CLASSPATH=`cygpath -wp $CF_CLASSPATH`
	CF_PATH=`cygpath -w $CF_PATH`
fi

if test "$MODE" = "iuf" -o "$MODE" = "entropy" ; then
	echo "Now computing the utility of users"
	echo "   Users are sampled in decreasing order of utility"
	echo "   When sampling is turned on, users are sampled in decreasing order of utility"; 
	echo
	exec $JAVA -cp $CF_CLASSPATH -Dcf.path=$CF_PATH org.recommender.util.CalcUserInfo $ARG $MODE
elif [ "$MODE" = "none" ]; then
	echo "Not computing utility"
else
	echo $Usage
	exit 1
fi
