/* 
 * ClientCppSession.cpp
 * 
 * Version 1.1
 * 
 * 28/10/2002
 *
 * Copyright (c) 2002 Oregon State Universtity. All rights reserved.
 */
 
#include <CFEngineCorbaC.h>
#include <ClientCppSession.h>
#include <fstream.h>
#include <iostream.h>

/**
 * The client session
 * Implement
 * 1. Rate an item.
 * 2. Get rating for an item.
 * 3. Get top n recommendation.
 * 4. Get top n recommendation of specific type.
 * 5. Get predicted rating for an item.
 *
 * @author  Yun Wang
 */

org::recommender::clients::ClientCppSession::ClientCppSession()
{
}

org::recommender::clients::ClientCppSession::~ClientCppSession()
{
}

org::recommender::clients::ClientCppSession::ClientCppSession(
	      org::recommender::server::corba::CFEngineCorba *p)
{
    engine = p;
    
}

void
org::recommender::clients::ClientCppSession::endSession()
{
    cout << endl ;
    cout << "Thanks for using the CF Prediction Engine..." << endl;
    cout << endl ;
    
    exit(0);
}


void 
org::recommender::clients::ClientCppSession::setRating() 
{
    
    CORBA::Long item;
    CORBA::Long rating = 0;
    
    do {
        cout << "Enter ItemID to rate: " << endl;
        cin >> item;
    } while (item <1);
    
    do {
        cout << "Enter a valid rating: " << endl;
        cin >> rating;
    } while (rating < 1 || rating > 5);
    
    try {
       engine -> addRating(userId, item, rating);
	} catch (org::recommender::server::corba::CFEngineCorba::CFIllegalParam & ex) {
		cerr << "Illegal Parameters: " << ex.msg << endl;  	
	} catch (const CORBA::Exception& ex) {
    
      cerr << "Uncaught CORBA exception: " << ex << endl;
      return;
    }
    
    cout << endl;
   
}

void 
org::recommender::clients::ClientCppSession::getRating()
{
     CORBA::Long item;
     org::recommender::server::corba::ItemRatingCorba_var p;
    
     do {
        cout << "Enter ItemID: " << endl;
        cin >> item;
    } while (item <1);
    
    try {
       p = engine -> getRating(userId, item);
    } catch (org::recommender::server::corba::CFEngineCorba::CFIllegalParam & ex) {
		cerr << "Illegal Parameters: " << ex.msg << endl;  	
	} catch (const CORBA::Exception& ex) {    
      cerr << "Uncaught CORBA exception: " << ex << endl;
      return;
    }
    
    cout << "The rating of Item ID " << item <<" is " << p->rating << endl;
    cout << endl;
}


void 
org::recommender::clients::ClientCppSession::getRecommendationsByType()
{
     CORBA::Long number;
	 CORBA::Long type;
     CORBA::Long offset = 0;
     
	 cout << "Please enter the type ID:" << endl;
     cin >> type;
     if (type < 0 )
		 return;
     
     cout << "Number of recommendations wanted:" << endl;
     cin >> number;
     
	 cout << "Starting from offset:" << endl;
     cin >> offset;
     
     if (number <= 0) {
	        return;
	    }

	 org::recommender::server::corba::seq_ItemPredictionCorba_var p;
     try {     
       p = engine -> getRecommendationsByType(userId, number, offset, type);
     } catch (org::recommender::server::corba::CFEngineCorba::CFIllegalParam & ex) {
		cerr << "Illegal Parameters: " << ex.msg << endl; 
	 } catch (org::recommender::server::corba::CFEngineCorba::CFEx& ex) {
		cerr << "The User Exception: " << ex.msg << endl;  	
	 } catch (const CORBA::Exception& ex) {    
      cerr << "Uncaught CORBA exception: " << ex << endl;
      return;
     }

	 if (p->length() == 0) 
		 cout << "No more recommendations starting from offset " << offset << endl;
	 else if (p->length() < number)
		 cout << "No more than " << p->length() 
		      << " recommendations starting from offset " << offset << endl;
	 else
		 cout << number << " recommendations starting from offset " << offset << endl;

	 cout << "      Item      Rating"  << endl;
	 cout << endl;
	  
     for (int i = 0; i < p->length() ; i++) {      
         org::recommender::server::corba::ItemPredictionCorba_var v = p[i];
         cout << i + 1 << "     " << v->itemID << "    " << v->prediction << endl;
     
     }
     cout << endl;
}

void 
org::recommender::clients::ClientCppSession::getRecommendations()
{
     CORBA::Long number;
     CORBA::Long offset = 0;
     
     
     cout << "Number of recommendations wanted:" << endl;
     cin >> number;
     
     cout << "Starting from offset:" << endl;
     cin >> offset;
     
     if (number <= 0) {
	        return;
	    }
     
     org::recommender::server::corba::seq_ItemPredictionCorba_var p;
     try {     
       p = engine -> getRecommendations(userId, number, offset);
     } catch (org::recommender::server::corba::CFEngineCorba::CFIllegalParam & ex) {
		cerr << "Illegal Parameters: " << ex.msg << endl; 
	 } catch (org::recommender::server::corba::CFEngineCorba::CFEx& ex) {
		cerr << "The User Exception: " << ex.msg << endl;  	
	 } catch (const CORBA::Exception& ex) {    
      cerr << "Uncaught CORBA exception: " << ex << endl;
      return;
     }    
     cout << "Item      Rating"  << endl;
             
     for (int i = 0; i < p->length() ; i++) {      
         org::recommender::server::corba::ItemPredictionCorba_var v = p[i];
         cout << v->itemID << "    " << v->prediction << endl;
     
     }
     cout << endl;
}
    

void 
org::recommender::clients::ClientCppSession::getPredictedRating()
{
     org::recommender::server::corba::ItemPredictionCorba_var  predict;
     int item;
     
     cout << "Enter ItemID to rate: " << endl;
     cin >> item;
     
     if (item < 1) {
        cout << "Not a valid item ID!" << endl;
        return;
     }
	 try {
		predict = engine->getPredictedRating(userId,item);
     } catch (org::recommender::server::corba::CFEngineCorba::CFIllegalParam & ex) {
		cerr << "Illegal Parameters: " << ex.msg << endl; 
	 } catch (org::recommender::server::corba::CFEngineCorba::CFEx& ex) {
		cerr << "The User Exception: " << ex.msg << endl;  	
	 } catch (const CORBA::Exception& ex) {    
      cerr << "Uncaught CORBA exception: " << ex << endl;
      return;
     }
     cout << " Prediction for itemID " << item << " is " << predict->prediction << endl;
     
     cout <<endl;
}

void
org::recommender::clients::ClientCppSession::getUserInput()
{
     do {
         cout << "Enter User ID: " << endl;
         cin >> userId;
     } while ( userId < 1 );
     
     cout << endl;
     cout << "Welcom, User " << userId <<"!" <<endl;
     
    while (1) {
    
        cout << "===================================" << endl;
        cout << "Please choose one of the following options" << endl;
        cout << "1. Rate an item" << endl;
        cout << "2. Get rating for an item" << endl;
        cout << "3. Get top n recommendations" << endl;
		cout << "4. Get top n recommendations by type" << endl;
        cout << "5. Get Predicted Rating for an item" << endl;
        cout << "6. Exit" << endl;
        cout << "===================================" << endl;
        
        int choice = 0;
        
        while (choice > 6 || choice < 1) {
               cout << "Enter choice: " << endl;
               cin >> choice;
         }
         switch (choice) {
                case 1:
                    setRating();
                    break;
                case 2:
                    getRating();
                    break;
                case 3:
                    getRecommendations();
                    break;
                case 4:
                    getRecommendationsByType();
                    break;
				case 5:
                    getPredictedRating();
                    break;                            
                case 6:
                    endSession();
                    break;
	        }//end of switch
          
    } //while
}


void 
org::recommender::clients::ClientCppSession::startSession() 
{   
    cout << "Welcome to the CF Prediction Engine" << endl;
    cout << "===================================" << endl;
    cout << "Please choose one option:" << endl;
    cout << "1. Login" << endl;
    cout << "2. Exit" << endl;
    
    int choice = 0;
    
    do {
         cout << "Enter choice: " << endl;
         cin >> choice;
     } while ( choice < 1 || choice > 2);
     
     if (choice == 1) {
         getUserInput();
     } else {
         endSession();
     }
     
}    
     
                              