############################################################
# config.sh
# Global shell configuration parameters for all shell scripts
############################################################

# This shell script depends on $CF_PATH being set

# Is this windows? If so, set to 1
WINDOWS=1

# Directory where the MySQL binaries are installed
MySqlBinDir=/usr/bin

############################################################
# SQL and DB Schema Configuration 
#  (Table names are case sensitive)
#  Note that this information is substituted in
#  lib/CFServer.properties
############################################################

SqlHost=localhost
SqlUser=cfengine
SqlPassword=cfengine
SqlDbName=cfengine

# Number of users to sample into the cache at startup time
# Users with the highest utility are selected.
maxSampledUsers=2000

# Number of other users (besides the ones sampled above)
# that should be kept in the cache after they visit
# the server for the first time.
maxCachedUsers=500

# Number of recommendations that should be computed when
# getRecommendations is called
numTopNRecords=200

############################################################
# Java Properties
############################################################

# Sets the Java maximum heap size 300m is 300MB
SERVER_MAX_HEAP=300m

# Location of your Java installation
#JAVA_HOME=/usr/java
#export JAVA_HOME

# The Java interpreter that you want to run the programs
JAVA=`which java`
export JAVA

# RMI nameserver
RmiNameServer=localhost

# Core developers: try and keep all third party libraries on a single line,
# so that we can strip them out when we distribute

# Place here classpath entries for third-party java libraries
THIRD_PARTY_LIBRARIES=

# OBJ is not a core element of the cfengine - just testing
if test "$OBJ_PATH" != "" ; then
	echo "**$OBJ_PATH**"
	THIRD_PARTY_LIBRARIES=$THIRD_PARTY_LIBRARIES:$OJB_PATH/lib/crimson.jar:$OJB_PATH/lib/jdbc2_0-stdext.jar:$OJB_PATH/lib/jta-spec1_0_1.jar:$OJB_PATH/lib/log4j.jar:
fi

# This is where the compiled Java classes are placed.
OUTPUT_ROOT_DIR=${CF_PATH}/obj

# Java classpath. 
CLASSPATH=${JAVA_HOME}/jre/lib/rt.jar:${OUTPUT_ROOT_DIR}:$THIRD_PARTY_LIBRARIES
export CLASSPATH

# Jikes compiles *way* faster than javac, so 
# we recommend it for development. 
# This is only relevant for development
JAVAC=javac

############################################################
# Probably don't need to change these
############################################################

# SQL DB for testing
TestDbName=testDB

# Is this for testing purpose? If so, set to 1
Test=0

# Set up some useful flags for the JVM
RMI_POLICY="$CF_PATH/lib/client.policy";

# Where the javadoc files are generated
DOCS_DIR=${CF_PATH}/Docs/api

# Name of relational table that generate the next ItemID and UserID
IdTable=ID_TABLE

# Name of relational table that holds the ratings
RatingTable=RATING_TABLE

# Name of relational table that holds the user info
UserInfoTable=USER_INFO

# For the sample console-based client - table that maps
# user ids to user names
UserTable=USER_TABLE

# For the sample console-based client - table that maps
# item ids to item names
ItemTable=ITEM_TABLE

# Name of relational table that holds the user info
TypeInfoTable=TYPE_INFO

# For the sample console-based client - table that maps
# item ids to to type ids
ItemTypeTable=ITEM_TYPE_TABLE

# For the sample console-based client - table the maps
# type names to to type ids
TypeTable=TYPE_TABLE
