/*
 * ItemToModalMapping.java
 * 
 * Version 1.0
 *
 * 29/03/2004
 * 
 * Copyright (c) 2004 Federal Universtity of Pernambuco. All rights reserved.
 */
package feers.recommender.algorithms.symbolic;

import feers.recommender.server.ItemContent;

/**
 * Defines how to add in a modal symbolic object of some user
 * profile the item encapsulated in this class.
 * 
 * @author <a href="mailto:bldb@cin.ufpe.br>Byron L. D. Bezerra</a>
 */
public class ItemToModalMapping {
	
	/*
	 * an item to be added in a modal symbolic object
	 */
	private ItemContent item;
	
	/*
	 * the key associated with a modal symbolic object in which
	 * the item must be added
	 */
	private ModalSymbolicObjectRelevance modalSymbolicObjectKey;
	
	/*
	 * the number of times the item must be added
	 */
	private int times;

	public ItemToModalMapping(ItemContent item, 
		ModalSymbolicObjectRelevance objectKey, int times) {
		this.item = item;
		this.modalSymbolicObjectKey = objectKey;
		this.times = times;
	}
	
	/**
	 * @return
	 */
	public ItemContent getItem() {
		return item;
	}

	/**
	 * @return
	 */
	public ModalSymbolicObjectRelevance getModalSymbolicObjectKey() {
		return modalSymbolicObjectKey;
	}

	/**
	 * @return
	 */
	public int getTimes() {
		return times;
	}

}
