/*
 * ModalSymbolicObjectRelevance.java
 * 
 * Version 1.0
 *
 * 29/03/2004
 * 
 * Copyright (c) 2004 Federal Universtity of Pernambuco. All rights reserved.
 */
package feers.recommender.algorithms.symbolic;

import java.io.Serializable;

/**
 * This class represents the relevance of a modal symbolic object for
 * a user profile. It defines the level of importance a modal symbolic 
 * object has for a user profile based on a weigh. Additionaly, it defines
 * the type of this importance, such as positive or negative.
 * 
 * @author <a href="mailto:bldb@cin.ufpe.br>Byron L. D. Bezerra</a>
 */
public class ModalSymbolicObjectRelevance implements Serializable {
	
	/*
	 * the level of importance a modal symbolic object has for a user profile
	 */
	private double weigh;
	
	/*
	 * the type of this importance, such as positive (+1) or negative (-1)
	 */
	private int polarity;
	
	private int hashCode;
	
	public ModalSymbolicObjectRelevance(double weigh, int polarity, int hashCode) {
		if (weigh >= 0)
			this.weigh = weigh;
		else 
			throw new IllegalArgumentException(
				"The weigh must be upper or equals to 0.");
		
		if (polarity == 1 || polarity == -1)
			this.polarity = polarity;
		else
			throw new IllegalArgumentException(
				"The polarity must be equals to 1 or -1.");
		
		this.hashCode = hashCode;
	}
	
	/**
	 * @return
	 */
	public int getPolarity() {
		return polarity;
	}

	/**
	 * @return
	 */
	public double getWeigh() {
		return weigh;
	}
	
	public boolean equals(Object other) {
		boolean r = false;
		if (other instanceof ModalSymbolicObjectRelevance) {
			ModalSymbolicObjectRelevance otherModal = (ModalSymbolicObjectRelevance) other;
			if (this.hashCode == otherModal.hashCode)
				r = true;
		}
		return r;
	}
	
	public int hashCode() {
		return hashCode;
	}
	
	public int compareTo(Object other) {
		ModalSymbolicObjectRelevance otherModal = (ModalSymbolicObjectRelevance) other;
		return ((this.hashCode < otherModal.hashCode) ? 
					-1 : ((this.hashCode > otherModal.hashCode) ? 1 : 0));
	}

}
