/*
 * TestaAlgoritimos.java
 * 
 * Version 1.0
 *
 * 06/04/2004
 * 
 * Copyright (c) 2004 Federal Universtity of Pernambuco. All rights reserved.
 */
package feers.recommender.algorithms.symbolic;

import java.sql.SQLException;

import org.recommender.algorithms.CFAlgorithm;
import org.recommender.algorithms.CFNotImplementedException;
import org.recommender.algorithms.SimplePearsonAlgorithm;
import org.recommender.server.DataManager;
import org.recommender.server.ItemPrediction;
import org.recommender.server.SystemProperties;

import feers.recommender.server.DataItemManager;

/**
 * Describe this classe here ...
 * 
 * @author <a href="mailto:bldb@cin.ufpe.br>Byron L. D. Bezerra</a>
 */
public class TestaAlgoritimos {

	public static void main(String[] args) {
		String installdir = System.getProperty("cf.path", ".");
		String filename = installdir + "/lib/" + "CFServer.properties";
		// Load properties
		SystemProperties.loadProperties(filename);
	
		DataManager dataMgr = new DataManager(true, true);
		DataItemManager dataItemMgr =
			new DataItemManager(1, dataMgr.getMaxItem());
	
		CFAlgorithm algorithm1 = new SimplePearsonAlgorithm(dataMgr);
		CFAlgorithm algorithm2 =
			new ModalSymbolicAlgorithm(dataMgr, dataItemMgr);
		CFAlgorithm algorithm3 = new CollabModalAlgorithm(dataMgr, dataItemMgr);
	
		try {
			long startTime, endTime;
			ItemPrediction[] predictions;
			
			System.out.print("\n" +
				"Best predictions for user 50 using " + algorithm1);
			startTime = System.currentTimeMillis();
			predictions =
				algorithm1.getRecommendations(50, 10);
			endTime = System.currentTimeMillis();
			System.out.println("First Time = " + (endTime - startTime) + " ms");
			startTime = System.currentTimeMillis();
			predictions =
				algorithm1.getRecommendations(50, 10);
			endTime = System.currentTimeMillis();
			System.out.println("Second Time = " + (endTime - startTime) + " ms");
			for (int i = 0; i < predictions.length; i++) {
				System.out.println(
					"Item "
						+ predictions[i].getItemID()
						+ " - grade "
						+ predictions[i].getPrediction());
			}
	
			System.out.print("\n" +
				"Best predictions for user 50 using " + algorithm2);
			startTime = System.currentTimeMillis();
			predictions = algorithm2.getRecommendations(50, 10);
			endTime = System.currentTimeMillis();
			System.out.println("First Time = " + (endTime - startTime) + " ms");
			startTime = System.currentTimeMillis();
			predictions = algorithm2.getRecommendations(50, 10);
			endTime = System.currentTimeMillis();
			System.out.println("Second Time = " + (endTime - startTime) + " ms");
			for (int i = 0; i < predictions.length; i++) {
				System.out.println(
					"Item "
						+ predictions[i].getItemID()
						+ " - grade "
						+ predictions[i].getPrediction());
			}
			
			System.out.print("\n" +
				"Best predictions for user 50 using " + algorithm3);
			startTime = System.currentTimeMillis();
			predictions = algorithm3.getRecommendations(50, 10);
			endTime = System.currentTimeMillis();
			System.out.println("First Time = " + (endTime - startTime) + " ms");
			startTime = System.currentTimeMillis();
			predictions = algorithm3.getRecommendations(50, 10);
			endTime = System.currentTimeMillis();
			System.out.println("Second Time = " + (endTime - startTime) + " ms");
			for (int i = 0; i < predictions.length; i++) {
				System.out.println(
					"Item "
						+ predictions[i].getItemID()
						+ " - grade "
						+ predictions[i].getPrediction());
			}
			
		} catch (CFNotImplementedException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		System.exit(0);
	}

}
