/*
 * ItemContent.java
 * 
 * Version 1.0
 *
 * 28/03/2004
 * 
 * Copyright (c) 2004 Federal Universtity of Pernambuco. All rights reserved.
 */
package feers.recommender.server;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import feers.recommender.server.descriptor.Descriptor;

/**
 * This class encapsulates the content of an item.
 * 
 * @author <a href="mailto:bldb@cin.ufpe.br>Byron L. D. Bezerra</a>
 */
public class ItemContent {
	
	private int itemId;
	
	/*
	 * Maintain a list of variables of some item. Each variable is a type
	 * of Descriptor.
	 */
	private List variables;
	
	public ItemContent(int itemId) {
		this.itemId = itemId;
		this.variables = new LinkedList();
	}
	
	public void addVariable(Descriptor d) {
		if (d != null)
			if (! variables.contains(d))
				variables.add(d);
			else 
				throw new IllegalArgumentException(
					"The variable " + d.getName() + " already exists in item " + itemId);
		else
			throw new IllegalArgumentException(
				"The variable argument in addVariable cannot be null.");
	}
	
	public Iterator variables() {
		return variables.iterator();
	}
	
	public String toString() {
		StringBuffer res = new StringBuffer("------------ Item ");
		res.append(itemId);
		res.append(" ------------\n");
		for (Iterator iter = variables.iterator(); iter.hasNext();) {
			Descriptor element = (Descriptor) iter.next();
			res.append(element.toString());
			res.append("\n");			
		}
		return res.toString();
	}

	/**
	 * @return
	 */
	public int getItemId() {
		return itemId;
	}

}
