/*
 * Descriptor.java
 * 
 * Version 1.0
 *
 * 28/03/2004
 * 
 * Copyright (c) 2004 Federal Universtity of Pernambuco. All rights reserved.
 */
package feers.recommender.server.descriptor;

import java.io.Serializable;

/**
 * This abstract class represents a variable of some item. 
 * The possible types are: numeric, interval, cathegorical univalued, 
 * cathegorical multivalued, textual. Every variable type
 * must have a subclass of Descriptor class.
 * 
 * @author <a href="mailto:bldb@cin.ufpe.br>Byron L. D. Bezerra</a>
 */
public abstract class Descriptor implements Serializable {
	
	private String name;
	
	public Descriptor(String name) {
		this.name = name;
	}
	
	public String getName() {
		return this.name;
	}
	
	public abstract Object getValue();
	
	public boolean equals(Object o) {
		boolean res = false;
		if (o instanceof Descriptor)
			if (((Descriptor) o).getName().equals(this.getName()))
				res = true;
		return res;				
	}
	
	/**
	 * Este mtodo retorna uma String contendo o nome que est 
	 * associado a essa varivel seguido ": ". As subclasses de
	 * Descriptor devem sobrescrever este mtodo para concatenar
	 * alm do nome dessa varivel, os valores armazenados.
	 * 
	 * @return String 
	 */
	public String toString() {
		String res = Character.toUpperCase(name.charAt(0)) 
			+ name.substring(1).toLowerCase() + ": ";
		return res;
	}

}
