/*
 * IntervalDescriptor.java
 * 
 * Version 1.0
 *
 * 28/03/2004
 * 
 * Copyright (c) 2004 Federal Universtity of Pernambuco. All rights reserved.
 */
package feers.recommender.server.descriptor;

import org.recommender.algorithms.Range;

/**
 * This class represents a interval variable.
 * 
 * @author <a href="mailto:bldb@cin.ufpe.br>Byron L. D. Bezerra</a>
 */
public class IntervalDescriptor extends Descriptor {

	private Range range;
	
	/**
	 * @param name
	 * @param startLimit
	 * @param endLimit
	 */
	public IntervalDescriptor(String name, float startLimit, float endLimit) {
		super(name);
		if (startLimit > endLimit)
			throw new IllegalArgumentException(
				"The startLimit must be lower or equal than endLimit in "
				+ this.getName() + " variable.");
		this.range = new Range(startLimit, endLimit);
	}

	/**
	 * Returns the value of this interval variable in a Range object.
	 * 
	 * @see feers.recommender.server.Descriptor#getValue()
	 * @see org.recommender.algorithms.Range
	 * @return Object
	 */
	public Object getValue() {
		return range;
	}
	
	public String toString() {
		return super.toString() + this.range;
	}

}
