/*
 * UserSets.java
 * 
 * Version 1.0
 *
 * 08/04/2004
 * 
 * Copyright (c) 2004 Federal Universtity of Pernambuco. All rights reserved.
 */
package feers.recommender.test;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;

/**
 * This class stores information about evaluations of a user divided 
 * in two sets: TrainingSet and TestSet. It is used in the experiments.
 * 
 * @author <a href="mailto:bldb@cin.ufpe.br>Byron L. D. Bezerra</a>
 */
public class UserSets implements Serializable {

	private int userId;

	/*
	 * This vector defines the training and the test items
	 * of one experiment.  
	 */
	private int[] trainingItems;
	private int[] testItems;

	private UserSets(int userId) {
		this.userId = userId;
	}
	
	public UserSets(int userId, int[] trainingItems, int[] testItems) {
		this.userId = userId;
		this.testItems = testItems;
		this.trainingItems = trainingItems;
	}

	/**
	 * @return
	 */
	public int[] getTestItems() {
		return testItems;
	}

	/**
	 * @return
	 */
	public int[] getTrainingItems() {
		return trainingItems;
	}

	/**
	 * @return
	 */
	public int getUserId() {
		return userId;
	}

	public static UserSets generateUserSets(
		int userId,
		int numberOfItemsInTrainingSet,
		int numberOfItemsForTestingEachUser, List originalItemsEvaluated) {
		UserSets userSets = new UserSets(userId);
		int[] trainingSet = new int[numberOfItemsInTrainingSet];
		int[] testSet = new int[numberOfItemsForTestingEachUser];
		// int[] testSet = new int[originalItemsEvaluated.size()-numberOfItemsInTrainingSet];
		Collections.shuffle(originalItemsEvaluated);
		for (int j = 0; j < trainingSet.length; j++) {
			trainingSet[j] =
				((Integer) originalItemsEvaluated.get(j)).intValue();
		}
		for (int j = 0; j < testSet.length; j++) {
			testSet[j] =
				((Integer) originalItemsEvaluated.get(j + trainingSet.length))
					.intValue();
		}
		userSets.trainingItems = trainingSet;
		userSets.testItems = testSet;
		return userSets;
	}
}
