/*
 * StringProcessing.java
 * 
 * Version 1.0
 *
 * 28/03/2004
 * 
 * Copyright (c) 2004 Federal Universtity of Pernambuco. All rights reserved.
 */
package feers.recommender.util;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;

/**
 * Provides some tools for String processing
 * 
 * @author <a href="mailto:bldb@cin.ufpe.br>Byron L. D. Bezerra</a>
 */
public class StringProcessing {

	//////////// Parameters ///////////////
	private transient static final String STOPWORDS_RESOURCE =
		"lib/stoplist.txt";
	private transient static Set STOPWORDS;
	private transient static final String STOPCHAR =
		" ,.!?:;-_\n\r\t\f=+*%@#(){}[]'`^~\"\\<>&|/";

	static {
		loadStopwords();
	}

	public static Set loadStopwords() {

		if ((STOPWORDS == null) || (STOPWORDS.size() == 0)) {
			STOPWORDS = new HashSet();
			InputStream inputStreamWords;
			try {
				inputStreamWords =
					new FileInputStream(StringProcessing.STOPWORDS_RESOURCE);

				BufferedReader words =
					new BufferedReader(new InputStreamReader(inputStreamWords));
				String currentWord = null;
				try {
					while ((currentWord = words.readLine()) != null) {
						String trimmedWord = currentWord.trim();
						if (trimmedWord.length() > 0) {
							STOPWORDS.add(trimmedWord);
							// as palavras esto em lowercase.
						}
					}
				} catch (IOException ex) {
					ex.printStackTrace();
					//XXX fazer algo melhor...
				}
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}

		return STOPWORDS;
	}

	/**
	 * Builds the centroid representation of this text removing the 
	 * words in a stoplist.
	 * 
	 * @param text
	 * @return HashMap the table of word and weight
	 */
	public static HashMap getCentroid(String text) {
		HashMap centroid = new HashMap();
		int numberOfWords = 0;

		StringTokenizer tokenizer = new StringTokenizer(text, STOPCHAR);
		while (tokenizer.hasMoreTokens()) {
			String word = tokenizer.nextToken();
			if (!STOPWORDS.contains(word)) {
				Double pesoAtual = (Double) centroid.get(word);
				Double newPeso = null;
				if (pesoAtual == null) {
					newPeso = new Double(1.0);
				} else {
					newPeso = new Double(pesoAtual.doubleValue() + 1.0);
				}
				centroid.put(word, newPeso);
				numberOfWords++;
			}
		}

		// normalizando
		for (Iterator iter = centroid.keySet().iterator(); iter.hasNext();) {
			String word = (String) iter.next();
			Double value = (Double) centroid.get(word);
			value = new Double((value.doubleValue() / numberOfWords));
			centroid.put(word, value);
		}

		return centroid;
	}

	public static void main(String[] args) {
		System.out.println(STOPWORDS);
	}
}
