/*
 * CFAlgorithm.java
 *
 * Version 1.1
 *
 * 15/08/2002
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */
package org.recommender.algorithms;

import java.sql.*;
import org.recommender.server.*;

/**
 * Interface for a collaborative filtering algorithm.
 *
 * @author Sameer Kadam
 * @author  <A HREF="mailto:mclaughm@cs.orst.edu">Matt McLaughlin</A>
 */
public interface CFAlgorithm {

  /**
   * Asks the prediction engine to return a prediction for how a
   * particular user will rate a particular item.
   *
   * Note: Predicted Rating must be in the range designated by the dataset.
   *		 Use DataManger's getMaxRating() and getMinRating to verify this.
   *
   * @param userID
   * @param itemID
   * @return predictedRating
   */
  public ItemPrediction predictRating(int userID, int itemID) throws
      CFNotImplementedException;

  /**
   * Return top n recommendations for the active user.
   *
   * Note: Returned array contains only recommendations for non-rated items.
   * All recommendations must be valid.  If there is an insufficient
   * number of recommendations to fulfill the requested amount, return
   * a filled array of size equal to the number of valid recommendations.
   * All recommendations' predicted values must comply to same
   * guidelines set forth by predictRating() in CFAlgorithm.
   *
   * @param activeUser
   * @param n
   * @return itemPredList
   * @throws SQLException
   */
  public ItemPrediction[] getRecommendations(int activeUser, int n) throws
      SQLException, CFNotImplementedException;

  /**
   * Return the name of algorithm.
   * @return String
   */
  public String toString();

  /**
   * Very useful to get the prediction time after a bunch of experiment
   * Should be implemented within each algorithm class.
   * @return average prediction time
   */
  public long getAveragePredictionTime() throws CFNotImplementedException;

  /**
   * Notify algorithm that the given user's ratings have changed.
   * @param userID
   */
  public void updateUser(int userID) throws CFNotImplementedException;

  /**
   * Added by Yun Wang
   * Return top n recommendations for the active user of specific type.
   *
   * Note: Returned array contains only recommendations for non-rated items.
   * All recommendations must be valid.  If there is an insufficient
   * number of recommendations to fulfill the requested amount, return
   * a filled array of size equal to the number of valid recommendations.
   * All recommendations' predicted values must comply to same
   * guidelines set forth by predictRating() in CFAlgorithm.
   *
   * @param activeUser
   * @param n
   * @param type
   * @return itemPredList
   * @throws SQLException
   */
  public ItemPrediction[] getRecommendationsByType(int activeUser, int n,
      int type) throws SQLException, CFNotImplementedException;
      
  public Stats getTimeStats();
}
