package org.recommender.algorithms.Experimental;

import org.recommender.server.*;
import java.util.*;

/**
 * Represents a tag for a binGuassian probability distribution, and 
 * its contents. 
 *
 * @author Daniel Lowd <dlowd@cs.orst.edu>
 */
public class BinGaussian
{
    float mean;
    float sd;
    float probMissing;
    float denominator;

    // Cache the last result
    float lastX;
    float lastProb;

    /** 
     * Construct an empty binGuassian.
     */
    public BinGaussian(float mean, float sd, float probMissing)
    {
        this.mean = mean;
        this.sd = sd;
        this.probMissing = probMissing;

        lastX = Float.NaN;
        lastProb = probMissing;

        denominator = sd * (float) Math.sqrt(2*Math.PI);
    }

    public float getMean() { return mean; }
    public float getSD()   { return sd; }

    /* Returns the probability density function of this probability
     * distribution evaluated at the target value, or the probability
     * that the variable is missing.  These return values are somewhat
     * different, but for the operations we're doing (Markov-blanket
     * inference), it shouldn't matter.  We only care about relative
     * probabilities.
     */
    public float getProb(float x)
    { 
        if (x == lastX)
            return lastProb;
        else 
        {
            if (Float.isNaN(x))
                lastProb = probMissing;
            else
                lastProb = (float) Math.exp(-(x-mean)*(x-mean)/(sd*sd))/denominator;

            lastX = x;
            return lastProb;
        }
    }

    /**
     * Generate an XML representation of this binGaussian distribution,
     * with each line preceded by the specified number of tabs.
     */
    public String toString(int numIndents)
    {
        String tabs = "";
        while (numIndents-- > 0)
            tabs += '\t';

        String ret = "";
        ret += tabs + "<Leaf>\n";
        ret += tabs + "\t<BinGaussian>\n";
        ret += tabs + "\t\t<Mean>" + mean + "</Mean>\n";
        ret += tabs + "\t\t<SD>" + sd + "</SD>\n";
        ret += tabs + "\t\t<ProbMissing>" + probMissing + "</ProbMissing>\n";
        ret += tabs + "\t</BinGaussian>\n";
        ret += tabs + "</Leaf>\n";
        return ret;
    }

    /**
     * Generate an XML representation of this binGaussian distribution,
     * left-justified.
     */
    public String toString()
    {
        return toString(0);
    }
}
