/* Represents a numerical range (or a missing value) for splits in decision
 * trees.
 *
 * @author Daniel Lowd &lt;dlowd@cs.orst.edu&gt;
 */

package org.recommender.algorithms.Experimental;

public class Range
{
    float min;
    float max;
    boolean missing;

    public Range(float min, float max)
    {
        this.min = min;
        this.max = max;
        missing = false;
    }

    public Range(boolean missing)
    {
        this.missing = missing;
        this.min = Float.POSITIVE_INFINITY;
        this.max = Float.POSITIVE_INFINITY;
    }

    /* Returns true if this value is in the range specified.
     */
    public boolean inRange(float val)
    {
        if (missing)
            return Float.isNaN(val);
        else
            return min <= val && val < max;
    }

    public String toString()
    {
        if (missing)
            return "Missing";
        else
            return "[" + min + ", " + max + ")";
    }
}
