/**
 * Parse numerical ranges for use in decision trees.
 *
 * @author Daniel Lowd &lt;dlowd@cs.orst.edu&gt;
 */

package org.recommender.algorithms.Experimental;

public class RangeTokenizer
{
    int pos;
    String input;
    boolean moreTokens;

    public RangeTokenizer(String input)
    {
        this.input = input;
        pos = 0;
        moreTokens = true;

        // Check to see if there are more tokens or not.
        // (Will update moreTokens if there are not.)
        hasMoreTokens();
    }

    public Range nextToken()
    {
        // Skip past whitespace
        while(Character.isSpace(input.charAt(pos)))
            pos++;

        if (input.startsWith("Missing", pos)) 
        {
            pos += 7; // strlen("Missing") = 7
            return new Range(true);
        }
        else if (input.charAt(pos) == '(' || input.charAt(pos) == '[')
        {
            // Find the middle of this range
            int midpos = input.indexOf(',', pos);

            // Find the end of this range
            int endpos;
            int parenpos = input.indexOf(')', midpos);
            int bracketpos = input.indexOf(']', midpos);

            if (parenpos != -1 && (bracketpos == -1 || parenpos < bracketpos))
            {
                endpos = parenpos;
            }
            else if (bracketpos != -1)
            {
                endpos = bracketpos;
            }
            else
            {
                moreTokens = false;
                return new Range(false);
            }

            // Get the min and max
            float min = convertToFloat(input.substring(pos+1, midpos));
            float max = convertToFloat(input.substring(midpos + 1, endpos));

            pos = endpos + 1;
        
            return new Range(min, max);
        }
        else
        {
            // ...ERROR...
            moreTokens = false;
            return new Range(false);
        }
    }

    float convertToFloat(String str)
    {
        String trimmed = str.trim();
        float ret;

        if (trimmed.equalsIgnoreCase("-Infinity"))
        {
            ret = Float.NEGATIVE_INFINITY;
        }
        else if (trimmed.equalsIgnoreCase("Infinity"))
        {
            ret = Float.POSITIVE_INFINITY;
        }
        else
        {
            try {
                ret = Float.parseFloat(trimmed);
            } catch (NumberFormatException e) {
                moreTokens = false;
                ret = Float.NaN;

                // DEBUG
                System.out.println("Error converting \"" + trimmed +
                        "\" in RangeTokenizer.convertToFloat()");
            }
        }

        return ret;
    }

    public boolean hasMoreTokens()
    {
        // Skip past whitespace
        while(pos < input.length() && Character.isSpace(input.charAt(pos)))
            pos++;

        if (pos >= input.length())
            moreTokens = false;

        return moreTokens;
    }
}
