/*
 * SVD_Pearson.java
 *
 * Version 1.1
 *
 * 3/15/2003
 *
 * Copyright (c) 2003 Oregon State University. All rights reserved.
 */

package org.recommender.algorithms.Experimental;
import org.recommender.algorithms.*;
import org.recommender.server.*;

import java.util.zip.*;
import java.sql.*;

import java.io.*;
import java.util.*;
import java.lang.Math.*;
import VisualNumerics.math.*;
import Jama.*;

/**
 * Describes an implementation of the CFAlgorithm interface
 * to make CF predictions using the SVD + Pearson correlation method.
 * This is just an experimentally testing algorithm, and only the predictRating
 * method is implemented.
 * Some parameters are hard-coded. If you want to test this algorithm yourself,
 * please change the parameters accordingly.
 * Default paramters:
 * number of users: 943
 * number of items: 1682
 * number of training ratings: 80000
 * number of test ratings: 20000
 *
 * @author  <A HREF="mailto:nong@cs.orst.edu">Shuzhen Nong</A>
 *
 */

public class SVD_Pearson implements CFAlgorithm
{



     private DataManager dataMgr;


    private long Time = 0;
    private long NumPredictions = 0;

    Vector SimilarityVector;
    double[][] A_original;
    double[] avg_row;

	/**
	 * Constructor, simply assigns DataManager.
	 * @param mgr
	 */
    public SVD_Pearson(DataManager mgr)
    {
		dataMgr = mgr;

		SVD_PearsonOperation();
    }

   /**
    * Not implemented
    */
   public ItemPrediction[] getRecommendationsByType(int activeUser, int n, int type)
               throws SQLException, CFNotImplementedException
   {
   		throw new CFNotImplementedException("getRecommendationsByType not implemented yet");
        // return null;

   }

   /**
    * Not implemented
    */
   public ItemPrediction[] getRecommendations(int activeUser, int n)
            throws SQLException,CFNotImplementedException
    {
    		throw new CFNotImplementedException("getRecommendations not implemented yet");
      //  return null;
    }

    /**
     * Predict ratings using nearest neighbor algorithm -- user correlations
     * are based on the Pearson correlations between users' ratings on "features"
     */
     public ItemPrediction predictRating (int activeUser, int item) throws CFNotImplementedException
    {
         int i, j, k;
         double numerator;
         double denominator;
         double n_neighbors;
         double similarity;
         int neighborID;

         numerator=0;
         denominator=0;
         n_neighbors=0;


         for(k=0;k<943;k++)
         {

              if(n_neighbors <= 30)
              {

                   similarity = (((SimilarityWeight)(((LinkedList)(SimilarityVector.elementAt(activeUser-1))).get(k))).getWeight());
                   neighborID = (((SimilarityWeight)(((LinkedList)(SimilarityVector.elementAt(activeUser-1))).get(k))).getNeighborID());

                   if(A_original[neighborID][item-1]>0)
                   {
                        ++n_neighbors;
                        numerator += similarity*(A_original[neighborID][item-1]-avg_row[neighborID]);
                        denominator += Math.abs(similarity);
                   }
               }
         }

         if(denominator > 0)
               return  new ItemPrediction(activeUser,item,(float)(avg_row[activeUser-1]+numerator/denominator)) ;
         else
               return  new ItemPrediction(activeUser,item,dataMgr.getErrRating());



    }

    /**
     * Not implemented
     */
     public void updateUser(int userID) throws CFNotImplementedException
    {
        	throw new CFNotImplementedException("updateUser not implemented yet");
    }


    /**
     * Not implemented
     */
    public long getAveragePredictionTime() throws CFNotImplementedException
    {
    		throw new CFNotImplementedException("getAveragePredictionTime not implemented yet");
       // return 0;
    }


    /**
     * return the name of this algorithm
     */
    public String toString()
    {
        return "SVD_Pearson.java";
    }

    /**
     * Compute the Pearson correlations based on users' ratings on "features"
     */
    public void SVD_PearsonOperation () {



        int DIMENSION = 14;
        int NUM_TRAINING = 80000;
        int NUM_TEST = 20000;

        int i,j,total,k, neighborID,n_neighbors;
        double similarity;
        double numerator, denominator;
        double n_rated, total_error, MAE;
        double avg_col=0;



        double[][] A = new double[944][1683];
        double[][] Acopy = new double[943][1682];



        double[][] U = new double[943][943];
        double[][] V = new double[1682][1682];
        double[] S = new double[943];

        double[][] Sk = new double[DIMENSION][DIMENSION];
        double[][] Uk = new double[943][DIMENSION];
        double[][] Vk = new double[1682][DIMENSION];

        double[][] V_transpose = new double[1682][1682];
        double[][] Vk_transpose = new double[DIMENSION][1682];

        double[][] result = new double[943][1682];

        //////// for computing cosine of two users /////
        double[] NormOfUser = new double[943];
        double[] AvgUser = new double[943];
        double[][] CosineOfUser = new double[943][943];

        double[][] SVDresult = new double[943][1682];


        int[] numRatings = new int[943];
        int num=0;



        SimilarityVector = new Vector(943);
        A_original=new double[943][1682];
        avg_row= new double[943];



        for(i=0;i<944;i++)
           for(j=0;j<1683;j++)
              A[i][j]=0;


      /* Move training set into a 943*1682 matrix */

          for(i=1;i<944;i++)
            for(j=1;j<1683;j++)
            {
               if(dataMgr.getRatingCached(i,j) != dataMgr.getErrRating())
                     A[i][j]=dataMgr.getRatingCached(i,j);
               else
                     A[i][j]=0;
            }


          for(i=1;i<944;i++)
          {
            for(j=1;j<1683;j++)
            {
              Acopy[i-1][j-1]=A[i][j];
            }
          }

          for(i=0;i<943;i++)
          {
            for(j=0;j<1682;j++)
            {
              A_original[i][j]=Acopy[i][j];
            }
          }



       /* For Rnorm: 1. filling missing rating cells with avg_col;

       */

       /* avg_col = sum of cells which are not empty over the column
                       / # of cells which are not empty.
       */
         for(j=0;j<1682;j++)
          {
              avg_col=0;
              total=0;

              for(i=0;i<943;i++)
              {
                 if(Acopy[i][j] > 0)
                 {
                    avg_col += Acopy[i][j];
                    ++total;
                 }
              }

               if(total > 0)
                  avg_col = avg_col/total;
               else
                  avg_col = 0;

             //  System.out.println(" i=" + i +"; j=" +j);

               for(i=0;i<943;i++)
                   if(Acopy[i][j] <= 0)
                      Acopy[i][j]=avg_col;

           }


          for(i=0;i<943;i++)
          {
              avg_row[i] = 0;
              total=0;

              for(j=0;j<1682;j++)
              {
                if(A_original[i][j] > 0)
                {
                   avg_row[i] += A_original[i][j];
                   ++total;
                }
              }

              if(total>0)
                avg_row[i] = avg_row[i]/total;
              else
                avg_row[i] = 0;


           }

           for(i=0;i<943;i++)
           {
              num=0;
              for(j=0;j<1682;j++)
              {
                 if(A_original[i][j]>0)
                    ++num;
              }

              numRatings[i]=num;
           }







        /* Begin SVD decomposition */

        DoubleSVD SVD = new DoubleSVD(Acopy);
          U = SVD.U();
          V = SVD.V();
          S = SVD.S();



         for(i=0;i<DIMENSION;i++)
             for(j=0;j<DIMENSION;j++)
             {
                 if(i==j)
                    Sk[i][j]=S[i];
                 else
                    Sk[i][j]=0;
             }


         for(i=0;i<943;i++)
           for(j=0;j<DIMENSION;j++)
           {
               Uk[i][j]=U[i][j];
           }

         Matrix VMatrix = new Matrix(V);
         V_transpose = (VMatrix.transpose()).getArray();

         for(i=0;i<DIMENSION;i++)
           for(j=0;j<1682;j++)
           {
              Vk_transpose[i][j] = V_transpose[i][j];
           }


         Matrix UkMatrix = new Matrix(Uk);
         Matrix SkMatrix = new Matrix(Sk);
         Matrix Vk_transposeMatrix = new Matrix(Vk_transpose);


         Matrix resultMatrix = new Matrix(943,DIMENSION);

         Matrix SVDresultMatrix = new Matrix(943,1682);
         SVDresultMatrix = (UkMatrix.times(SkMatrix)).times(Vk_transposeMatrix);
         SVDresult = SVDresultMatrix.getArray();


        Matrix AcopyMatrix = new Matrix(Acopy);
        resultMatrix=AcopyMatrix.times(Vk_transposeMatrix.transpose());

         result = resultMatrix.getArray();



       /* compute the average rating for each user in the result array */
       for(i=0;i<943;i++)
           AvgUser[i]=0;

        for(i=0;i<943;i++)
        {
           for(j=0;j<DIMENSION;j++)
                AvgUser[i] += result[i][j];

           AvgUser[i] = AvgUser[i]/DIMENSION;
        }


       /* compute norm of users */
       for(i=0;i<943;i++)
           NormOfUser[i]=0;

        for(i=0;i<943;i++)
        {
            for(j=0;j<DIMENSION;j++)
                NormOfUser[i] += (result[i][j]-AvgUser[i])*(result[i][j]-AvgUser[i]);

            NormOfUser[i] = Math.sqrt(NormOfUser[i]);

        }





         /* compute Pearson Correlation of two users */
          for(i=0;i<943;i++)
          {
             for(j=0;j<DIMENSION;j++)
                result[i][j] -= AvgUser[i];
          }

          resultMatrix = new Matrix(result);

          Matrix CosineMatrix = new Matrix(943,943);
          CosineMatrix = resultMatrix.times(resultMatrix.transpose());
          CosineOfUser = CosineMatrix.getArray();




         for(i=0;i<943;i++)
            for(j=0;j<943;j++)
            {
               CosineOfUser[i][j] = CosineOfUser[i][j]/(NormOfUser[i]*NormOfUser[j]);

            }


         for(i=0;i<943;i++)
         {

             LinkedList NeighborWeights = new LinkedList();

             for(j=0;j<943;j++)
             {
                NeighborWeights.add(j, new SimilarityWeight(j, (float)(CosineOfUser[i][j])));
             }


             Collections.sort(NeighborWeights, Collections.reverseOrder());
             SimilarityVector.add(i,NeighborWeights);
          }






    }



}