/*
 * SimpleSVD.java
 *
 * Version 1.1
 *
 * 3/15/2003
 *
 * Copyright (c) 2003 Oregon State University. All rights reserved.
 */

package org.recommender.algorithms.Experimental;
import org.recommender.algorithms.*;
import org.recommender.server.*;

import java.util.zip.*;
import java.sql.*;

import java.io.*;
import java.util.*;
import java.lang.Math.*;
import VisualNumerics.math.*;
import Jama.*;


/**
 * Describes an implementation of the CFAlgorithm interface
 * to make CF predictions using the Simple SVD method.
 * This is just an experimentally testing algorithm, and only the predictRating
 * method is implemented.
 * Some parameters are hard-coded. If you want to test this algorithm yourself,
 * please change the parameters accordingly.
 * Default paramters:
 * number of users: 943
 * number of items: 1682
 * number of training ratings: 80000
 * number of test ratings: 20000
 *
 * @author  <A HREF="mailto:nong@cs.orst.edu">Shuzhen Nong</A>
 *
 */

public class SimpleSVD implements CFAlgorithm
{


    private DataManager dataMgr;


    private long Time = 0;
    private long NumPredictions = 0;

    double[][] result;

	/**
	 * Constructor, simply assigns DataManager.
	 * @param mgr
	 */
    public SimpleSVD(DataManager mgr)
    {
		dataMgr = mgr;

		SimpleSVDOperation();
    }

   /**
    * Not implemented
    */
   public ItemPrediction[] getRecommendationsByType(int activeUser, int n, int type)
               throws SQLException, CFNotImplementedException
   {
   		throw new CFNotImplementedException("getRecommendationsByType not implemented yet");
        // return null;

   }

   /**
    * Not implemented
    */
   public ItemPrediction[] getRecommendations(int activeUser, int n)
            throws SQLException, CFNotImplementedException
    {
    		throw new CFNotImplementedException("getRecommendations not implemented yet");
       // return null;
    }


    /**
     * Simply return the rating in the SVD-reduced matrix
     */
     public ItemPrediction predictRating (int activeUser, int item) throws CFNotImplementedException
    {

         return new ItemPrediction(activeUser, item, (float)result[activeUser-1][item-1]);
    }


    /**
     * Not implemented
     */
     public void updateUser(int userID) throws CFNotImplementedException
    {
        	throw new CFNotImplementedException("updateUser not implemented yet");
    }


    /**
     * Not implemented
     */
    public long getAveragePredictionTime() throws CFNotImplementedException
    {
    		throw new CFNotImplementedException("getAveragePredictionTime not implemented yet");
        //return 0;
    }


    /**
     * return the name of this algorithm
     */
    public String toString()
    {
        return "SimpleSVD.java";
    }

    /**
     * Construct the SVD-reduced matrix
     */
    public void SimpleSVDOperation() {


        int DIMENSION = 14;
        int NUM_TRAINING = 80000;
        int NUM_TEST = 20000;
        int i,j,total;
        double n_rated, total_error, MAE;
        double avg_col=0;
        double[] avg_row= new double[943];


        double[][] A = new double[944][1683];
        double[][] Acopy = new double[943][1682];
        double[][] A_original = new double[943][1682];



        double[][] TestSet = new double[NUM_TEST][4];
        double[][] BigTestMatrix = new double[944][1683];
        double[][] CorrectTestMatrix = new double[943][1682];

        double[][] U = new double[943][943];
        double[][] V = new double[1682][1682];
        double[] S = new double[943];

        double[][] Sk = new double[DIMENSION][DIMENSION];
        double[][] Uk = new double[943][DIMENSION];
        double[][] Vk = new double[1682][DIMENSION];

        double[][] V_transpose = new double[1682][1682];
        double[][] Vk_transpose = new double[DIMENSION][1682];



        double[][] SkHalf = new double[DIMENSION][DIMENSION];


        result = new double[943][1682];

        for(i=0;i<944;i++)
           for(j=0;j<1683;j++)
              A[i][j]=0;



        /* read in user-item rating table into array A[][] */
          for(i=1;i<944;i++)
            for(j=1;j<1683;j++)
            {
               if(dataMgr.getRatingCached(i,j) != dataMgr.getErrRating())
                     A[i][j]=dataMgr.getRatingCached(i,j);
               else
                     A[i][j]=0;
            }



          for(i=1;i<944;i++)
          {
            for(j=1;j<1683;j++)
            {
              Acopy[i-1][j-1]=A[i][j];
            }
          }

          for(i=0;i<943;i++)
          {
            for(j=0;j<1682;j++)
            {
              A_original[i][j]=Acopy[i][j];
            }
          }




       /* For Rnorm: 1. filling missing rating cells with avg_col;
                      2. subtract avg_row from each rating
       */

       /* avg_col = sum of cells which are not empty over the column
                       / # of cells which are not empty.
       */
         for(j=0;j<1682;j++)
          {
              avg_col=0;
              total=0;

              for(i=0;i<943;i++)
              {
                 if(Acopy[i][j] > 0)
                 {
                    avg_col += Acopy[i][j];
                    ++total;
                 }
              }

               if(total > 0)
                  avg_col = avg_col/total;
               else
                  avg_col = 0;

             //  System.out.println(" i=" + i +"; j=" +j);

               for(i=0;i<943;i++)
                   if(Acopy[i][j] <= 0)
                      Acopy[i][j]=avg_col;

           }






           /* avg_row = sum of cells which are not empty over the row
                        / # of cells which are not empty
           */

          for(i=0;i<943;i++)
           {
              avg_row[i] = 0;
              total=0;

              for(j=0;j<1682;j++)
              {
                if(A_original[i][j] > 0)
                {
                   avg_row[i] += A_original[i][j];
                   ++total;
                }
              }

              if(total>0)
                avg_row[i] = avg_row[i]/total;
              else
                avg_row[i] = 0;

              for(j=0;j<1682;j++)
              {
                 Acopy[i][j] -= avg_row[i];
              }
            }


   /*     for(i=942;i<943;i++)
           for(j=1650;j<1682;j++)
             System.out.println("  user i=" + i + " item j=" + j + " rating = " + Acopy[i][j]);

       */



        /* Begin SVD decomposition */

          DoubleSVD SVD = new DoubleSVD(Acopy);
          U = SVD.U();
          V = SVD.V();
          S = SVD.S();


         for(i=0;i<DIMENSION;i++)
             for(j=0;j<DIMENSION;j++)
             {
                 if(i==j)
                    Sk[i][j]=S[i];
                 else
                    Sk[i][j]=0;
             }

         for(i=0;i<DIMENSION;i++)
             for(j=0;j<DIMENSION;j++)
             {
                 SkHalf[i][j]=Math.sqrt(Sk[i][j]);
             }


         for(i=0;i<943;i++)
           for(j=0;j<DIMENSION;j++)
           {
               Uk[i][j]=U[i][j];
           }

         Matrix VMatrix = new Matrix(V);
         V_transpose = (VMatrix.transpose()).getArray();

         for(i=0;i<DIMENSION;i++)
           for(j=0;j<1682;j++)
           {
              Vk_transpose[i][j] = V_transpose[i][j];
           }


         Matrix UkMatrix = new Matrix(Uk);
         Matrix SkMatrix = new Matrix(Sk);
         Matrix Vk_transposeMatrix = new Matrix(Vk_transpose);

         Matrix SkHalfMatrix = new Matrix(SkHalf);
         Matrix UkSkHalfMatrix = new Matrix(943,DIMENSION);
         UkSkHalfMatrix = UkMatrix.times(SkHalfMatrix);
         Matrix SkHalfVkTransMatrix = new Matrix(DIMENSION,1682);
         SkHalfVkTransMatrix = SkHalfMatrix.times(Vk_transposeMatrix);

         Matrix resultMatrix = new Matrix(943,1682);
         resultMatrix = UkSkHalfMatrix.times(SkHalfVkTransMatrix);

         result = resultMatrix.getArray();


         for(i=0;i<943;i++)
           for(j=0;j<1682;j++)
           {
               if(A_original[i][j]<=0)
                   result[i][j] += avg_row[i];
               else
                   result[i][j] = A_original[i][j];
           }





    }



}
