/*
 * UserItemAlgorithm.java
 *
 * Version 1.1
 *
 * 23/08/2002
 *
 * Copyright (c) 2003 Oregon State University. All rights reserved.
 */
package org.recommender.algorithms.Experimental;
import org.recommender.algorithms.*;
import org.recommender.server.*;
import java.io.*;
import java.util.*;
import java.sql.*;

/**
 * Describes an implementation of the CFAlgorithm interface
 * to make CF predictions using a modification of the Simple
 * Pearson Correlation method.  We find neighbors using
 * Pearson correlations, but weight the effect of each item
 * by its similarity to the target item we're predicting.
 * i.e. in computing the similarity between two users, some
 * items have a larger effect than others.
 *
 * @author  Daniel Lowd &lt;dlowd@cs.orst.edu&gt;
 */

 /* In my limited experiments, this did not turn out to be any
 * more effective than SimplePearsonAlgorithm, so it was
 * abandoned, even before it was fully commented.  It's possible
 * that it's a good algorithm, just buggy, or needing another
 * idea or two... but I consider it a failed experiment.
 *
 * Comments are spotty, but most of the code was lifted from
 * SimplePearsonAlgorithm and ItemItemAlgorithm.
 */

public class UserItemAlgorithm implements CFAlgorithm
{
    private DataManager dataMgr;

    ///////////// Parameters /////////////////
    // DL Some explanations are ambiguous or incomplete.  Some may change...

    /**
     * Minimum number of neighbors required.  If we have fewer than
     * this number of neighbors, return ERR_RATING rather than a
     * prediction.
     */
    private int MIN_NEIGHBORHOOD_SIZE;

    /**
     * Maximum number of neighbors to use for generating a prediction.
     */
    private int MAX_NEIGHBORHOOD_SIZE;

    /**
     * Minimum number of items in common for a correlation to be valid.
     */
    private int MIN_COMMON_ITEMS;

    /**
     * Maximum number of items to consider for weighting a correlation
     * based on the number of items.  Correlations benefit from being
     * based on a larger number of items up to this number.
     */
    private int MAX_COMMON_ITEMS;

    /**
     * Neighbor weight threshold.  Neighbors with weights below this value
     * will not be used.
     */
    private float MIN_CORRELATION;

    private long NumPredictions  = 0;
    private long Time  = 0;

    // ItemItem stuff... TODO: cmt
    private int maxItem;
    float[][] itemWeights;
    float[] itemAverages;

    /**
     * Initialize parameters, set up neighborhood, and calculate item average.
     * @param dataMgr the DataManager to get data from
     */
    public UserItemAlgorithm(DataManager dataMgr)
    {
        this.dataMgr = dataMgr;

        // Get parameters
        String prefix = "CFServer.alg.useritem.";
        MIN_NEIGHBORHOOD_SIZE = SystemProperties.getInt(
                prefix + "minNeighborhoodSize", 5);
        MAX_NEIGHBORHOOD_SIZE = SystemProperties.getInt(
                prefix + "maxNeighborhoodSize", 30);
        MIN_COMMON_ITEMS  = SystemProperties.getInt(
                prefix + "minCommonItems", 5);
        MAX_COMMON_ITEMS  = SystemProperties.getInt(
                prefix + "maxCommonItems", 60);
        MIN_CORRELATION   = SystemProperties.getFloat(
                prefix + "minCorrelation", 0);

        setUpNeighborhoods();
        calcItemAverages();
    }

    /**
     * Set up neighborhood using pearson correlations.
     */
    void setUpNeighborhoods()
    {
        // Set up neighborhoods...
        maxItem = dataMgr.getMaxItem();
        itemWeights = new float[maxItem + 1][maxItem + 1];
        for (int item = 0; item < maxItem; item++)
            for (int neighbor = 0; neighbor < maxItem; neighbor++)
            {
                itemWeights[item][neighbor] = getWeight(item, neighbor);

                // Only use positive correlations
                if (itemWeights[item][neighbor] < 0)
                    itemWeights[item][neighbor] = 0;
            }
    }

    // Item-Item weights
   /**
    * Weight the effect of each item
    * by its similarity to the target item we're predicting.
    * @param activeitem
    * @param neighbor
    */
	public float getWeight(int activeitem, int neighbor)
    {
        float accum = 0;
        float den1 = 0;
        float den2 = 0;

        int numUsers = 0;

        // Consider all the users who have rated this item
        Iterator iter = dataMgr.getNeighborhood(activeitem);
        while (iter.hasNext())
        {
            ItemRating ir = (ItemRating)iter.next();
            float ourRating = ir.getRating();
            float neighborRating = dataMgr.getRatingCached(ir.getUserID(),
                    neighbor);

            // Only consider users who have rated the neighbor item as well
            if (neighborRating == dataMgr.getErrRating())
                continue;

            float userMean = dataMgr.getMeanRatingCached(ir.getUserID());
            float ourOffset = ourRating - userMean;
            float theirOffset = neighborRating - userMean;
            accum += ourOffset * theirOffset;
            den1 += (ourOffset*ourOffset);
            den2 += (theirOffset*theirOffset);
            numUsers++;
        }

        float weight;
        if (den1 > 0 && den2 > 0)
            weight = accum/((float) Math.sqrt(den1) * (float) Math.sqrt(den2));
        else
            weight = 0;

        // TODO: weight by # of users again?

        return numUsers*weight;
    }

    /**
     * Calculate item averages.
     */
    void calcItemAverages()
    {
        // Calculate item averages
        itemAverages = new float[maxItem+1];
        for (int i = 0; i < maxItem; i++)
        {
            float accum = 0;
            int denom = 0;
            Iterator iter = dataMgr.getNeighborhood(i);
            while (iter.hasNext())
            {
                ItemRating ir = (ItemRating)iter.next();
                accum += ir.getRating();
                denom++;
            }

            if (denom > 0)
                itemAverages[i] = accum/denom;
            else
                itemAverages[i] = 0;
        }
    }

    /**
     * Not yet complete.
     */
    public ItemPrediction[] getRecommendations(int activeUser, int n)
            throws SQLException, CFNotImplementedException
    {
    	throw new CFNotImplementedException("getRecommendations not implemented yet");
        //return null;
    }

    /**
     * Not yet complete
     */
    public ItemPrediction[] getRecommendationsByType(int activeUser, int n, int type)
               throws SQLException, CFNotImplementedException
   {
   		throw new CFNotImplementedException("getRecommendationsByType not implemented yet");
        // return null;

   }

    /**
     * Predicts the rating for the given user and item.
     * Similar to SimplePearsonAlgorithm
     * @param activeUser
     * @param item
     * @return predicted rating
     */
    public ItemPrediction predictRating (int activeUser, int item) throws CFNotImplementedException
    {
        float numerator = 0;
        float denominator = 0;

        long startTime = System.currentTimeMillis();
        // get list of Neighbors
        Iterator neighbors = dataMgr.getNeighborhood(item);
        List neighborCoeffs = getSimilarityWeights(activeUser, neighbors, item);

        int numNeighbors = neighborCoeffs.size();
        if (numNeighbors < MIN_NEIGHBORHOOD_SIZE)
        {
            return new ItemPrediction(0,0,0);
        }
        else if (MAX_NEIGHBORHOOD_SIZE > 0 &&
                numNeighbors > MAX_NEIGHBORHOOD_SIZE)
        {
            numNeighbors = MAX_NEIGHBORHOOD_SIZE;
        }

        // sort descending, so we can use just the best
        Collections.sort(neighborCoeffs, Collections.reverseOrder());

        // now, calculate the prediction based on each neighbor
        Iterator iter = neighborCoeffs.iterator();
        int neighborsUsed = 0;
        while (iter.hasNext())
        {
            SimilarityWeight w = (SimilarityWeight) iter.next();

            float rating = dataMgr.getRatingCached(w.getNeighborID(), item);
            float neighborMean = dataMgr.getMeanRatingCached(w.getNeighborID());
            float weight = w.getWeight();

            numerator += weight * (rating-neighborMean);
            denominator += Math.abs(weight);

            // Select only the best neighbors.
            neighborsUsed++;
            if (neighborsUsed >= numNeighbors)
                break;
        }

        // start the final calc!

        float retval;
        if (denominator != 0 && neighborsUsed >= MIN_NEIGHBORHOOD_SIZE)
        {
            float activeUserMean = dataMgr.getMeanRatingCached(activeUser);
            retval = activeUserMean + (numerator/denominator);

            if (retval > dataMgr.getMaxRating())
                retval = dataMgr.getMaxRating();
            else if (retval < dataMgr.getMinRating())
                retval = dataMgr.getMinRating();
        }
        else
        	retval= dataMgr.getErrRating();


        long stopTime = System.currentTimeMillis();

        NumPredictions++;
        Time += (stopTime-startTime);

        return new ItemPrediction(activeUser,item,retval); // return predicted rating
    }

    /**
     * Return average prediction time.
     * @return average prediction time.
     */
	public long getAveragePredictionTime() throws CFNotImplementedException
    {
		return (Time/NumPredictions);
	}

    /**
     * Calculate CorrelatinCoeff for an item between the activeuser and one neighbor.
     * This is a method in SimplePearsonAlgorithm
     * @param activeuser
     * @param neighbor
     * @param itemId
     * @return SimilarityWeight
     */
	public SimilarityWeight getWeight(int activeuser, int neighbor, int item)
    {
   		//****Declarations*****************************************
        float accum = 0;
        float coeff = 0;

        float activeUserSD = 0;
        float activeUserMean = 0;
        float neighborSD = 0;
        float neighborMean = 0;

        int numCommonItems= 0;

      	//**********************************************************

      	//Get active user and neighbor average rating and standard deviation
        try{
			activeUserMean = dataMgr.getMeanRatingCached(activeuser);
			activeUserSD = dataMgr.getUserSD(activeuser);
        	//now the neighbor's items
			neighborMean = dataMgr.getMeanRatingCached(neighbor);
        	neighborSD = dataMgr.getUserSD(neighbor);
		} catch(java.lang.NullPointerException ex){
			System.out.println("Error in getWeight::Can't get user ratings stats, "+ neighbor);
		}

		// If either user has only rated one item, the SD is zero
        // and the correlation is worthless.
        if (activeUserSD == 0 || neighborSD == 0)
        {
            return new SimilarityWeight(neighbor, 0);
        }

        if (activeUserMean < 0 || activeUserSD < 0 ||
                neighborMean < 0 || neighborSD < 0)
        {
            // DEBUG
            System.out.println("Error: could not access stats for user "
                    + activeuser + " or " + neighbor + ".");
            return new SimilarityWeight(neighbor, 0);
        }

   		Iterator iter = dataMgr.getItemList(activeuser);
        while (iter.hasNext())
        {
            ItemRating activeUserRating = (ItemRating)iter.next();
            int currItem = activeUserRating.getItemID();
            float neighborRatingVal = dataMgr.getRatingCached(neighbor,
                    currItem);

            if (neighborRatingVal != dataMgr.getErrRating())
            {
                // The item has been rated by both
                if (itemWeights[item][currItem] > 0)
                    numCommonItems++;

                // get the rating values
                float activeUserRatingVal = activeUserRating.getRating();

                // add the contribution of this common item to the sum
                // (Pearson correlation numerator)
                // Weight by the similarity/relevance of the given item
                accum += itemWeights[item][currItem]
                    * (activeUserRatingVal - activeUserMean)
                    * (neighborRatingVal - neighborMean);
            }
        }


   		//Compute Coeff
        coeff= accum/activeUserSD/neighborSD;

        if (coeff < MIN_CORRELATION)
            coeff = 0;

        // If too few ratings then the correlation is irrelevant => set to zero
        int activeNumRatings = dataMgr.getNumRatings(activeuser);
        if (numCommonItems < MIN_COMMON_ITEMS)
        {
            coeff = 0;
		}

        // Penalize correlations with fewer ratings if below the cutoff
        else if (numCommonItems < MAX_COMMON_ITEMS)
            coeff *= ((float)numCommonItems)/MAX_COMMON_ITEMS;

        return (new SimilarityWeight (neighbor,coeff));
   	}

	/**
     * Return a List of SimilarityWeights for an item between the activeUser and each of
     * his/her neighbors. Similar to SimplePearsonAlgorithm.
     * @param activeUser
     * @param neighbors
     * @param itemId
     * @return List of SimilarityWeights
     */
    private List getSimilarityWeights (int activeUser, Iterator neighbors,
            int item)
    {
        LinkedList vals = new LinkedList();

        while (neighbors.hasNext())
        {
            ItemRating neighborInitial = (ItemRating) neighbors.next();
            vals.add(getWeight(activeUser, neighborInitial.getUserID(),
                        item));
        }

        return vals;
	}

	/**
	 * Not yet complete.
	 */
    public void updateUser(int userID) throws CFNotImplementedException
    {
   		throw new CFNotImplementedException("getRecommendationsByType not implemented yet");
    }

    /**
     * Return the name of this algorithm.
     * @return the name of this algorithm.
     */
    public String toString()
    {
        return "UserItemAlgorithm.java\n";
    }
}
