/* BoolParameter.java
 *
 * Author: Daniel Lowd <dlowd@cs.orst.edu>
 *
 * Represents a mutable algorithm parameter with an boolean value.  Contains
 * methods for parsing itself (given a StringTokenizer) and permuting itself.
 * In the case of a boolean parameter, this is very simple -- there is only
 * one possible permutation.  Used by MutableParameterSet.
 */
package org.recommender.algorithms.Experimental.tuning;

import java.util.*;

public class BoolParameter extends Parameter
{
    String name;
    boolean value;

    public BoolParameter(String name, StringTokenizer st)
    {
        this.name = name;

        String val = st.nextToken();
        if (val.equalsIgnoreCase("true"))
            value = true;
        else
            value = false;
    }

    public BoolParameter(String name, boolean value)
    {
        this.value = value;
    }

    public BoolParameter(BoolParameter other)
    {
        name  = other.name;
        value = other.value;
    }

    // Generate a new parameter with opposite boolean value
    public Parameter permute()
    {
        return new BoolParameter(name, !value);
    }

    public Parameter iterPermute()
    {
        return new BoolParameter(name, !value);
    }

    public boolean equals(Parameter other)
    {
        if (other.getType() != TYPE_BOOL)
            return false;

        BoolParameter p = (BoolParameter)other;

        return (value == p.value);
    }

    public int getType()
    {
        return TYPE_BOOL;
    }

    public String getName()
    {
        return name;
    }

    public String getValue()
    {
        return "" + value;
    }
}
