/* StringParameter.java
 *
 * Author: Daniel Lowd <dlowd@cs.orst.edu>
 *
 * Represents an immutable algorithm parameter with an string value.  Contains
 * methods for parsing itself (given a StringTokenizer) and randomly permuting
 * itself.  However, string parameters do not permute themselves.  A string
 * parameter may be used for any immutable parameter, since ParameterSet
 * requests parameters as strings anyway and converts them itself.  Used
 * by MutableParameterSet.
 */
package org.recommender.algorithms.Experimental.tuning;

import java.util.*;

public class StringParameter extends Parameter
{
    String name;
    String value;

    public StringParameter(String name, StringTokenizer st)
    {
        this.name = name;
        value = st.nextToken();
    }

    public StringParameter(String name, String value)
    {
        this.name = name;
        this.value = value;
    }

    public Parameter permute()
    {
        return this;
    }

    public Parameter iterPermute()
    {
        return this;
    }

    public boolean equals(Parameter other)
    {
        if (other.getType() != TYPE_STRING)
            return false;

        StringParameter p = (StringParameter)other;

        return name.equals(p.name);
    }

    public int getType()
    {
        return TYPE_STRING;
    }

    public String getName()
    {
        return name;
    }

    public String getValue()
    {
        return value;
    }
}
