package org.recommender.algorithms;

/**
 * Represents the similarity between the active user (or item) and some
 * neighbor user (or item).  The ID of the active user (or item) is not
 * actually stored; it is assumed that users of this class will keep track of
 * that or infer it from context.  Used by many nearest-neighbor algorithms of
 * various varieties.  This was once called "CorrelationCoeff", but its name
 * was changed since it was being used for more than just correlations.
 *
 * @author  Daniel Lowd
 */
public class SimilarityWeight implements Comparable
{
    private int neighborID;
    private float weight;

    /**
     * Construct a new SimilarityWeight.
     *
     * @param neighborID ID representing the neighbor in question.
     * @param weight degree of similarity to neighbor
     */
    public SimilarityWeight(int neighborID, float weight)
    {
        this.neighborID = neighborID;
        this.weight = weight;
    }
    
    /**
     * Returns the ID representing the neighbor we are considering.
     */
    public int getNeighborID() 
    {
        return neighborID;
    }

    /**
     * Returns the similarity weight for this neighbor.
     */
    public float getWeight()
    {
        return weight;
    }

    /** 
     * Implementation of the Comparable interface.  
     * Allows SimilarityWeights to be sorted by weight.
     */
    public int compareTo(Object o)
    {
        SimilarityWeight otherWeight = (SimilarityWeight)o;
        if (otherWeight.getWeight() == weight)
            return 0;
        else if (otherWeight.getWeight() > weight)
            return -1;
        else
            return 1;
    }
}
