/*
* CFEngineClient.java
*
* Version 1.1
*
* 15/08/2002
*
* Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */

package org.recommender.clients.console;

import org.recommender.server.CFEngine;
import org.recommender.server.SystemUtils;
import org.recommender.server.SystemProperties;
import java.rmi.RMISecurityManager;
import java.rmi.Naming;

/**
 * The CF Engine client - using RMI API.
 * This class is the class that the users will interact with.
 * It takes requests from the users and uses the RMI registry
 * to fwd these requests to the CF Engine server.
 *
 * @author  (<A HREF="mailto:sameerk@cs.orst.edu">Sameer Kadam</A>)
 * @author Jon Herlocker
 * @since   JDK 1.2
 * @version Version 1.0.0, 05-26-2001
 */

public class CFEngineClient {
    /**
     * Create a client and start userSession.
     */
    public static void main(String[] args) {
        if (args.length > 1) {
            System.err.println("CFEngineClient takes one paramter - the host name of the CFEngine server");
            System.exit(1);
        }
        String serverHostName = null;
        if (args.length == 1) {
            serverHostName = args[0];
        }

        // Create a client
        CFEngineClient client = new CFEngineClient();

        // Object managing connections to cfengine
        ClientCFManager cfManager = ClientCFManager.getInstance(serverHostName);

        // Start getting user input
        ClientTextUI userSession = new ClientTextUI(cfManager);
        userSession.menuStartSession();
    }

    public CFEngineClient() {
        // Load properties - the cf.path property must be set
        // Usually this is done by passing -Dcf.path=... to the
        // Java VM when it starts.
        String installdir = System.getProperty("cf.path",".");

        // Many of the properties of the cfengine server are set
        // in a global properties file
        SystemProperties.loadProperties(installdir + "/lib/client.properties");
    }
}
