/*
* User.java
*
* Version 1.1
*
* 15/08/2002
*
* Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */

package org.recommender.clients.console;

import java.io.*;

/**
 * Describes the user info for the CF Engine.
 *
 * @author  <A HREF="mailto:sameerk@cs.orst.edu">Sameer Kadam</A>
 * @author  <A HREF="mailto:iyoon@cs.orst.edu">Irwin Yoon</A>
 * @author Jon Herlocker
 *
 */
public class User implements Serializable {

    private int id;
    private String userName;

    /**
     * Use this constructor if have all the information.
     * @param id
     * @param userName
     */
    public User(int id, String userName) {
        this.id = id;
        this.userName = userName;
    }

    /**
     * Constructor when a server-side id exists, but there isn't a client-side
     * user name
     * @param id The cfengine server-side userid
     */
    public User(int id) {
        this.id = id;
        this.userName = ClientConstants.NAME_UNAVAILABLE_STRING;
    }

    /**
     * Only use this constructor if don't have the userID available.
     * This usually occurs before the user has been added to the database.
     *
     * IMPORTANT: you should call setID as soon as you get the userID
     *            This is typically done after the user has been added
     *            to the database and the user id is assigned.
     * @param userName
     */
    public User(String userName) {
        this.id = -1;
        this.userName = userName;
    }

    // Accessors and set methods

    /**
     * Call this method as soon as the user id is assigned.
     * This should be called if you used the constructor that did not
     * contain the userid.
     * @param id
     */
    public void setID(int id) {
        this.id = id;
    }

    /**
     * Accessor, get userID.
     * @return this.id
     */
    public int getID() {
        return this.id;
    }

    /**
     * Accessor, get userName.
     * @return this.userName
     */
    public String getUserName() {
        return this.userName;
    }

    /**
     * toString
     */
    public String toString() {
        return getUserName() + " (id=" + getID() + ")";
    }
}