package org.recommender.server;

/* *
 * CFSynchronize.java
 *
 * Version 1.1
 * 18/05/2003
 *
 * @author  <A HREF="mailto:mclaughm@cs.orst.edu">Matt McLaughlin</A>
 * @author  <A HREF="mailto:wangyun@cs.orst.edu">Yun Wang</A>
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */


public class CFSynchronize {
     //True if object no longer valid
    private boolean deleted;
    //Number of readers who currently have lock
    private int readers;
    //Just one writer ia allowed at a time
    private boolean writers;

    //Number of writers waiting
    private int writersWaiting;

    //next Available number
    private int nextWriter;

    //Writer number who has the right to the lock
    private int nowServing;

    //Queue for writers
    //private Object writersQueue;

    public CFSynchronize(){
        deleted = false;
        readers = 0;
        writers = false;
        writersWaiting = 0;
        nextWriter = 0;
        nowServing = 0;

    }


    /**
     * getReadersLock -- Note, do not write when you have this lock.
     *
     * @param wait True if you want to wait for the lock
     * @return true if you successfully retained lock
     */
    public synchronized boolean getReadersLock(boolean wait){
        if(deleted) return false;
        if(!wait && (writers || writersWaiting > 0)) return false;
        while(!deleted && (writers || (writersWaiting > 0)) ) {
            try{
                this.wait();
            }catch(Exception e){
                System.out.println(e);
                e.printStackTrace();
                return false;
            }
        }
        //have to check again, may have been deleted while you were asleep
        if(deleted) return false;
        readers++;
        return true;
    }
    
    /**
     * getWritersLock  -- Note, you may have to wait for this lock
     * If the object has been deleted, return false.
     * Anyone who wants to wait for the lock have to get a writersNumber.
     * He has to wait until either the object been deleted, or no more writer and readers own the lock
     * lock and his turn coming.
     *
     * @return true if you successfully retained lock
     */
    public synchronized boolean getWritersLock(){
        if(deleted) return false;
        writersWaiting++;
        int writersNumber = nextWriter;
        nextWriter++;
        while( !deleted &&
              ( writers || readers > 0 || (writersNumber != nowServing)) )
        {
            try{
                this.wait();
            }catch(Exception e){
                e.printStackTrace();
                return false;
            }
        }
         //have to check again, may have been deleted while you were asleep
        writersWaiting--;
        if(deleted) return false;

        writers = true;
        return true;
     }


    public synchronized void releaseWritersLock(){
        writers = false;
        nowServing++;
        if(deleted) return;
        this.notifyAll();
    }

    public synchronized void releaseReadersLock(){
        readers--;
        if(deleted) return;
        this.notifyAll();
    }

    /**
     * Marks object as deleted and notifies everyone who is waiting;
     */
    public synchronized void delete(){
        deleted = true;
        this.notifyAll();
    }
}
