/* 
 * ItemInfo.java
 * 
 * Version 1.1
 * 
 * 15/08/2002
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */ 
package org.recommender.server;

import org.recommender.util.*;
import java.util.*;

/**
 * This class implements a item's records, including its ratings
 * list, mean rating and Standard deviation.
 * The class also provides some flags that are used by the 
 * cache replacement algorithm.
 *
 * @author  <A HREF="mailto:godde@cs.orst.edu">Olivier Godde</A>
 * @author  <A HREF="mailto:mclaughm@engr.orst.edu">Matt McLaughlin</A>
 */

public class ItemInfo extends CFSynchronize{
    // The following fields are synchronized to ensure that you always
    // receive the most up-to-date values.
	private IntHashMap itemRatingsHash;
	private float itemMean;
	private float itemSD;

	private float minRating = SystemProperties.getFloat("CFServer.db.minRating", 0);
	
    // sum
    private float x;
    // sum of squares
    private float x2;
    private int num;
    
    /**
	 * Constructor with param of only itemID.
	 * @param ID
	 */
	public ItemInfo(int ID){
        super();
        setItemRatingsHash(new IntHashMap());
		setItemMean(minRating - 1000);
		setItemSD(minRating - 1000);
		//samplingFlag= false;
		//referenced= false;
		
		x = -999;
		x2 = -999;
		num = 0;
	}
	
	/**
	 * Constructor with params of itemID, ratings, mean, standard deviation, sum, squaredSum, and total num.
	 * @param ID
	 * @param ratings
	 * @param mean
	 * @param SD
	 * @param sum
	 * @param squaredSum
	 * @param num
	 */
	public ItemInfo(int ID, IntHashMap ratings, float mean, float SD, float sum, float squaredSum, int num){
        super();

        setItemRatingsHash(ratings);
		setItemMean(mean);
		setItemSD(SD);
		//samplingFlag= false;
		//referenced= false;
		
		x = sum;
		x2 = squaredSum;
		this.num = num;
	}

	/**
	 * Adds rating and updates the necessary stats
	 * @param rating
	 */
	public synchronized void addRating(ItemRating rating){
		float temp = rating.getRating();
		if(num == 0){
			num++;
			x = temp;
			x2 = temp*temp;
			setItemSD(0);
			setItemMean(temp);
		}
		else{
			num++;
			x += temp;
			x2 += (temp * temp);
			setItemSD((float) Math.sqrt ((x2-x*x/num)/(num-1)));
			setItemMean(x/num);
		}
		getItemRatingsHash().put(rating.getUserID(), rating);
	}

	/**
	 * Removes rating and updates the necessary stats
	 * @param userID
	 */
	public synchronized void removeRating(int userID){
		//float temp = rating.getRating();
    //    float temp = ((ItemRating) getItemRatingsHash().get(userID)).getRating();
        ItemRating r = (ItemRating) getItemRatingsHash().get(userID);
        float temp = 0;
		try{
			getItemRatingsHash().remove(userID);
            temp = r.getRating();
		}catch(Exception e){
			System.out.println("Item  not currently cached.  Cannot remove.");
            return;
		}
		
		if(num == 1){
			num--;
			x = minRating - 1000;
			x2 = minRating - 1000;
			setItemSD(minRating - 1000);
			setItemMean(minRating - 1000);
		}
		else{
			num--;
			x -= temp;
			x2 -= (temp * temp);
			setItemSD((float) Math.sqrt ((x2-x*x/num)/(num-1)));
			setItemMean(x/num);
		}
	}

    public synchronized IntHashMap getItemRatingsHash() {
        return itemRatingsHash;
    }

    public synchronized void setItemRatingsHash(IntHashMap itemRatingsHash) {
        this.itemRatingsHash = itemRatingsHash;
    }

    public synchronized float getItemMean() {
        return itemMean;
    }

    public synchronized void setItemMean(float itemMean) {
        this.itemMean = itemMean;
    }

    public synchronized float getItemSD() {
        return itemSD;
    }

    public void setItemSD(float itemSD) {
        this.itemSD = itemSD;
    }
}
