/* 
 * Stats.java
 * 
 * Version 1.1
 * 
 * 15/08/2002
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */ 
package org.recommender.server;

import java.util.*;

/**
 * Calculates Mean or Standard Deviation  for a set of numbers.
 * To use this class, add each number of the set separately.
 * Then call calcMean or calcSD. 
 *
 * @author  <A HREF="mailto:iyoon@cs.orst.edu">Irwin Yoon</A>
 *
 */
public class Stats {

    // sum
    private float x = 0;
    // sum of squares
    private float x2 = 0;
    private int num = 0;

    /**
     * Default constructor.
     * Does nothing
     */
    public Stats()
    { }

    /**
     * Adds the value to the existing set of numbers.
     * @param val
     */
    public void addNum(float val)
    {
        x  += val;
        x2 += val*val;
        num++;
    }

   
    /**
     * Returns the standard deviation for the numbers in the set.
     * @return SD
     */ 
    public float calcSD()
    {
        float retVal = (float) Math.sqrt((x2-x*x/num)/(num-1));

        if (Float.isNaN(retVal))
            retVal = 0;

        return retVal;
    }


    /**
     * Returns the mean for the numbers in the set.
     * @return mean
     */ 
    public float calcMean()
    {
    	try{
        	return x/num;
        	
        } catch (Exception ex){
        	System.out.println("DEBUG:Stats:Wrong value:No user mean rating calculated");
        	return x/num;
        }
    }
    
    public float calcTValue() {
    	float std = this.calcSD();
    	float t = this.calcMean() / (float) Math.sqrt(std*std/this.num);
    	return t;
    }

    /**
     * Returns the number of numbers in the set.
     * @return num
     */ 
    public int size()
    {
        return num;
    }
}
