/* 
 * CFEngineCorbaImpl.java
 * 
 * Version 1.1.1
 * 
 * 29/10/2002
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */
 

package org.recommender.server.corba;

import org.recommender.server.corba.CFEngineCorbaPackage.*;
import org.recommender.algorithms.*;
import org.recommender.server.*;
import org.recommender.util.*;
import java.util.*;
import java.sql.*;
import java.lang.reflect.*;

import org.omg.CORBA.*;

/**
 * Implementation of the CFEngineCorba interface - using pure Corba .
 *
 * @author <A HREF="mailto:wangyun@cs.orst.edu">Yun Wang</A>
 * @since   JDK 1.2
 * @version 
 */
 
public class CFEngineCorbaImpl extends CFEngineCorbaPOA
{
    //private org.omg.PortableServer.POA poa_;
    private ORB orb;
   
    private String host;
    private DataManager dataMgr = null;
   // private CFAlgorithm algorithm = null;
   
    public Class algorithm = null;
    public java.lang.Object newAlgObj=null;
    
   /* public
    CFEngineCorbaImpl(org.omg.PortableServer.POA poa)
    {
        poa_ = poa;
    }

    public org.omg.PortableServer.POA
    _default_POA()
    {
        if(poa_ != null)
            return poa_;
        else
            return super._default_POA();
    }*/
    
    public CFEngineCorbaImpl(ORB orb)
    {
       this.orb = orb;
     } 
    
    public CFEngineCorbaImpl(String propertiesFilename, ORB orb) 
    	throws CFInternalError  
    {
        //poa_ = poa;
        this.orb = orb;       
        
        
        // Load properties
        SystemProperties.loadProperties(propertiesFilename);

    //    host = SystemProperties.getString("CFServer.rmi.nameserver", 
    //            "localhost");
        String testWhere = SystemProperties.getString("CFServer.db.testSet",
                "0");
        String mode = SystemProperties.getString("CFServer.mode", "server");
        
        //for the cached recommendations
        int sizeOfCache = 1000;
       
        if (mode.equals("experiment"))
            dataMgr = new DataManager(false, false);
        else if (mode.equals("autotest"))
            dataMgr = new DataManager(true, false);
        else                
            dataMgr = new DataManager(true, true);           

        try {
           
			String algName = SystemProperties.getString("CFServer.alg.name");
			try{
            	algorithm = Class.forName(algName);
             	SystemUtils.printDebug("Using algorithm: " + algorithm.getName());
             
             	Constructor[] cons = algorithm.getConstructors();
             
             	SystemUtils.printDebug("Constructing a new alg Object, please wait...");
             	
             	long start = System.currentTimeMillis();
             	newAlgObj=cons[0].newInstance(new java.lang.Object[] {dataMgr});
            	
            	long stop = System.currentTimeMillis();
             	SystemUtils.printDebug("Finish constructing a new alg Object, time = "+(stop-start));
             
           	} catch(Exception e) {
              	throw new CFInternalError(" class not found Exception", e.toString());
           	}     
           
            if (algorithm == null)
            {
                // Unknown algorithm name.  Print an error and quit.
                System.out.println("ERROR: Could not construct algorithm.");
                System.exit(0);
            }


           	SystemUtils.printDebug("Using algorithm: " + algorithm);
            
               
           	// Run some automatic tests on our algorithm.   
           	if (mode.equals("autotest"))    
           	{
            	AutoTester.runTests(dataMgr, (CFAlgorithm)newAlgObj, testWhere);     
               
           	}
          
            
        } catch (RuntimeException e) {

            e.printStackTrace();
        }

       
        if (mode.equals("autotest"))
        {
            SystemUtils.printDebug("Tests done, killing server.");
            System.exit(0); 
        }

		// Start a new Thread here, DBManager which is responsible for any dynamic database access
		DBManager db= new DBManager();
		db.start(); 
		
		// The following Thread is responsible for refreshing the cache whenever it fills up
		CacheUpdate cacheThread= new CacheUpdate(dataMgr);
		cacheThread.start();
		
		// The following Thread is responsible for resampling the data
		Resample samplingThread= new Resample(dataMgr);
		samplingThread.start();
    }
    
    /**
     * Test wether the CFEngine is alive on the host.
     * @return String
     */
    public String test()
    {
       // String _r = "CFEngine is alive on " + host;
        String _r = "CFEngine is alive ";
        return _r;
    }

    /**
	 * Retrieve a rating for user' rating from cache.
	 * @param userID
	 * @param itemID
	 * @return ItemRatingCorba
	 * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFIllegalParam
	 * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFInternalError
	 */
    public ItemRatingCorba getRating(int userID, int itemID)
        throws CFIllegalParam, CFInternalError
        
    {
        
		if (userID < 0) 
       		throw new CFIllegalParam("User ID "+ userID +" is out of bounds");
	
		if(itemID < 0) 
			throw new CFIllegalParam("Item ID "+ itemID +" is out of bounds");

            
       	// Retrieve data from cache instead of database.
	    //float rating = dataMgr.getRatingCached(userID, itemID);
	    ItemRating ir = null;
	    try {
            ItemInfo it = dataMgr.getItemInfo(itemID);
            if (it == null) 
            	return new ItemRatingCorba(itemID, dataMgr.getErrRating(), userID);
            else{
                if (!it.getReadersLock(true)) {
                    throw new CFInternalError("Unable to obtain lock", null);
                }
                ir = (ItemRating) it.getItemRatingsHash().get(userID);
                it.releaseReadersLock();
                if (ir == null) {
                    return new ItemRatingCorba(itemID, dataMgr.getErrRating(), userID);
                }
            }
        } catch (Exception e) {
            throw new CFInternalError("Error in getRating", e.toString());
        }

	    return new ItemRatingCorba(ir.getItemID(), 
	    						ir.getRating(),  
	    						ir.getUserID() );
    }
    
    /**
     * Retrieves ratings for user's items from cache.
     *
     * If the user has not rated the item, returns an ItemRating with
     * an ErrRating. @see getErrRating()
     *
     * @param userID
     * @param itemIDs
     * @return array of ItemRatingCorba
     *
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFIllegalListParam Throws exception if userID or itemID[] are invalid
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFInternalError
     */
    public ItemRatingCorba[] getRatingList(int userID, int[] itemIDs)
    	throws CFIllegalListParam, CFInternalError
    {
    	if (userID < 0) 
            throw new CFIllegalListParam("User ID "+ userID +" is out of bounds", 0, null);
        
        if (itemIDs == null) 
            throw new CFIllegalListParam("itemIDs cannot be null", 0, null);
        

        ItemRatingCorba[] returnList = new ItemRatingCorba[itemIDs.length];
        ItemInfo ii = null;       
        for (int i = 0; i < itemIDs.length; i++) {
            if (itemIDs[i] < 0) {
                throw new CFIllegalListParam("Item ID " + itemIDs[i] +
                        " is out of bounds", i, null);
            }
            ItemRating r = null;
            try {
                ii = dataMgr.getItemInfo(itemIDs[i]);
                if (ii == null)
                    returnList[i] = new ItemRatingCorba(itemIDs[i], dataMgr.getErrRating(), userID);
                else {
                    ii.getReadersLock(true);
                    r = (ItemRating) ii.getItemRatingsHash().get(userID);
                    ii.releaseReadersLock();
                    if (r == null) 
                    	returnList[i] = new ItemRatingCorba(itemIDs[i], dataMgr.getErrRating(), userID);
                    else
                    	returnList[i] = new ItemRatingCorba(r.getItemID(),
            									r.getRating(),
            									r.getUserID());
                }
            } catch (Exception e) {
                throw new CFInternalError("Error in getRatingList", e.toString());
            }
			            
        } // for
        
        return returnList;
    }
    
    /**
     * Get top n recommendation for user.
     *
     * First load the active user into the cache if nessary.
     * Parameters specify the number of recommendations to retrieve and the offset 
	 * (returns "number" recommendations starting from recommendation # "offset")
     * @param curUser
     * @param number
     * @param offset
     * @return ItemPredictionCorba[]
     *
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFInternalError
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFIllegalParam
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFEx
     */ 
    public ItemPredictionCorba[] getRecommendations(int userID, int number, int offset)
       throws CFInternalError, CFIllegalParam, CFEx
    {
        
        if(userID < 0) 
        	throw new CFIllegalParam("User ID "+ userID +" is out of bounds");
		
		if(number < 0) 
			throw new CFIllegalParam("Param number must be greater than 0.  number = " + number);
		
		if(offset < 0)
			throw new CFIllegalParam("Param offset must be greater or equal to 0.  offset = " + offset);	
        
        if (userID > dataMgr.getMaxUser()){
      		return new ItemPredictionCorba[0];
    	}
    	
    	//This function will load the active user into the cache if necessary
        //Perform this check here since we don't have login sessions
		try {
            dataMgr.checkCachedUser(userID);
        } catch (Exception e) {
            throw new CFInternalError("Error in getRecommendations", e.toString());
        }
		
		ItemPrediction[] p;
		
		if (!dataMgr.checkCachedTopN(userID) 
			|| ( (offset == 0) && dataMgr.getAddedRatings(userID) > 0) ){
		    
		    java.lang.Object retVal = null;		    
		    java.lang.Object[] argObj = {new Integer(userID), 
		    							new Integer(dataMgr.getNumTopNRecords())};
		    
            try{
                 Method met = findgetRecommendations(algorithm);
                 retVal = met.invoke(newAlgObj, argObj);
            }catch(CFNotImplementedException e) {  
        		dataMgr.decUseCount(userID);
        		throw new CFEx("This method haven't been implemented.");
        		  
            }catch(Exception e) {
                dataMgr.decUseCount(userID);
                throw new CFInternalError(" exception in invoking getRecommendations method ", e.toString());
            }
             
		    dataMgr.cacheUserTopN(userID, (ItemPrediction[]) retVal);		
			
			dataMgr.resetUserCounter(userID);
			
			p = dataMgr.getCachedTopN(userID, number, offset);
			        
        } else{
            p = dataMgr.getCachedTopN(userID, number, offset);
        }	
        
        ItemPredictionCorba[] c = new ItemPredictionCorba[p.length];;
		for (int i = 0; i < p.length; i++) {
			c[i] = new ItemPredictionCorba(p[i].getItemID(),
        	                               p[i].getPrediction(),
        	                               p[i].getUserID());
		}
		dataMgr.decUseCount(userID);
        return c;        
        
    }

    /**
     * Get top n recommendations for current user of specific type.
     *
     * Returns array of size equal to the number of valid predictions in the
     * range requested.  If there are no valid recommendations in the specified
     * range, returns 0 length array.
     *
     * @param userID User to get recommendations for.
     * @param number Number of recommendations to be retrieved.
     * @param offset Returns number of recommendations, starting at the offset
     * @param type Catagory to gat recommendations from.
     * @return ItemPredictionCorba[]
     *
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFEx
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFIllegalParam
     * @throw CFInternalError
     */
    public ItemPredictionCorba[] getRecommendationsByType(int userID, int number, 
                                                     	  int offset, int type) 
            throws CFEx, CFIllegalParam, CFInternalError
    {
         
    	if(userID < 0 ) 
    		throw new CFIllegalParam("User ID " + userID + " is out of bounds");
		
		if(number < 0) 
			throw new CFIllegalParam("Param number must be greater than 0.  number = "+number);
		
		if(offset < 0) 
			throw new CFIllegalParam("Param offset must be greater or equal to 0.  offset = "+offset);	
		
		if(type < 0 || type > dataMgr.getMaxType()) 
			throw new CFIllegalParam("Param type must be valid.  type = "+ type);	
       
        if (userID > dataMgr.getMaxUser()) {
      		return new ItemPredictionCorba[0];
    	}
    	try {
            dataMgr.checkCachedUser(userID);
        } catch (Exception e) {
            throw new CFInternalError("Error caching user", e.toString());
        }
        
        java.lang.Object retVal = null;		    
		java.lang.Object[] argObj = {new Integer(userID), 
									new Integer(dataMgr.getNumTopNRecords()), 
									new Integer(type)};
        try{
              Method met = findgetRecommendationsByType(algorithm);
              retVal = met.invoke(newAlgObj, argObj);
        }catch(CFNotImplementedException e) {  
        	dataMgr.decUseCount(userID);
        	throw new CFEx("This method haven't been implemented.");
        	 
		}catch(Exception e) {
         	dataMgr.decUseCount(userID);
            throw new CFInternalError(" exception in invoking getRecommendationsByType method ", e.toString());
        
        }	
        
        ItemPrediction[] pred = dataMgr.getTopNByType(userID, number, offset, type, 
		                             (ItemPrediction[])retVal );	
		
		
		ItemPredictionCorba[] pcorba = new ItemPredictionCorba[pred.length];
		
		for (int i = 0; i < pred.length; i++) {
		  pcorba[i] = new ItemPredictionCorba(pred[i].getItemID(),
        	                       pred[i].getPrediction(),
        	                       pred[i].getUserID());
        }
        dataMgr.decUseCount(userID);
        return pcorba;
    }
    
    
    /**
     * Get predicted rating for user's item.
     *
     * If there is an error predicting rating, returns an ItemPrediction with
     * an ErrRating. @see getErrRating()
     *
     * @param userID
     * @param itemID
     * @return predictedRating
     *
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFInernalError
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFIllegalParam
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFEx
     */ 
    public ItemPredictionCorba getPredictedRating(int userID, int itemID)
        throws CFInternalError, CFIllegalParam, CFEx
    {
        if(userID < 0)
        	throw new CFIllegalParam("User ID "+ userID +" is out of bounds");
		
		if(itemID < 0) 
			throw new CFIllegalParam("Item ID "+ itemID +" is out of bounds");
        
        if (userID > dataMgr.getMaxUser() || itemID > dataMgr.getMaxItem()) {
      				return new ItemPredictionCorba(itemID, dataMgr.getErrRating(),userID);
    	}
        
        //This function will load the active user into the cache if necessary
        //Perform this check here since we don't have login sessions
        try {
            dataMgr.checkCachedUser(userID);
        } catch (Exception e) {
            throw new CFInternalError("Error caching user", e.toString());
        }      
        
        java.lang.Object returnRating = null;
       	java.lang.Object[] argObj = {new Integer(userID), new Integer(itemID)};
        try{
          	Method predictRating = findpredictRating(algorithm);
          	returnRating = predictRating.invoke(newAlgObj, argObj);
        
        }catch(CFNotImplementedException e) {  
        	dataMgr.decUseCount(userID);
        	throw new CFEx("This method haven't been implemented.");
        } catch(Exception e) {
        	dataMgr.decUseCount(userID);
          	throw new CFInternalError(" exception in invoking predictRating method ", e.toString());
        }
       
       ItemPrediction pred= (ItemPrediction) returnRating;
       dataMgr.decUseCount(userID);      
       return new ItemPredictionCorba(pred.getItemID(), 
       								pred.getPrediction(), 
       								pred.getUserID());
    }

    /**
     * Gets predicted ratings for a list of items.
     *
     * For each invalid item in itemID[], returns an ItemPrediction with a
     * predicted value of ErrRating. @see getErrRating()
     *
     * @param userID
     * @param itemID a list of items
     * @return ItemPredictionCorba[]
     *
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFInternalError
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFIllegalListParam
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFEx
	 */
    public ItemPredictionCorba[] getPredictedRatingList(int userID,int[] itemID)
        throws CFInternalError, CFIllegalListParam, CFEx
    {
        if (itemID == null)
			throw new CFIllegalListParam("Item ID array is null", 0, null);
	
		if(userID < 0 ) 
			throw new CFIllegalListParam("User ID "+ userID +" is out of bounds", 0, null);
        
                 
        ItemPredictionCorba[] retval = new ItemPredictionCorba[itemID.length]; 
         
        int i = 0;
        try {
         	for(; i < retval.length;i++)
         	{
             retval[i] = getPredictedRating(userID, itemID[i]);
          	}    
         
         } catch (CFIllegalParam e) {
            throw new CFIllegalListParam("Element in list failed.", i, e.msg);
         }
         
         return retval; 
    } 
    
    /**
     * Return an array of itemIDs which the given user has rated.
     *
     * Note: returns 0 length array if there are no ratings.
     *
     * @param userID
     * @return an array of itemIDs
     *
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFIllegalParam Throws exception if userID is invalid
     */
    public ItemRatingCorba[] getUserRatingList(int userID)
    	throws CFIllegalParam
    {
        if(userID < 0 ) 
        	throw new CFIllegalParam("User ID "+ userID +" is out of bounds");
        
        if (userID > dataMgr.getMaxUser()) {
            return new ItemRatingCorba[0];
        }
        
        UserInfo userInfo = dataMgr.getUserInfo(userID);
        if (userInfo == null) {
            // Return a zero-length array
            return new ItemRatingCorba[0];
        }
        
        if (!userInfo.getReadersLock(true)) return new ItemRatingCorba[0];
        RatingArrays ratings = userInfo.getRatings();
        ItemRating[] ret = ratings.getItemRatingsArray();
        userInfo.releaseReadersLock();
        
        if (ret == null) return new ItemRatingCorba[0];
        
        ItemRatingCorba[] rateCorba = new ItemRatingCorba[ret.length];
        for(int i = 0; i < ret.length; i++)
        {
        	rateCorba[i] = new ItemRatingCorba(ret[i].getItemID(),
        									ret[i].getRating(),
        									ret[i].getUserID());
        }
        return rateCorba;
    }

    /**
     * Return an array of userIDs who have rated the given item.
     *
     * Note: returns 0 length array if there are no ratings.
     *
     * @param itemID
     * @return an array of userID
     *
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFIllegalParam
     */    
    public ItemRatingCorba[] getItemRatingList(int itemID)
    	throws CFIllegalParam
    {
        if(itemID < 0 ) 
        	throw new CFIllegalParam("Item ID "+ itemID +" is out of bounds");
       
        ItemInfo itemInfo = dataMgr.getItemInfo(itemID);
        if(itemInfo == null) return new ItemRatingCorba[0];
        if(!itemInfo.getReadersLock(true)) return new ItemRatingCorba[0];
        IntHashMap itemHash = itemInfo.getItemRatingsHash();
        if(itemHash == null){
            itemInfo.releaseReadersLock();
            return new ItemRatingCorba[0];
        }
        
        ItemRatingCorba[] rateCorba = new ItemRatingCorba[itemHash.size()];
        
        Iterator iter = itemHash.iterator();
        int i = 0;
        iter = itemHash.iterator();
        while (iter.hasNext()) {
            ItemRating ret = (ItemRating) iter.next();
            rateCorba[i] = new ItemRatingCorba(ret.getItemID(),
        									ret.getRating(),
        									ret.getUserID());
            i++;
        }
        itemInfo.releaseReadersLock();
        
        return rateCorba;
    }

    
   
     /**
     * Send a new rating the CFEngine server.
     *
     * Note: If rating already exists, it replaces it.
     *
     * @param user
     * @param item
     * @param ratingValue
     *
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFIllegalParam
     * @throws CFInternalError
     */
    public void setRating(int user, int item, float ratingValue)
    	throws CFIllegalParam, CFInternalError
    {
               
        if(user < 0 ) 
        	throw new CFIllegalParam("User ID "+ user +" is out of bounds");
		
		if(item < 0 ) 
			throw new CFIllegalParam("Item ID "+ item +" is out of bounds");
		
		if (ratingValue < dataMgr.getMinRating() 
			|| ratingValue > dataMgr.getMaxRating())
	    	throw new CFIllegalParam("Rating Value " + ratingValue + " is out of bounds.");
                
    	ItemRating rating = new ItemRating(user, item, ratingValue);
   		try {
   			dataMgr.addRating(rating);
   		} catch (Exception e) {
            throw new CFInternalError("Error caching user", e.toString());
        }
    	// Update any algorithm models that may exist
        //algorithm.updateUser(user);  Until not sure
    }

    /**
     * Add a list of new ItemRatings to both database and the cache.
     * If a rating already exists for a (user, item), then the old rating
     * will be overwritten.
     *
     * @param newRatings a list of new ratings wanted to add
     * @return numSuccess Returns the number of new ratings that were successfully added to the database
     *
     * @throws CFIllegalListParam
     * @throws CFInternalError
     */
    public int setRatingList(ItemRatingCorba[] newRatings)
    	throws CFIllegalListParam, CFInternalError
    {
    	if (newRatings == null) {
            throw new CFIllegalListParam("newRatings can not be null", 0, null);
        } 
        float minRating = dataMgr.getMinRating();
        float maxRating = dataMgr.getMaxRating();
        
        for (int i = 0; i < newRatings.length; i++) {
            if (newRatings[i].userID < 0 || newRatings[i].itemID < 0 ||
                    newRatings[i].rating < minRating ||
                    newRatings[i].rating > maxRating) 
            {
                throw new CFIllegalListParam("Elements parameter was out of range", i, null);
            }
            ItemRating r = new ItemRating(newRatings[i].userID,
            							newRatings[i].itemID,
            							newRatings[i].rating);
            try {
                dataMgr.addRating(r);
            } catch (Exception e) {
                throw new CFInternalError("Error caching user", e.toString());
            }
        }
        return newRatings.length;
    }
    
    /**
     * Remove the rating from both database and the cache.
     *
     * Removes rating from both database and cache.  If rating doesn't exist, it
     * just returns.
     *
     * @param user
     * @param item
     *
     * @throws org.recommender.server.corba.CFEngineCorbaPackage.CFIllegalParam
     * @throws CFInternalError
     */
    public void removeRating(int user, int item)
    	throws CFIllegalParam, CFInternalError
    {
        if(user < 0)
        	throw new CFIllegalParam("User ID "+ user +" is out of bounds");
	
		if(item < 0) 
			throw new CFIllegalParam("Item ID "+ item +" is out of bounds");
        
        if (user > dataMgr.getMaxUser() || item > dataMgr.getMaxItem()) {
            return;
        }
        
        try {
            dataMgr.removeRating(user, item);
        } catch (Exception e) {
            throw new CFInternalError("Error caching user", e.toString());
        }
 		
 		// Update any algorithm models that may exist
        //algorithm.updateUser(user); 	Until not sure
    }
    
    /**
     * Removes list of ratings from users profile
     *
     * @param user
     * @param itemIDs
     *
     * @throws CFInternalError
     * @throws CFIllegalListParam Throws exception if userID or itemID[] are invalid
     */
    public void removeRatingList(int user, int[] itemIDs)
    	throws CFIllegalListParam, CFInternalError
    {
    	if (user < 0) {
            throw new CFIllegalListParam("User ID " + user +
                    " is out of bounds", 0, null);
        }
        if (itemIDs == null) {
            throw new CFIllegalListParam("itemIDs cannot be null", 0, null);
        }
        
        for (int i = 0; i < itemIDs.length; i++) {
             if (itemIDs[i] < 0) {
                 throw new CFIllegalListParam("Item ID " + itemIDs[i] +
                         " is out of bounds", i, null);
             }
             try {
                 dataMgr.removeRating(user, itemIDs[i]);
             } catch (Exception e) {
                 throw new CFInternalError("Error caching user", e.toString());
             }
         }
        
    }
   
   /**
   	* Graciously shut the server down. Execute all pending DB requests before exiting.
   	* 
   	*/
    public void shutdown()
    {
        dataMgr.shutdown();
    }
    
    
   /**
	*  Get the maximum rating allowed for the current server configuration.
    *
    * @return max
    *
    * @throws RemoteException Thrown upon client/server connection error
    */
	public float getMaxRating()
	{
    	return dataMgr.getMaxRating();
  	}

   /**
  	*  Get the minimum rating allowed for the current server configuration.
   	*
  	* @return min
  	*
   	* @throws RemoteException Thrown upon client/server connection error
   	*/
	public float getMinRating()
  	{
    	return dataMgr.getMinRating();
  	}
  	
  	/**
     *  Get the error rating for the current server configuration.
     *
     * @return error rating
     *
     * @throws RemoteException Thrown upon client/server connection error
     */
    public float getErrRating() {
        return dataMgr.getErrRating();
    }
  	
	/**
     * @return An unused integer userid that is guaranteed to be unique
     */
    public int getNextUserId() throws CFInternalError {
        int id = -1;
        try {
            id = dataMgr.getNextUserId();
        } catch (Exception e) {
            throw new CFInternalError("Error getting user id", e.toString());
        }
        return id;
    }
   
	/**
     * @return An unused integer itemid that is guaranteed to be unique
     */
    public int getNextItemId() throws CFInternalError {
        int id = -1;
        try {
            id = dataMgr.getNextItemId();
        } catch (Exception e) {
            throw new CFInternalError("Error getting item id", e.toString());
        }
        return id;
    }

    public String getStatistics() {
        return dataMgr.getStatistics();
    }	
	//////////////////////////////////////////////////////////////////////////
    
    /** Private method: Added for finding the corresponding methods 
     */
    private static Method findpredictRating(Class clazz) throws Exception
    {
        Method[] methods = clazz.getMethods();
        for(int i=0; i<methods.length;i++)
        {
            if(methods[i].getName().equals("predictRating"))
               return methods[i];
        }
        
        return null;
    } 
    
    /** Private method: Added for finding the corresponding methods 
     */
    private static Method findgetRecommendations(Class clazz) throws Exception
    {
        Method[] methods = clazz.getMethods();
        for(int i=0; i<methods.length;i++)
        {
            if(methods[i].getName().equals("getRecommendations"))
               return methods[i];
        }
        
        return null;
    } 
    
    /** Private method: Added for finding the corresponding methods 
     */
    private static Method findgetRecommendationsByType(Class clazz) throws Exception
    {
        Method[] methods = clazz.getMethods();
        for(int i=0; i<methods.length;i++)
        {
            if(methods[i].getName().equals("getRecommendationsByType"))
               return methods[i];
        }
        
        return null;
    }                      

    //////////////////////////////////////////////////////////////////////////////

}
