/* 
 * CFEngineServerCorba.java
 * 
 * Version 1.1
 * 
 * 30/10/2002
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */

package org.recommender.server.corba;

import org.recommender.util.*;
import org.recommender.server.*;
//import org.omg.PortableInterceptor.*;
import java.io.*;

/**
 * The CF Engine server implementation - using CORBA.
 * Create a CFEngineImpl server object and start API.
 * Remote clients lookup for this object and send 
 * their CF requests to this server.
 *
 * @author  <A HREF="mailto:wangyun@cs.orst.edu">Yun Wang</A>
 */
public class CFEngineServerCorba {
   
   
    static int run(org.omg.CORBA.ORB orb, String[] args)
               throws org.omg.CORBA.UserException
    {
        //
        // Create & install value factory - if we forget to do this,
        // the client will receive a MARSHAL exception when attempting
        // to transmit ItemPrediction and ItemRating value.
       /* ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
            ItemPredictionCorbaHelper.id(), new ItemPredictionCorbaValueFactory_impl());
            
        ((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
            ItemRatingCorbaHelper.id(), new ItemRatingCorbaValueFactory_impl()); 
            
        */
        //
        // Resolve Root POA
        //
        org.omg.PortableServer.POA rootPOA =
	        org.omg.PortableServer.POAHelper.narrow(
	          orb.resolve_initial_references("RootPOA"));
       
       
        rootPOA.the_POAManager().activate();

        //
        // Create implementation object
        //
        String installdir = System.getProperty("cf.path",".");
       
        String filename = installdir + "/lib/" 
                       + "CFServer.properties";
	     
	   
	    CFEngineCorbaImpl engineImpl = new CFEngineCorbaImpl(filename, orb);
	   // org.omg.CORBA.Object obj = engineImpl._this(orb);
	   org.omg.CORBA.Object obj = 
             rootPOA.servant_to_reference(engineImpl);
      // CFEngineCorba engine = CFEngineCorbaHelper.narrow(obj);

	          
        
        //
        // Export the object reference to a file
        //
        try
        {
            String ref = orb.object_to_string(obj);
            String refFile = installdir + "/cfengine.ref";
            java.io.FileOutputStream file =
                new java.io.FileOutputStream(refFile);
            java.io.PrintWriter out = new java.io.PrintWriter(file);
            out.println(ref);
            out.flush();
            file.close();
        }
        catch(java.io.IOException ex)
        {
            System.err.println("cfengine.Server: can't write to `"
                               + ex.getMessage() + "'");
            return 1;
        }
        
        //
        // Run implementation
        //
       
        orb.run();
        
        return 0;
        
     }
     
     public static void main(String[] args) {
     
        System.out.println("start main");
       // java.util.Properties props = System.getProperties();
       // props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
       // props.put("org.omg.CORBA.ORBSingletonClass",
       //           "com.ooc.CORBA.ORBSingleton");

        int status = 0;
	    org.omg.CORBA.ORB orb = null;
	    
	    try
        {
            orb = org.omg.CORBA.ORB.init(args, null);
            status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);     
     
     }


}