/* 
 * AddNewRatingListTest.java
 * 
 * Version 1.1
 * 
 * 03/27/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */

package org.recommender.test;
 
import org.recommender.server.*;

import java.io.IOException;
import java.sql.*;
import java.util.Random;
import java.util.Vector;

/**
 * Test for setRatingList()
 *
 * @author	 <A HREF="mailto:wangyun@cs.orst.edu">Yun Wang</A>
 * @since   JDK 1.2 
 */
public class AddNewRatingListTest{

	private CFEngine engine = null;
	
	private static String ratingTableName;
    private static String userFieldName;
    private static String itemFieldName;
    private static String ratingFieldName;
    
    private float minRating = 0;
    private float maxRating = 0;
    private static Statement stmt = null;
    
    private int maxUser = 0, maxItem = 0;
    
    /**
     * Constructor
     * Create a new AddNewRatingListTest object
     * @param s
     */
    public AddNewRatingListTest(CFEngine s)
    {
  		engine = s;
  		
  		String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                + "CFServer.properties";
        
        SystemProperties.loadProperties(filename);
        
        minRating = SystemProperties.getFloat("CFServer.db.minRating", 0);
        maxRating = SystemProperties.getFloat("CFServer.db.maxRating", 0);
        
        // Load table and field names
        ratingTableName = SystemProperties.getString(
                "CFServer.db.ratingsTable", "");
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
        
        String query = "SELECT MAX(" + userFieldName + "), MAX(" + itemFieldName +
        			") FROM " + ratingTableName; 
	    
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
              conn = getConnection();
              stmt = conn.createStatement();
              rs = stmt.executeQuery(query); 
              if (rs != null && rs.next()){ 
               maxUser = rs.getInt(1);
               maxItem = rs.getInt(2);
               
           	}
        	rs.close();
        	conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
        } catch (IOException e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
      
  	} // AddNewRatingListTest()
  	
  	/**
  	 * singleItemTest1()
  	 * Add a item (existed), check value in 2 ways
  	 * 1. getRating() -- directly from cache
  	 * 2. directly from database
  	 *
  	 * @param user
  	 * @return int indicating whether passing the test or not
  	 */
  	private int singleItemTest1(int user)
  	{
  		System.out.println("Check Existed single Item for User " + user + " ...");
  		int pass = 1;
  		
  		Vector v = getItems(user);
  		int length = v.size();
  	
  		if (length <= 0) 
  		{
  			System.out.println("\t no item available.");
  			return 0;
  		} 
  		 				
  		Random r = new Random();
  		int item = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  		ItemRating rating = null;
  		
  		System.out.println("Before Add");
  		try {
  			rating = engine.getRating(user, item);
  		} catch (Exception e) {
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}	 
  		System.out.println("\tItem " + item + "\tin Cache " + rating.getRating());
  		
  		float rate;
  		if (rating.getRating() < minRating || rating.getRating() > maxRating )
  			rate = r.nextFloat() * (maxRating - 1) + 1;
  		else
  			rate = 1 + rating.getRating() / 3;
  		
  		ItemRating[] it = new ItemRating[1];
  		it[0] = new ItemRating(user, item, rate);
  		
  		System.out.println("After Add");	
  		int number = 0;
  		try {
  			number = engine.setRatingList(it);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		System.out.println(number + " rating added");
  		
  		try {
  			rating = engine.getRating(it[0].getUserID(), it[0].getItemID());
  		} catch (Exception e) {
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		 
  		System.out.println("\tItem " + rating.getItemID() + "\tin Cache  " + rating.getRating() 
  						+ "\tin DB" + getRating(it[0].getUserID(), it[0].getItemID()));
  		
  		System.out.println("");	
  		return pass; 
  	
  	} // singleItemTest1()
  	
  	/**
  	 * singleItemTest2()
  	 * Add a item (randomly), check value in 2 ways
  	 * 1. getRating() -- directly from cache
  	 * 2. directly from database
  	 *
  	 * @param user
  	 * @return int indicating whether passing the test or not
  	 */
  	private int singleItemTest2(int user)
  	{
  		System.out.println("Check Randomly Selected single Item for User " + user + " ...");
  		int pass = 1;
  		
  		Random r = new Random();
  		int item = (int) (r.nextFloat() * (maxItem - 1) + 1);
  		
  		System.out.println("Before Add");
  		
  		ItemRating rating = null;
  		try {
  			rating = engine.getRating(user, item);
  		} catch (Exception e) {
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}	 
  		
  		System.out.println("\tItem " + item + "\tin Cache " + rating.getRating());
  		
  		float rate = -1;
  		if (rating.getRating() < minRating || rating.getRating() > maxRating )
  			rate = r.nextFloat() * (maxRating - 1) + 1;
  		else
  			rate = 1 + rating.getRating() / 3;
  		
  		ItemRating[] it = new ItemRating[1];
  		it[0] = new ItemRating(user, item, rate);
  			
  		int number = 0;
  		try {
  			number = engine.setRatingList(it);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		System.out.println(number + " rating added.  After Add:");
  		
  		try {
  			rating = engine.getRating(it[0].getUserID(), it[0].getItemID());
  		} catch (Exception e) {
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		
  		System.out.println("\tItem " + it[0].getItemID() + "\tin Cache " + rating.getRating() 
  						+ "\tin DB" + getRating(it[0].getUserID(), it[0].getItemID()));
  		System.out.println("");	 
  		return pass;
  		
  	} // singleItemTest2()
  	
  	/**
  	 * tenItemTest()
  	 * Add 10 items (5 existed, 5 randomly), check values. Test in 3 ways:
  	 * 1. change 5 existed items' ratings.
  	 * 2. add ratings to 5 randomly selected items.
  	 *
  	 * @param user
  	 * @return int indicating whether passing the test or not
  	 */
  	private int tenItemTest(int user)
  	{
  		System.out.println("Check 10 Items for User " + user + " ...");
  		int pass = 1;
  		
  		int[] item = new int[10];
  		float[] rating = new float[10];
  		ItemRating[] it = new ItemRating[10];
  		
  		Random r = new Random();	
  		
  		Vector v = getItems(user);
  		int length = v.size();
  	//System.out.println("  Have " + length +" available Rated Items.");
  		if (length <= 0) 
  		{
  			System.out.println("\t no item available.");
  			return 0;
  		} 
  		
  		System.out.println("Before Add:");  		 				
  		for (int i = 0; i < item.length; i++)
  		{
  			if ( i < 5 && length > 5)
  			{	
  				item[i] = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  			} 
  			else if (i < 5 && length <= 5 && length > (i+1))
  			{
  				item[i] = ((Integer) v.get(i)).intValue();  					
  			} 
  			else 
  			{
  				item[i] = (int) (r.nextFloat() * (maxItem - 1) + 1);
  			}
  			
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(user, item[i]);
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			
  			rating[i] = getRating(user, item[i]);
  			System.out.println("\t\tItem " + item[i] + "\tin Cache " + itRating.getRating());
  			
  			float rate;
  			if (itRating.getRating() < minRating || itRating.getRating() > maxRating)
  				rate = r.nextFloat() * (maxRating - 1) + 1;
  			else
  				rate = 1 + itRating.getRating() / 3;
  				
  			it[i] = new ItemRating(user, item[i], rate);
  		} // for
  		
  		int number = 0;	
  		try {
  			number = engine.setRatingList(it);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}  			
  		System.out.println(number + " rating added. After Add: ");
  		
  		for (int i = 0; i < number; i++)
  		{
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(it[i].getUserID(), it[i].getItemID());
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	   			
  			System.out.println("\t\tItem " + itRating.getItemID() + "\tin Cache " + itRating.getRating() 
  							+ "\tin DB" + getRating(it[i].getUserID(), it[i].getItemID()));
  		}
  		System.out.println("");
  		return pass;
  			
  	} // tenItemTest()
  	
  	/**
  	 * multiUserTest1()
  	 * Test for 5 users with 2 items each:
  	 * 1. 1 existed items
  	 * 2. 1 randomly selected items
  	 *
  	 * @return int indicating whether passing the test or not
  	 */
  	private int multiUserTest1()
  	{
  		System.out.println("Check multi users, each for 2 items ...");
  		int pass = 1;
  		
  		Random r1 = new Random();
  		Random r2 = new Random();
  		
  		int[] user = new int[5];
  		ItemRating[] it = new ItemRating[10];
  		
  		System.out.println("Before Add: ");
  		for (int i = 0; i < user.length; i++)
  		{
  			user[i] = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  			  			
  			Vector v = getItems(user[i]);
  			int length = v.size();
  	
  			if (length <= 0) 
  			{
  				System.out.println("\t no item available.");
  				return 0;
  			} 
  		 	
  		 	int item = ((Integer) v.get((int) (r1.nextFloat() * length))).intValue();
  		 	ItemRating itRating = null;
  		 	try {
  		 		itRating = engine.getRating(user[i], item);
  		 	} catch (Exception e) {
  		 		System.out.println("Exception: " + e);
  				e.printStackTrace();  		 	
  		 	}
  			System.out.println("\tUser" + user[i] + "\tItem " + item + "\tin Cache " 
  								+ itRating.getRating() );
  			float rate = -1;
  			if (itRating.getRating() < minRating || itRating.getRating() > maxRating)
  				rate = r2.nextFloat() * (maxRating - 1) + 1;
  			else
  				rate = 1 + itRating.getRating() / 3;
  		 	it[i] = new ItemRating(user[i], item, rate);
  		}
  		
  		for (int i = user.length; i < it.length; i++)
  		{
  			int item = (int) (r1.nextFloat() * (maxItem - 1) + 1);
  			ItemRating itRating = null;
  			
  			try {
  		 		itRating = engine.getRating(user[i-5], item);
  		 	} catch (Exception e) {
  		 		System.out.println("Exception: " + e);
  				e.printStackTrace();  		 	
  		 	}
  			System.out.println("\tUser " + user[i- 5] + "\tItem " + item 
  								+ "\tin Cache " + itRating.getRating() );
  			float rate = -1;
  			if (itRating.getRating() < minRating || itRating.getRating() > maxRating)
  				rate = r2.nextFloat() * (maxRating - 1) + 1;
  			else
  				rate = 1 + itRating.getRating() / 3;
  				
  		 	it[i] = new ItemRating(user[i- 5], item, rate); 
  		}
  		
  		int number = 0;
  		try {
  			number = engine.setRatingList(it);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}  
  		System.out.println(number + " rating added. After Add: ");
  		
  		for (int i = 0; i < number; i++)
  		{
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(it[i].getUserID(), it[i].getItemID());
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("\tUser " + itRating.getUserID() + "\tItem " + itRating.getItemID() + "\tin Cache "  
  								+ itRating.getRating() + "\tin DB " + getRating(it[i].getUserID(), it[i].getItemID()));  
  		}
  		System.out.println();
  		return pass;			
  	
  	} // multiUserTest1()
  	
  	/**
  	 * multiUserTest2()
  	 * Test for 5 users with 10 items each:
  	 * 1. 5 existed items
  	 * 2. 5 randomly selected items
  	 *
  	 * @return int indicating whether passing the test or not
  	 */
  	private int multiUserTest2()
  	{
  		System.out.println("Check multi users, each for 10 items ...");
  		int pass = 1;
  		
  		Random r1 = new Random();
  		Random r2 = new Random();
  		
  		int[] user = new int[5];
  		ItemRating[] it = new ItemRating[50];
  		
  		System.out.println("Before Add: ");
  		for (int i = 0; i < 5; i++)
  		{
  			user[i] = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  			  			
  			Vector v = getItems(user[i]);
  			int length = v.size();
  	
  			if (length <= 0) 
  			{
  				System.out.println("\t no item available.");
  				return 0;
  			}
  			System.out.println("\tFor User " + user[i]);
  			
  			for (int k = i * 10; k < (i * 10 + 10); k++)
  			{
  		 		int item = 0;
  		 		if ( k < (i * 10 + 5) && length > 5)
  				{	
  					item = ((Integer) v.get((int) (r2.nextFloat() * length))).intValue();
  				} 
  				else if (k < (i * 10 + 5) && length <= 5 && length > (k % 10 + 1))
  				{
  					item = ((Integer) v.get(k % 10)).intValue();  					
  				} 
  				else 
  				{
  					item = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  				}
  		 		
  		 		ItemRating itRating = null;
  		 		try {
  		 			itRating = engine.getRating(user[i], item);
  		 		} catch (Exception e) {
  		 			System.out.println("Exception: " + e);
  					e.printStackTrace();  		 	
  		 		}
  				System.out.println("\t\tItem " + item + "\tin Cache " + itRating.getRating() );
  				float rate = -1;
  				if (rate < minRating || rate > maxRating)
  					rate = r1.nextFloat() * (maxRating - 1) + 1;
  				else
  					rate = 1 + rate / 3;
  		 		it[k] = new ItemRating(user[i], item, rate);
  		 	}
  		} // for
  		
  		int number = 0;
  		try {
  			number = engine.setRatingList(it);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}  
  		
  		System.out.println(number + " rating added. After Add: ");
  		for (int i = 0; i < 5; i++)
  		{
  			  			
  			for (int k = i * 10; k < (i * 10 + 10); k++)
  			{
  				ItemRating itRating = null;
  		 		try {
  		 			itRating = engine.getRating(it[k].getUserID(), it[k].getItemID());
  		 		} catch (Exception e) {
  		 			System.out.println("Exception: " + e);
  					e.printStackTrace();  		 	
  		 		}
  				System.out.println("\tUser " + itRating.getUserID() + "\tItem " + itRating.getItemID() + "\tin Cache " 
  								+ itRating.getRating() + "\tin DB " + getRating(it[k].getUserID(), it[k].getItemID()));
  			}
  		}
  		System.out.println();
  		return pass;  	
  		
  	} // multiUserTest2()
  	
  	/**
  	 * newItemUserTest()
  	 * Test for the new items and new users
  	 *
  	 * @return int indicating whether passing the test or not
  	 */
  	private int newItemUserTest()
  	{
  		System.out.println("Checking elements for new Item ...");
  		int pass = 1;
  		
  		ItemRating[] it = new ItemRating[10];
  		
  		Random r1 = new Random();
  		Random r2 = new Random();
  		
  		for (int item = maxItem + 1, k = 0; k < 5; k++, item++)
  	  	{
  	  		int u = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  	  		float rate = r2.nextFloat() * (maxRating - 1) + 1;
  	  		it[k] = new ItemRating(u, item, rate);
  	  	}
  	  	for (int user = maxUser + 1, k = 5; k < 10; k++, user++)
  	  	{
  	  		int item = (int) (r1.nextFloat() * (maxItem - 1) + 1);
  	  		float rate = r2.nextFloat() * (maxRating - 1) + 1;
  	  		it[k] = new ItemRating(user, item, rate);
  	  	}
  	  	
  	  	int number = 0;
  	  	try {
  			number = engine.setRatingList(it);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		System.out.println(number + " rating add."); 
  		
  		for ( int i = 0; i < it.length; i++)
  		{
  			ItemRating itRating = null;
  			try {
  		 		itRating = engine.getRating(it[i].getUserID(), it[i].getItemID());
  		 	} catch (Exception e) {
  		 		System.out.println("Exception: " + e);
  				e.printStackTrace();  		 	
  		 	}
  			System.out.println("\tUser " + itRating.getUserID() + " \tItem " + itRating.getItemID()
  						+ "\tin Cache " + itRating.getRating() + "\tin DB " + getRating(it[i].getUserID(), it[i].getItemID()));
  		} 
  		System.out.println();
  		return pass;
  	  	 
  	}
  	
  	/**
  	 * illegalParamTest()
  	 * Test for illegal parameters
  	 * 1. item < 1
  	 * 2. user < 1
  	 * 3. rating < minRating and rating > maxRating
  	 * 4. itemRating = null
  	 *
  	 * @return int indicating whether passing the test or not
  	 */
  	private int illegalParamTest()
  	{
  		
  	 	System.out.println("Checking the illegal elements...");
  	 	int pass = 1;
  	 	
  	 	Random r1 = new Random();
  		Random r2 = new Random();
  	    ItemRating[] it = new ItemRating[5];	  
  	 	
  	 	// item < 1
  	 	for (int item = 2, k = 0; k < 5; k++, item--)
  	  	{
  	 	 	int u = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  	  		float rating = r2.nextFloat() * (maxRating - 1) + 1;
  	 	 	it[k] = new ItemRating(u, item, rating);
  	 	}
  	 	int number = 0;
  	 	try {
  			number = engine.setRatingList(it);
  		} catch (CFIllegalListParameterException e) {
  			System.out.println("Exception: " + e);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		System.out.println(number + " rating added");
  		 
  		for (int i = 0; i < number; i++)
  	  	{
  	 		ItemRating itRating = null;
  		
  			try {
  		 		itRating = engine.getRating(it[i].getUserID(), it[i].getItemID());
  		 	} catch (CFIllegalListParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  		 		System.out.println("Exception: " + e);
  				e.printStackTrace();  		 	
  		 	}
  			System.out.println("\tUser " + it[i].getUserID() + " \tItem " + it[i].getItemID()
  						+ "\tin Cache " + itRating.getRating()
  						+ "\tin DB " + getRating(it[i].getUserID(), it[i].getItemID()));
  		} 
  		System.out.println();
  		
  		// user < 0
  		for (int user = 2, k = 0; k < 5; k++, user--)
  	  	{
  	 	 	int item = (int) (r1.nextFloat() * (maxItem - 1) + 1);
  	  		float rating = r2.nextFloat() * (maxRating - 1) + 1;
  	 	 	it[k] = new ItemRating(user, item, rating);
  	 	}
  	 	
  	 	number = 0;
  	 	try {
  			number = engine.setRatingList(it);
  		} catch (CFIllegalListParameterException e) {
  			System.out.println("Exception: " + e);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		System.out.println(number + " rating added"); 
  		
  		for (int i = 0; i < number; i++)
  	  	{
  	 		ItemRating itRating = null;
  			try {
  		 		itRating = engine.getRating(it[i].getUserID(), it[i].getItemID());
  		 	} catch (CFIllegalListParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  		 		System.out.println("Exception: " + e);
  				e.printStackTrace();  		 	
  		 	}
  			System.out.println("\tUser " + it[i].getUserID() + " \tItem " + it[i].getItemID()
  							+ "\tin Cache " + itRating.getRating()
  							+ "\tin DB " + getRating(it[i].getUserID(), it[i].getItemID()));
  		}
  		System.out.println();
  		
  		// rating < minRating
  		float rating = (minRating + 1);
  		for (int k = 0; k < 5; k++, rating -= 1)
  	 	{
  	  		int i = (int) (r1.nextFloat() * (maxItem - 1) + 1);
  	  		int u = (int) (r2.nextFloat() * (maxUser - 1) + 1);
  			it[k] = new ItemRating(u, i, rating);
  	 	}
  	 	number = 0;
  	 	try {
  			number = engine.setRatingList(it);
  		} catch (CFIllegalListParameterException e) {
  			System.out.println("Exception: " + e);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		System.out.println(number + " rating added");
  		 
  		for (int i = 0; i < number; i++)
  	  	{
  	 		ItemRating itRating = null;
  			try {
  		 		itRating = engine.getRating(it[i].getUserID(), it[i].getItemID());
  		 	} catch (CFIllegalListParameterException e) {
  				System.out.println("Exception: " + e);
  		 	} catch (Exception e) {
  		 		System.out.println("Exception: " + e);
  				e.printStackTrace();  		 	
  		 	}
  			System.out.println("\tUser " + it[i].getUserID() + " \tItem " + it[i].getItemID()
  						+ "\tin Cache " + itRating.getRating() 
  						+ "\tin DB " + getRating(it[i].getUserID(), it[i].getItemID()));
  		}
  		System.out.println();
  	 	
  	 	// rating > maxRating
  	 	rating = (maxRating + 2);
  		for (int k = 0; k < 5; k++, rating -= 1)
  	 	{
  	  		int i = (int) (r1.nextFloat() * (maxItem - 1) + 1);
  	  		int u = (int) (r2.nextFloat() * (maxUser - 1) + 1);
  			it[k] = new ItemRating(u, i, rating);
  	 	}
  	 	number = 0;
  	 	try {
  			number = engine.setRatingList(it);
  		} catch (CFIllegalListParameterException e) {
  			System.out.println("Exception: " + e);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		System.out.println(number + " rating added");
  		 
  		for (int i = 0; i < number; i++)
  	  	{
  	 		ItemRating itRating = null;
  			try {
  		 		itRating = engine.getRating(it[i].getUserID(), it[i].getItemID());
  		 	} catch (CFIllegalListParameterException e) {
  				System.out.println("Exception: " + e);
  		 	} catch (Exception e) {
  		 		System.out.println("Exception: " + e);
  				e.printStackTrace();  		 	
  		 	}
  			System.out.println("\tUser " + it[i].getUserID() + " \tItem " + it[i].getItemID()
  			//			+ "\tin Cache " + itRating.getRating()
  						+ "\tin DB " + getRating(it[i].getUserID(), it[i].getItemID()));
  		}
  		System.out.println();
  		
  		// list is null
  		it = null;
  		number = 0;
  		try {
  			number = engine.setRatingList(it);
  		} catch (CFIllegalListParameterException e) {
  			System.out.println("Exception: " + e);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		System.out.println(number + " rating added");
  		System.out.println();
  		return pass; 
  		
  	 } // illegalParamTest()
  	
  	/**
     * Get connection with database. 
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException
    {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");

      
         
        return DriverManager.getConnection(url, username, password);
        
        // We now use a connection pooling approach. The real SQL driver
        // is set in the static initializer of this class.
       // return DriverManager.getConnection("jdbc:jdc:jdcpool");
        
    } // getConnection()
    
  	/**
  	 * getRating()
  	 * Get rating from database.
  	 *
  	 * @param user
  	 * @param item
     * @return rating
     * 
     */
  	private float getRating(int user, int item)
    {
    	Connection conn = null;
    	float rating = -1;
    	String query = "SELECT " + ratingFieldName + " FROM " + ratingTableName
    				   + " WHERE " + userFieldName + " = " + user + " and "
    				   + itemFieldName + " = " + item; 
   	
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
        	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);           
            
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();
        } catch (Exception e) {           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           if (rs != null && rs.next()){ 
               rating = rs.getFloat(1);
            }
            rs.close();
        	conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        }
             
    // System.out.println(""+rating);   
        return rating;
    
    } // getRating()
    
    /**
  	 * getItems()
  	 * Get rated items for a user
     * @return Vector
     * 
     */
    private Vector getItems(int user)
    {
    	Vector items = new Vector();
    	String query = "SELECT " + itemFieldName + " FROM " + ratingTableName  + " WHERE " 
    	 				  + userFieldName + " = " + user + " order by " + itemFieldName;
    	
    	Connection conn = null;		    
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
        	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);             
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();
        } catch (Exception e) {           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           while (rs != null && rs.next()) 
           		items.add(new Integer(rs.getInt(1))); 
 			rs.close();
 			conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
        
        return items;
    } // getItems()
  	
  	/**
  	 * testSeq()
  	 */
  	public void testSeq()
  	{
  		System.out.println("############################");
  		System.out.println("Test for setRating() ...");
  		System.out.println("############################");
  		System.out.println("");
  		int pass = 1;
  		
  		Random r = new Random();
  		
  		System.out.println("Start Single Item Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = singleItemTest1(user);
			if (t < 0 )
				pass = -1;
		}
		for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = singleItemTest2(user);
			if (t < 0 )
				pass = -1;
		}
		
		System.out.println("Start 10 Items Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = tenItemTest(user);
			if (t < 0 )
				pass = -1;
		}	
		
		System.out.println("Start multi user Test ... ");
    	System.out.println("");
    	int t = multiUserTest1();
    	if (t < 0 )
			pass = -1;
				
    	t = multiUserTest2();
    	if (t < 0 )
			pass = -1;
			
    	System.out.println("Start new user and new item Test ... ");
    	System.out.println("");
    	t = newItemUserTest();
    	if (t < 0 )
			pass = -1;
			
    	System.out.println("Start illegal param Test ... ");
    	System.out.println("");
    	t = illegalParamTest();
    	if (t < 0 )
			pass = -1;	
    	
    	System.out.println();
		if (pass >= 0) {
            System.out.println("------ Pass! setRatingList() Test! ------");
		    System.err.println("------ Pass! setRatingList() Test! ------");
        } else {
            System.out.println("------ Fail! setRatingList() Test! ------");
            System.err.println("------ Fail! setRatingList() Test! ------");
        }

    	System.out.println();  			  	
  	}
  	
} // class AddNewRatingListTest