/* 
 * AddRatingTest.java
 * 
 * Version 1.1
 * 
 * 03/12/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */


package org.recommender.test;
 
import java.sql.*;
import java.util.*;
import java.io.*;
import org.recommender.server.*;

/**
 * Test for setRating()
 * @author Yun Wang
 * 
 */
public class AddRatingTest{

	private CFEngine engine = null;
	
	private static String ratingTableName;
    private static String userFieldName;
    private static String itemFieldName;
    private static String ratingFieldName;
    
    private float minRating = 0;
    private float maxRating = 0;
    private static Statement stmt = null;
    
    private int maxUser = 0, maxItem = 0;
    
    public AddRatingTest(CFEngine s)
    {
  		engine = s;
  		
  		String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                + "CFServer.properties";
        
        SystemProperties.loadProperties(filename);
        
        minRating = SystemProperties.getFloat("CFServer.db.minRating", 0);
        maxRating = SystemProperties.getFloat("CFServer.db.maxRating", 0);
        
        // Load table and field names
        ratingTableName = SystemProperties.getString(
                "CFServer.db.ratingsTable", "");
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
        
        String query = "SELECT MAX(" + userFieldName + "), MAX(" + itemFieldName +
        			") FROM " + ratingTableName; 
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
              conn = getConnection();
              stmt = conn.createStatement();
              rs = stmt.executeQuery(query); 
              if (rs != null && rs.next()){ 
               maxUser = rs.getInt(1);
               maxItem = rs.getInt(2);
               
           }
           rs.close();
           conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
        } catch (IOException e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
     
  	} // AddRatingTest()
  	
  	/**
  	 * singleItemTest1()
  	 * Add a item (existed), check value in 2 ways
  	 * 1. getRating() -- directly from cache
  	 * 2. directly from database
  	 * @return int
  	 */
  	public int singleItemTest1(int user)
  	{
  		System.out.println("Check Existed single Item for User " + user + " ...");
  		int pass = 1;
  		
  		Vector v = getItems(user);
  		int length = v.size();
  	
  		if (length <= 0) 
  		{
  			System.out.println("\t no item available.");
  			return 0;
  		} 
  		 				
  		Random r = new Random();
  		int item = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  		
  		ItemRating itRating = null;
  		System.out.println("Before Add");
  		try {
  			itRating = engine.getRating(user, item);
  		} catch (Exception e) {
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}	 
  		System.out.println("\tItem " + item + "\tin Cache " + itRating.getRating());
  		
  		float rating = -1;
  		if (itRating.getRating() < minRating || itRating.getRating() > maxRating )
  			rating = r.nextFloat() * (maxRating - 1) + 1;
  		else
  			rating = 1 + itRating.getRating() / 3;
  		
  		System.out.println("After Add");	
  		try {
  			engine.setRating(user, item, rating);
  			
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRating()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		itRating = null;
  		try {
  			itRating = engine.getRating(user, item);
  		} catch (Exception e) {
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}	 
  		 
  		System.out.println("\tItem " + itRating.getItemID() + "\tin Cache " + itRating.getRating() 
  							+ "\tin DB " + getRating(user, item));

  		System.out.println();
  		return pass;	 
  	
  	} // singleItemTest1()
  	
  	/**
  	 * singleItemTest2()
  	 * Add a item (randomly), check value in 2 ways
  	 * 1. getRating() -- directly from cache
  	 * 2. directly from database
  	 * @return int
  	 */
  	public int singleItemTest2(int user)
  	{
  		System.out.println("Check Existed single Item for User " + user + " ...");
  		int pass = 1;
  		
  		Random r = new Random();
  		int item = (int) (r.nextFloat() * (maxItem - 1) + 1);
  		
  		ItemRating itRating = null;
  		System.out.println("Before Add");
  		try {
  			itRating = engine.getRating(user, item);
  		} catch (Exception e) {
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}	 
  		System.out.println("\tItem " + item + "\tin Cache " + itRating.getRating());
  		
  		float rating = -1;
  		if (itRating.getRating() < minRating || itRating.getRating() > maxRating )
  			rating = r.nextFloat() * (maxRating - 1) + 1;
  		else
  			rating = 1 + itRating.getRating() / 3;
  		
  		System.out.println("After Add");	
  		try {
  			engine.setRating(user, item, rating);
  			
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRating()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		itRating = null;
  		try {
  			itRating = engine.getRating(user, item);
  		} catch (Exception e) {
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		System.out.println("\tItem " + itRating.getItemID() + "\tin Cache " + itRating.getRating() + "\tin DB " + getRating(user, item));

  		System.out.println("");	 
  		return pass;
  	
  	} // singleItemTest2()
  	
  	/**
  	 * tenItemTest()
  	 * Add 10 items (5 existed, 5 randomly), check values. Test in 3 ways:
  	 * 1. change 5 existed items' ratings
  	 * 2. add ratings to 5 randomly selected items
  	 * 3. change ratings to newly commited items
  	 * @return int
  	 */
  	public int tenItemTest(int user)
  	{
  		System.out.println("Check 10 Items for User " + user + " ...");
  		int pass = 1;
  		
  		int[] item = new int[10];
   		
  		Random r = new Random();
  		
  		// for 5 existed items
  		System.out.println("  For 5 existed Items ");  		
  		
  		Vector v = getItems(user);
  		int length = v.size();
  	
  		if (length <= 0) 
  		{
  			System.out.println("\t no item available.");
  			return 0;
  		} 
  		
  		System.out.println("\tBefore Add:");  		 				
  		for (int i = 0; i < 5; i++)
  		{
  			if ( i < 5 && length > 5)
  			{	
  				item[i] = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  			} 
  			else if (i < 5 && length <= 5 && length > (i+1))
  			{
  				item[i] = ((Integer) v.get(i)).intValue();  					
  			} 
  			else 
  			{
  				item[i] = (int) (r.nextFloat() * (maxItem - 1) + 1);
  			}
  			  			
  			
  			ItemRating itRating = null;
  		
  			try {
  				itRating = engine.getRating(user, item[i]);
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			
  		//	rating[i] = getRating(user, item[i]);
  			System.out.println("\t\tItem " + item[i] + "\tin Cache " + itRating.getRating());
  			
  			float rate;
  			if (itRating.getRating() < minRating || itRating.getRating() > maxRating)
  				rate = r.nextFloat() * (maxRating - 1) + 1;
  			else
  				rate = 1 + itRating.getRating() / 3;
  				
  			try {
  				engine.setRating(user, item[i], rate);
  				
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			  			
  			
  		} // for
  		
  		System.out.println("\tAfter Add:");
  		for (int i = 0; i < 5; i++)
  		{
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(user, item[i]);
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}  			
  			System.out.println("\t\tItem " + itRating.getItemID() + "\tin cache " + itRating.getRating()
  						 + "\tin DB " + getRating(user, item[i]));
  		}
  		System.out.println("");
  		
  		// for randomly selected items  		
  		System.out.println("  For 5 Randomly selected items");  		
  		System.out.println("\tBefore Add:"); 
  		for (int i = 5; i < 10; i++)
  		{ 
  			item[i] = (int) (r.nextFloat() * (maxItem - 1) + 1);
  			ItemRating itRating = null;
  		
  			try {
  				itRating = engine.getRating(user, item[i]);
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			  			
  			System.out.println("\t\tItem " + item[i] + "\tin Cache " + itRating.getRating());
  			
  			float rate;
  			if (itRating.getRating() < minRating || itRating.getRating() > maxRating)
  				rate = r.nextFloat() * (maxRating - 1) + 1;
  			else
  				rate = 1 + itRating.getRating() / 3;
  			
  			try {
  				engine.setRating(user, item[i], rate);
  				
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			
  		}
  		System.out.println("\tAfter Add:");
  		for (int i = 5; i < 10; i++)
  		{
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(user, item[i]);
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}  			
  			System.out.println("\t\tItem " + itRating.getItemID() + "\tin cache " + itRating.getRating()
  						 + "\tin DB " + getRating(user, item[i]));
  		}
  		System.out.println("");  		
  		
  		// change ratings to newly commited items
  		System.out.println("  Change ratings to  5 newly commited items");
  		
  		for (int i = 0; i < 10; i += 2)
  		{
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(user, item[i]);
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}  		
  			float rate = 1 + itRating.getRating() / 3;
  			
  			try {
  				engine.setRating(user, item[i], rate);
  				
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			
  		}  
  		System.out.println("\tAfter Add");
  		for (int i = 0; i < 10; i += 2)
  		{
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(user, item[i]);
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}  			
  			System.out.println("\t\tItem " + itRating.getItemID() + "\tin cache " + itRating.getRating() 
  							+ "\tin DB " + getRating(user, item[i]));
  		}
  		System.out.println(""); 
  		return pass;   		
  	
  	} // tenItemTest()
  	
  	/**
  	 * multiUserTest()
  	 * Test for 5 users 10 items:
  	 * 1. 5 existed items
  	 * 2. 5 randomly selected items
  	 * 3. change ratings for above 10 items
  	 * @return int
  	 */
  	public int multiUserTest()
  	{
  		System.out.println("Check multi users, each for 20 items ...");
  		int pass = 1;
  		
  		Random r1 = new Random();
  		Random r2 = new Random();
  		
  		int[] user = new int[5];
  		int[][] item = new int[5][10];
  		float[][] rating = new float[5][10];
  		
  		System.out.println("  existed 5 items and randomly selected 5 items "); 
  		System.out.println("\tBefore Remove: ");		
  		
  		// for 1 and 2 tasks 		
  		for (int i = 0; i < 5; i++)
  		{
  			user[i] = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  			System.out.println("\t  For User " + user[i]);
  			
  			
  			Vector v = getItems(user[i]);
  			int length = v.size();
  		
  			if (length <= 0) 
  			{
  				System.out.println("\t no item available.");
  				return 0;
  			} 
  			
  			
  			for (int k = 0 ; k < 10; k++)
  			{
  				if ( k < 5 && length > 5)
  				{	
  					item[i][k] = ((Integer) v.get((int) (r2.nextFloat() * length))).intValue();
  				} 
  				else if (k < 5 && length <= 5 && length > (k+1))
  				{
  					item[i][k] = ((Integer) v.get(k)).intValue();					
  				} 
  				else 
  				{
  					item[i][k] = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  				}
  			  				
  				ItemRating itRating = null;
  				try {
  					 itRating = engine.getRating(user[i], item[i][k]);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}	 
  				rating[i][k] = getRating(user[i], item[i][k]);
  				System.out.println("\t\tItem " + item[i][k] + "\tin cache " + itRating.getRating() + "\tin DB " + rating[i][k] );
  				
  				float r = -1;
  				if (itRating.getRating() < minRating || itRating.getRating() > maxRating)
  					r = r1.nextFloat() * (maxRating - 1) + 1;
  				else
  					r = 1 + itRating.getRating() / 3;
  				
  				try {
  					engine.setRating(user[i], item[i][k], r);
  					
  				} catch (Exception e) {
  					pass = -1;
  					System.out.println("Error Testing for setRating()");
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}  			
  									
  			} 
  			
  			// for randomly selected items
  			/*for (int k = 5 ; k < 10; k++)
  			{
  				item[i][k] = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  				
  				
  				float r = -1;
  				try {
  					r = engine.getRating(user[i], item[i][k]);  					
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}	 
  			
  				rating[i][k] = getRating(user[i], item[i][k]);
  				System.out.println("\t\tItem " + item[i][k] + "\tin cache " + r + "\tin DB " + rating[i][k] );
  				
  				if (rating[i][k] < minRating || rating[i][k] > maxRating)
  					rating[i][k] = r1.nextFloat() * (maxRating - 1) + 1;
  				else
  					rating[i][k] = 1 + rating[i][k] / 3;
  					
  				try {
  					engine.setRating(user[i], item[i][k], rating[i][k]);
   				} catch (Exception e) {
  					pass = -1;
  					System.out.println("Error Testing for setRating()");
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				try {
  					rating[i][k] = engine.getRating(user[i], item[i][k]);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}  	  	
  			} 	*/	
  				
  		
  		} // for
  		
  		System.out.println("\tAfter Add: ");	
  		
  		// print out the 1st, 2nd tasks' results and start 3rd task
  		for (int i = 0; i < 5; i++)
  		{
  			System.out.println("\t  For User " + user[i]);
  			for (int k = 0; k < 10; k++)
  			{
  				ItemRating itRating = null;
  				try {
  					 itRating = engine.getRating(user[i], item[i][k]);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}	 
  				
  				System.out.println("\tItem " + itRating.getItemID() + "\tin cache " + itRating.getRating() 
  								+ "\tin DB " + getRating(user[i], item[i][k]));	
  				
  				float rate;
  				if (itRating.getRating() < minRating || itRating.getRating() > maxRating)
  					rate = r1.nextFloat() * (maxRating - 1) + 1;
  				else
  					rate = 1 + itRating.getRating() / 3;
  				  				
  				try {
  					engine.setRating(user[i], item[i][k], rate);
  				} catch (Exception e) {
  					pass = -1;
  					System.out.println("Error Testing for setRating()");
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				 	
  			}
  		
  		}
  		
  		
  		// for the 3rd task
  		System.out.println("");
  		System.out.println("change ratings again for above 10 items");	
  		System.out.println("\tAfter change again:");	
  		for (int i = 0; i < 5; i++)
  		{
  			System.out.println("\t  For User " + user[i]);
  			for (int k = 0; k < 10; k++)
  			{
  				ItemRating itRating = null;
  				try {
  					 itRating = engine.getRating(user[i], item[i][k]);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}	 
  				
  				System.out.println("\tItem " + itRating.getItemID() + "\tin cache " + itRating.getRating() 
  								+ "\tin DB " + getRating(user[i], item[i][k]));	
  						
  			}  		
  		} 		
  		System.out.println("");
  		return pass;
  	
  	} // multiUserTest()
  	
  	/**
  	 * newItemUserTest()
  	 * Test for the new items and new users
  	 * @return int
  	 */
  	public int newItemUserTest()
  	{
  		System.out.println("Checking elements for new Item ...");
  		int pass = 1;
  		
  		Random r1 = new Random();
  		Random r2 = new Random();
  		
  		int[] u = new int[5];
  		float[] rating = new float[5];
  		
  		for (int item = maxItem + 1, k = 0; k < 5; k++, item++)
  	  	{
  	  		u[k] = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  	  		rating[k] = r2.nextFloat() * (maxRating - 1) + 1;
  	  		
  	  		try {
  	  			engine.setRating( u[k], item, rating[k]);
  	  		} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  		}
  		
  		for (int item = maxItem + 1, k = 0; k < 5; k++, item++)
  		{	
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(u[k], item);  					
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("  Rated for User " + u[k] + "\tItem " + item + "\tin cache " + itRating.getRating()
  								+ "\tin DB " + getRating(u[k], item));
  	  			
  	 	}
  		
  		
  		System.out.println("Checking elements for new User ...");
  		
  		int[] i = new int[5];
  		
  		for (int user = maxUser + 1, k = 0; k < 5; k++, user++)
  	 	{
  	  		i[k] = (int) (r1.nextFloat() * (maxItem - 1) + 1);
  	  		rating[k] = r2.nextFloat() * (maxRating - 1) + 1;
  	  		
  	  		try {
  	  			engine.setRating(user, i[k], rating[k]);
  	  		} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  		}
  		
  		for (int user = maxUser + 1, k = 0; k < 5; k++, user++)
  	 	{	
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(user, i[k]);  					
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("  Rated for User " + user + "\tItem " + i[k] + "\tin cache " + itRating.getRating()
  								+ "\tin DB " + getRating(user, i[k]));
  			
  	  	}
  	  	System.out.println();
  	  	return pass;
  	
  	}
  	
  	/**
  	 * illegalParamTest()
  	 * Test for illegal parameters
  	 * 1. item < 1
  	 * 2. user < 1
  	 * 3. rating < minRating and rating > maxRating
  	 * @return int 
  	 */
  	public int illegalParamTest()
  	{
  		 System.out.println("Checking the illegal elements...");
  		 int pass = 1;
  		 
  		 Random r1 = new Random();
  		 Random r2 = new Random();
  	 	 
  	 	// test for itemID < 1
  	  	for (int item = 0, k = 0; k < 5; k++, item--)
  	  	{
  	  		int u = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  	  		float rating = r2.nextFloat() * (maxRating - 1) + 1;
  	  		
  	  		try {
  	  			engine.setRating(u, item, rating);
  	  		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(u, item);  					
  			
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);  					
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			System.out.println("User " + u + "\tItem " + item 
  			//					+ "\tin cache " + itRating.getRating()
  								+ "\tin DB " + getRating(u, item));
  	 	}
  	 	
  		// user < 1
  		for (int user = 0, k = 0; k < 5; k++, user--)
  	 	{
  	  		int i = (int) (r1.nextFloat() * (maxItem - 1) + 1);
  	  		float rating = r2.nextFloat() * (maxRating - 1) + 1;
  	  		
  	  		try {
  	  			engine.setRating(user, i,  rating);
  	  		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(user, i);  					
  			
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);  					
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			System.out.println("User " + user + "\tItem " + i 
  			//					+ "\tin cache " + itRating.getRating()
  								+ "\tin DB " + getRating(user, i));
  	  	}
  	  	
  	  System.out.println("rating checking");	
  	  	// rating < minRating
  	  	float rating = (minRating - (float) 0.1);
  	  	for (int k = 0; k < 5; k++, rating -= 1)
  	 	{
  	  		int i = (int) (r1.nextFloat() * (maxItem - 1) + 1);
  	  		int u = (int) (r2.nextFloat() * (maxUser - 1) + 1);
  	  		
  	  		try {
  	  			engine.setRating(u, i, rating);
  	  		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  	  		} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(u, i);  					
  							
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			System.out.println("User " + u + "\tItem " + i + "\t added " + rating 
  			//					+ "\tin cache " + itRating.getRating()
  								+ "\tin DB " + getRating(u, i));
  	  	}
  	  	
  	  	// rating > maxRating
  	  	rating = (float) 5.1;
  	  	for (int k = 0; k < 5; k++, rating += 1)
  	 	{
  	  		int i = (int) (r1.nextFloat() * (maxItem - 1) + 1);
  	  		int u = (int) (r2.nextFloat() * (maxUser - 1) + 1);
  	  		
  	  		try {
  	  			engine.setRating(u, i, rating);
  	  		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  	  		} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			
  			ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(u, i);  							 					
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			System.out.println("User " + u + "\tItem " + i + "\t added " + rating 
  			//					+ "\tin cache " + itRating.getRating()
  								+ "\tin DB " + getRating(u, i));
  	  	}
  	  	System.out.println();
  	  	return pass;
  	
  	} // illegalParamTest()
  	
  	/**
     * Get connection with database. 
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException
    {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");      
         
        return DriverManager.getConnection(url, username, password);
        
        // We now use a connection pooling approach. The real SQL driver
        // is set in the static initializer of this class.
       // return DriverManager.getConnection("jdbc:jdc:jdcpool");
    } // getConnection()
    
  	/**
  	 * getRating()
  	 * Get rating from database.
     * @return rating
     * 
     */
  	private float getRating(int user, int item)
    {
    	float rating = -1;
    	String query = "SELECT " + ratingFieldName + " FROM " + ratingTableName
    				   + " WHERE " + userFieldName + " = " + user + " and "
    				   + itemFieldName + " = " + item; 
   
   		Connection conn = null;	
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
        	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
              // precompileSQLStatements(conn);          
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();    
        } catch (Exception e) {           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           if (rs != null && rs.next()){ 
               rating = rs.getFloat(1);
            }
            rs.close();
        	conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
    // System.out.println(""+rating);   
        return rating;
    
    } // getRating()
    
    /**
  	 * getItems()
  	 * Get rated items for a user
     * @return Vector
     * 
     */
    private Vector getItems(int user)
    {
    	Vector items = new Vector();
    	String query = "SELECT " + itemFieldName + " FROM " + ratingTableName  + " WHERE " 
    	 				  + userFieldName + " = " + user + " order by " + itemFieldName;
    	
    	Connection conn = null;			    
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
       		conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
              // precompileSQLStatements(conn);          
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();    
        } catch (Exception e) {
            System.out.println("Exception:" + e);
            e.printStackTrace();
        }
        
        try {
           while (rs != null && rs.next()) 
           		items.add(new Integer(rs.getInt(1))); 
 			rs.close();
 			conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
        
        return items;
    } // getItems()
  	
  	/**
  	 * testSeq()
  	 */
  	public void testSeq()
  	{
  		System.out.println("############################");
  		System.out.println("Test for setRating() ...");
  		System.out.println("############################");
  		System.out.println("");
  		int pass = 1;
  		
  		Random r = new Random();
  		
  		System.out.println("Start Single Item Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = singleItemTest1(user);
			if (t < 0 )
				pass = -1;
		}		  	
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = singleItemTest2(user);
			if (t < 0 )
				pass = -1;
		}
		
		System.out.println("Start 10 Items Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = tenItemTest(user);
			if (t < 0 )
				pass = -1;
		}	
		
		System.out.println("Start multi user Test ... ");
    	System.out.println("");
    	int t = multiUserTest();
    	if (t < 0 )
			pass = -1;
    	
    	System.out.println("Start new user and new item Test ... ");
    	System.out.println("");
    	t = newItemUserTest();
    	if (t < 0 )
			pass = -1;
    	
    	System.out.println("Start illegal param Test ... ");
    	System.out.println("");
    	t = illegalParamTest();
    	if (t < 0 )
			pass = -1;
    	
    	System.out.println();
		if (pass >= 0){
			System.out.println("------ Pass! setRating() Test! ------");
            System.err.println("------ Pass! setRating() Test! ------");
        } else {
            System.out.println("------ Fail! setRating() Test! ------");
			System.err.println("------ Fail! setRating() Test! ------");
        }
         System.out.println();
  	} // testSeq()

} // class AddRatingTest
