/*
 * CFOperation.java
 *
 * Version 1.1
 *
 * 04/14/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */


package org.recommender.test;

import org.recommender.server.*;

import java.sql.*;
import java.util.Random;
import java.util.Vector;

/**
 * Operations for a client.
 * A thread simulate a client performing one operation or a sequence of operations at a time.
 *
 * @author	 <A HREF="mailto:wangyun@cs.orst.edu">Yun Wang</A>
 * @since   JDK 1.2
 */
public class CFOperation implements Runnable {
    private CFEngine engine = null;
	
	// These fields hold the names of relational database table columns 
    private static String ratingTableName;
    private static String userFieldName;
    private static String itemFieldName;
    private static String ratingFieldName;
    private static String itemTypeTableName;
    private static String typeFieldName;

    private float minRating = 0;
    private float maxRating = 0;
    private static Statement stmt = null;

    private int maxUser = 0, maxItem = 0;
    private int maxType = 0;

    private int u = 0;      // thread representing a client
    private int op = 0;		// operation number

    private String threadName;
	
	/**
	 * Constructor
	 * Create a new CFOperation object for a thread.
	 * @param cfengine
	 * @param name
	 */
    public CFOperation(CFEngine cfengine, String name) {
        engine = cfengine;
        threadName = name;
        String installdir = System.getProperty("cf.path", ".");
        String filename = installdir + "/lib/"
                + "CFServer.properties";

        SystemProperties.loadProperties(filename);

        // Load table and field names
        itemTypeTableName = SystemProperties.getString("CFServer.db.itemTypeTable", "");
        typeFieldName = SystemProperties.getString("CFServer.db.typeField", "");
        ratingTableName = SystemProperties.getString(
                "CFServer.db.ratingsTable", "");
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");

        minRating = SystemProperties.getFloat("CFServer.db.minRating", 0);
        maxRating = SystemProperties.getFloat("CFServer.db.maxRating", 0);

        String query = "SELECT MAX(" + userFieldName + "), MAX(" + itemFieldName +
                ") FROM " + ratingTableName;

        Connection conn = null;
        ResultSet rs = null;
        try {
            conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);
            if (rs != null && rs.next()) {
                maxUser = rs.getInt(1);
                maxItem = rs.getInt(2);
            }
            rs.close();
            // conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
        }

        query = "SELECT MAX(" + typeFieldName + ") FROM " + itemTypeTableName;
        try {
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);
            if (rs != null && rs.next()) {
                maxType = rs.getInt(1);
            }
            rs.close();
            conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }

        } catch (Exception e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        //  System.out.println("maxUser = " + maxUser + "\tmaxItem = " + maxItem);
    } // CFOperation()

    /**
     * Perform an operation for a user
     *
     * @param user
     * @param operation
     *
     */
    public void doDBop(int user, int operation) {
        u = user;
        op = operation;
    }

    /**
     * Checking if the thread is doing some operations
     *
     * @return true or false
     *
     */ 
    public boolean isBusy() {
        return u != 0;
    }
    

	/**
     * Run the thread
     *
     */
    public void run() {
        while (true) {
            while (u == 0) {
                try {
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                    System.out.println(e);
                }
            }
            switch (op) {
                case 0:
                    getRating(u);
                    break;
                case 1:
                    addRating(u);
                    break;
                case 2:
                    removeRating(u);
                    break;
                case 3:
                    addNewRatingList(u);
                    break;
                case 4:
                    getPredictedRating(u);
                    break;
                case 5:
                    getRecommendations(u);
                    break;
                case 6:
                    getRecommendationsByType(u);
                    break;
                case 7:
                    getPredictedRatingList(u);
                    break;
                  case 8:
                    getUserRatingList(u);
                    break;
                case 9:
                    getItemRatingList();
                    break;
              case 10:
                    testSeq1(u);
                    break;
                case 11:
                    testSeq2(u);
                    break;
                case 12:
                    testSeq3(u);
                    break;
                case 13:
                    addRating1(u);
                    break;
                case 14:
                    removeRating1(u);
                    break;
                case 15:
                    getNextUserId();
                    break;
                case 16:
                    getNextItemId();
                    break;

            }
            u = 0;
        } // while

    } // run()
	
	/**
     * Get rating for a specific user.  Item is randomly generated.
     *
     * @param user
     *
     */ 
    private void getRating(int user) {

        // System.out.println("Tread " + threadName + "\tUser" + user + "\top 0\tgetRating()");
        Random r = new Random();
        int item = (int) (r.nextFloat() * (maxItem - 1) + 1);
        ItemRating rating = null;
        try {
            rating = engine.getRating(user, item);
        } catch (Exception e) {
            System.out.println("Error Testing for getRating()");
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        System.out.println("Tread " + threadName +"\tUser" + user + "\tItem " + item + "\tgetRating(): " + rating.getRating());


    } // getRating()
	
	/**
     * Add rating of a randomly selected item to a specific user
     *
     * @param user
     *
     */ 
    private void addRating(int user) {

        Random r = new Random();
        ItemRating rating = null;
        int item = (int) (r.nextFloat() * (maxItem - 1) + 1);
 		// System.out.println("Thread " + threadName +"\tUser" + user + " item " + item + "\top 1\taddRating()");
        try {
            rating = engine.getRating(user, item);
        } catch (Exception e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }

        float rate = -1;
        if (rating.getRating() < minRating || rating.getRating() > maxRating)
            rate = r.nextFloat() * (maxRating - 1) + 1;
        else
            rate = 1 + rating.getRating() / 3;
        try {
            engine.setRating(user, item, rate);

        } catch (Exception e) {
            System.out.println("Error Testing for setRating()");
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
      System.out.println("Thread " + threadName + "\tUser" + user + "\top 1\taddRating() finished");
  //      System.out.println("\tUser " + user + "\tItem " + item + "\tBefore setRating(): " + rating.getRating()
  //              + "\tAfter: " + rate);

    } // addRating()
	
	
	/**
     * Remove rating of a randomly selected item from a specific user
     *
     * @param user
     *
     */ 
    private void removeRating(int user) {
     // System.out.println("Thread " + threadName +"\tUser" + user + "\top 2\tremoveRating()");
        Random r = new Random();
        ItemRating rating = null;
        int item = (int) (r.nextFloat() * (maxItem - 1) + 1);
        try {
            engine.removeRating(user, item);

        } catch (Exception e) {
            System.out.println("Error Testing for setRating()");
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        try {
            rating = engine.getRating(user, item);
        } catch (Exception e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
    System.out.println("Thread " + threadName +"\tUser" + user + "\top 2\tremoveRating() finished");
  //      System.out.println("\tUser " + user + "\tItem " + item + "\tAfter removeRating(): " + rating.getRating());

    } // removeRating()
	
	
	/**
     * Add ratings of a seauence of randomly selected items to a specific user
     *
     * @param user
     *
     */ 
    private int addRating1(int user) {
    //  System.out.println("Thread " + threadName +"\tUser" + user + "\top 13\taddRating1()");
        int pass = 1;
        ItemRating itRating = null;
        Random r = new Random();

        for (int i = 0; i < 5; i++) {

            int item = (i + 1) * 5;
            try {
                itRating = engine.getRating(user, item);
            } catch (Exception e) {
                System.out.println("Exception: " + e);
                e.printStackTrace();
            }
            float rate;
            if (itRating.getRating() < minRating || itRating.getRating() > maxRating)
                rate = r.nextFloat() * (maxRating - 1) + 1;
            else
                rate = 1 + itRating.getRating() / 3;
            itRating = null;
            try {
                engine.setRating(user, item, rate);
                itRating = engine.getRating(user, item);
            } catch (Exception e) {
                pass = -1;
                System.out.println("Exception: " + e);
                e.printStackTrace();
            }
       //     System.out.println("\tUser " + user + "\tItem " + item + "\taddRating()1: " + rate
       //             + "\tAfter : " + itRating.getRating());
        }
        if (pass > 0)
            System.out.println("Thread " + threadName + "\tUser " + user + " added rating of items 5,10,15,20,25.");
        else
           System.out.println("Thread " + threadName + "\tUser " + user + " failed to add rating of items 5,10,15,20,25.");
        return pass;

    } // addRating1()
	
	
	/**
     * Remove ratings of a sequence of randomly selected items to a specific user
     *
     * @param user
     *
     */ 
    private int removeRating1(int user) {
     // System.out.println("Thread " + threadName +"\tUser" + user + "\top 14\tremoveRating1()");
        int pass = 1;
        ItemRating[] rating = new ItemRating[5];

        for (int i = 0; i < 5; i++) {
            int item = (i + 1) * 5;
            ItemRating itRating = null;
            try {
                rating[i] = engine.getRating(user, item);
                engine.removeRating(user, (i + 1) * 5);
                itRating = engine.getRating(user, item);
            } catch (Exception e) {
                pass = -1;
                System.out.println("Exception: " + e);
                e.printStackTrace();
            }
      //      System.out.println("\tUser " + user + "\tItem " + item + "\tBefore removeRating()1: "
      //              + rating[i].getRating() + "\tAfter : " + itRating.getRating());
        }
        if (pass > 0)
            System.out.println("Thread " + threadName +"\tUser " + user + " removed rating of items 5,10,15,20,25.");
        else
            System.out.println("Thread " + threadName +"\tUser " + user + " failed to remove rating of items 5,10,15,20,25.");
        return pass;

    } // removeRating1()
	
	/**
     * Add an array of ratings to a specific user. Items and ratings are randomly generated.
     *
     * @param user
     *
     */ 
    private void addNewRatingList(int user) {
     //System.out.println("Thread " + threadName +"\tUser" + user + "\top 3\tsetRatingList()");
        int[] item = new int[10];
        ItemRating[] rating = new ItemRating[10];
        ItemRating[] it = new ItemRating[10];
        Random r = new Random();

        Vector v = getItems(user);
        int length = v.size();

        if (length <= 0) {
            System.out.println("\t no item available.");
            return;
        }

        for (int i = 0; i < 10; i++) {
            if (i < 5 && length > 5) {
                item[i] = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
            } else if (i < 5 && length <= 5 && length > (i + 1)) {
                item[i] = ((Integer) v.get(i)).intValue();
            } else {
                item[i] = (int) (r.nextFloat() * (maxItem - 1) + 1);
            }


            try {
                rating[i] = engine.getRating(user, item[i]);
            } catch (Exception e) {
                System.out.println("Exception: " + e);
                e.printStackTrace();
            }

            float rate = -1;
            if (rating[i].getRating() < minRating || rating[i].getRating() > maxRating)
                rate = r.nextFloat() * (maxRating - 1) + 1;
            else
                rate = 1 + rating[i].getRating() / 3;

            it[i] = new ItemRating(user, item[i], rate);

        } // for
        int number = 0;
        try {
            number = engine.setRatingList(it);
        } catch (Exception e) {
            //pass = -1;
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        System.out.println("Thread " + threadName +"\tActually Added " + number + " NewRatingList out of 10 for user " + user);
     /*   for (int i = 0; i < it.length; i++) {
            ItemRating rate = null;
            try {
                rate = engine.getRating(it[i].getUserID(), it[i].getItemID());
            } catch (Exception e) {
                System.out.println("Exception: " + e);
                e.printStackTrace();
            }
            System.out.println("\tUser " + user + "\tItem " + item[i] + "\tBefore addNewRatingList(): "
                    + rating[i].getRating() + "\tAfter add: " + rate.getRating());

        }
        System.out.println("");*/

    }

    /**
     * Generate prediction to a randomly selected item for a specific user
     *
     * @param user
     *
     */ 
    private void getPredictedRating(int user) {
		//System.out.println("Thread " + threadName +"\tUser" + user + "\top 4\tgetPredictedRating()");
        Random r = new Random();
        int item = (int) (r.nextFloat() * (maxItem - 1) + 1);
        ItemPrediction pred = null;
        try {
            pred = engine.getPredictedRating(user, item);

        } catch (Exception e) {
            System.out.println("Error Testing for getPredictedRating()");
            System.out.println("Exception: " + e);
            e.printStackTrace();

        }
       System.out.println("Thread " + threadName +"\tUser" + user + "\top 4\tgetPredictedRating() finished");
  

    }
	
	/**
     * Generate a list of predictions to randomly selected items for a specif user. 
     *
     * @param user
     *
     */ 
    private void getPredictedRatingList(int user) {
    	// System.out.println("Thread " + threadName +"\tUser" + user + "\top 7\tgetPredictedRatingList()");
        Random r = new Random();
        int length = r.nextInt(30);
        int[] item = new int[length];
        ItemPrediction[] it = new ItemPrediction[length];

        for (int i = 0; i < length; i++) {
            item[i] = (int) (r.nextFloat() * (maxItem - 1) + 1);
        }

        try {
            it = engine.getPredictedRatingList(user, item);

        } catch (Exception e) {
            System.out.println("Error Testing for getPredictedRating()");
            System.out.println("Exception: " + e);
            e.printStackTrace();

        }
        System.out.println("Thread " + threadName +"\tTotal " + it.length + " out of " + length + " predictions.");
       /* for (int i = 0; i < it.length; i++) {
            System.out.println("\tUser " + user + "\tItem " + item[i]
                    + "\tPrediction: " + it[i].getPrediction());
        }*/
    }
	
	/**
     * Generate recommendations for a specific user
     *
     * @param user
     *
     */ 
    private void getRecommendations(int user) {
     // System.out.println("Thread " + threadName +"\tUser" + user + "\top 5\tgetRecommendations()");
        Random r = new Random();
        int number = r.nextInt(50);
        int offset = r.nextInt(300);

        ItemPrediction[] itemPred = null;
        try {
            itemPred = engine.getRecommendations(user, number, offset);

        } catch (Exception e) {
            System.out.println("Error Testing for getPredictedRating()");
            System.out.println("Exception: " + e);
            e.printStackTrace();

        }
        System.out.println("Thread " + threadName+"\tTotal " + itemPred.length + " predictions from Offset "
                + offset + " for User " + user);
     /*   if (itemPred.length >= 1) {
            for (int i = 0; i < itemPred.length; i++) {
                System.out.println("\tUser " + user + "\tItem " + itemPred[i].getItemID()
                        + "\tRecommendation " + itemPred[i].getPrediction());
            }
        } else {
            System.out.println("Server returned no recommendations");
        }*/

    }
	
	/**
     * Generate recommendations to a randomly selected category for a specific user
     *
     * @param user
     *
     */ 
    private void getRecommendationsByType(int user) {
     //System.out.println("Thread " + threadName +"\tUser" + user + "\top 6\tgetRecommendationsByType()");
        Random r = new Random();
        int number = r.nextInt(50);
        int offset = r.nextInt(50);
        int type = r.nextInt(maxType) + 1;

        ItemPrediction[] itemPred = null;
        try {
            itemPred = engine.getRecommendationsByType(user, number, offset, type);

        } catch (Exception e) {
            System.out.println("Error Testing for getRecommendationsByType(). Type is "+ type);
            System.out.println("Exception: " + e);
            e.printStackTrace();

        }
        System.out.println("Thread " + threadName +"\tTotal " + itemPred.length + " predictions of Type "
                + type + " from Offset " + offset + " for User " + user);
       /* if (itemPred.length >= 1) {
            for (int i = 0; i < itemPred.length; i++) {
                System.out.println("\tUser " + user + "\tType " + type + "\tItem " + itemPred[i].getItemID()
                        + "\tRecommendation " + itemPred[i].getPrediction());
            }
        } else {
            System.out.println("Server returned no recommendations");
        }*/
    }
	
	/**
     * Provide a list of rated items for a specific user
     *
     * @param user
     *
     */ 
    private void getUserRatingList(int user) {
      //System.out.println("Thread " + threadName +"\tUser" + user + "\top 8\tgetUserRatingList()");
        ItemRating[] itemArr = null;

        try {
            itemArr = engine.getUserRatingList(user);
        } catch (Exception e) {
            System.out.println("Error Testing for getUserRatingList()");
            System.out.println("Exception:" + e);
            e.printStackTrace();
        }
        if (itemArr != null)
        	System.out.println("Thread " + threadName +"\tUser " + user + " getUserRatingList(). Total " + itemArr.length + " items rated");
        else
        	System.out.println("Thread " + threadName +"\tNull! User " + user + " getUserRatingList().");
        /*  for (int i = 0; i < itemArr.length; i++)
          {
              System.out.println("\tUser " + user + "\trated Item " + itemArr[i]);
          }*/

    }
	
	/**
     * Provided a list of users that rated a randomly selected item.
     *
     */ 
    private void getItemRatingList() {
    	// System.out.println("Thread " + threadName +"\top 9\tgetItemRatingList()");
        Random r = new Random();
        int item = r.nextInt(maxItem) + 1;
        ItemRating[] userArr = null;

        try {
            userArr = engine.getItemRatingList(item);
        } catch (Exception e) {
            System.out.println("Error Testing for getItemRatingList()");
            System.out.println("Exception:" + e);
            e.printStackTrace();
        }
        System.out.println("Thread " + threadName +"\tItem " + item + " getItemRatingList(). Total " + userArr.length + " users rated");
        /*  for (int i = 0; i < userArr.length; i++)
          {
              System.out.println("\tItem " + item + "\trated by User " + userArr[i]);
          }*/

    }
	
	/**
     * Generate a new user ID.
     *
     */ 
    private void getNextUserId()
    {
    	//System.out.println("Thread " + threadName +"\top 15\tgetNextUserId()");
        int id = -1;
    	try {
    		id = engine.getNextUserId();
    	} catch (Exception e) {
            System.out.println("Error Testing for getItemArray()");
            System.out.println("Exception:" + e);
            e.printStackTrace();
        }
        System.out.println("Thread " + threadName +"\tUser ID is " + id);

    }
	
	/**
     * Generate a new item ID.
     *
     */ 
    private void getNextItemId()
    {  
    	//System.out.println("Thread " + threadName +"\top 16\tgetNextItemId()");
    	int id = -1;
    	try {
    		id = engine.getNextItemId();
    	} catch (Exception e) {
            System.out.println("Error Testing for getItemArray()");
            System.out.println("Exception:" + e);
            e.printStackTrace();
        }
        System.out.println("Thread " + threadName +"\tItem ID is " + id);

    }
	
	/**
     * A sequence of operations a specific user performs
     *
     * @param user
     *
     */ 
    private void testSeq1(int user) {
       //System.out.println("Thread " + threadName +"\tUser" + user + "\top 10\ttestSeq1()");
      
        getRecommendations(user);
        getRating(user);
        getPredictedRatingList(user);
        addNewRatingList(user);
        removeRating(user);
        removeRating(user);

   //  System.out.println("2nd recommendation in testSeq1");
       getRecommendations(user);
       System.out.println("Thread " + threadName +"\tUser" + user + "\top 10\ttestSeq1() finished");
    }
	
	
	/**
     * A sequence of operations a specific user performs
     *
     * @param user
     *
     */ 
    private void testSeq2(int user) {
     	// System.out.println("Thread " + threadName +"\tUser" + user + "\top 11\ttestSeq2()");
    	getUserRatingList(user);
        getRecommendationsByType(user);
        addRating(user);
        removeRating(user);
        getPredictedRatingList(user);
        removeRating(user);
        addRating(user);
    
    	System.out.println("Thread " + threadName +"\tUser" + user + "\top 11\ttestSeq2() finished");
    }

	/**
     * A sequence of operations a specific user performs
     *
     * @param user
     *
     */ 
    private void testSeq3(int user) {
      	//System.out.println("Thread " + threadName +"\tUser" + user + "\top 12\ttestSeq3()");
    	getPredictedRating(user);
        getItemRatingList();
        removeRating(user);
        removeRating(user);
        getRecommendationsByType(user);
        getRating(user);
        getRecommendations(user);
        addRating(user);
        addRating(user);
   
        System.out.println("Thread " + threadName +"\tUser" + user + "\top 12\ttestSeq3() finished");
    }

    /**
     * Get connection with database.
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     */
    private Connection getConnection() throws SQLException {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");

        return DriverManager.getConnection(url, username, password);
    } // getConnection()

    /**
     * Get rating from database.
     *
     * @param user
     * @param item
     * @return rating
     *
     */
    private float getDBRating(int user, int item) {
        float rating = -1;
        String query = "SELECT " + ratingFieldName + " FROM " + ratingTableName
                + " WHERE " + userFieldName + " = " + user + " and "
                + itemFieldName + " = " + item;

        Connection conn = null;
        ResultSet rs = null;
        // Establish connection with databases
        try {
            conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);
            if (rs != null && rs.next()) {
                rating = rs.getFloat(1);
            }
            rs.close();
            conn.close();
        } catch (Exception e) {

            System.out.println("Exception: " + e);
            e.printStackTrace();
        }


        return rating;

    } // getDBRating()

    /**
     * Get rated items for a user
     *
     * @param userID
     * @return	Vector
     *
     */
    private Vector getItems(int userID) {
        Vector items = new Vector();
        String query = "SELECT " + itemFieldName + " FROM " + ratingTableName + " WHERE "
                + userFieldName + " = " + userID + " order by " + itemFieldName;

        Connection conn = null;
        ResultSet rs = null;
        // Establish connection with databases
        try {
            conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);
            while (rs != null && rs.next())
                items.add(new Integer(rs.getInt(1)));
            rs.close();
            conn.close();
        } catch (Exception e) {
            System.out.println("Exception:" + e);
            e.printStackTrace();
        }

        return items;
    } // getItems()

} // class CFOperation
