/* 
 * ComponentTest.java
 * 
 * Version 1.1
 * 
 * 03/11/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */ 
 
package org.recommender.test;
 
import java.sql.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.rmi.*;
import java.rmi.server.*;

import org.recommender.server.*;

/**
 * Test for cfengine peripheral components.
*
 * @author	 <A HREF="mailto:wangyun@cs.orst.edu">Yun Wang</A>
 * @since   JDK 1.2
 */
public class ComponentTest{

	private static CFEngine engine = null;
	
	
	private static GetRatingTest get = null;
	private static GetItemArrayTest itemArray = null;
	private static GetUserArrayTest userArray = null;
	private static GetRatingListTest getlist = null;
	
	//private static TestTest ts = null;
	//private static ShutdownTest shut = null;
	
	/**
	 * Constructor
	 * Either through RMI to locate the CFEngine Server, or directly create a CFEngine.
	 *
	 * @param direct
	 *
	 */
    public ComponentTest(boolean direct) {
    	
    	String installdir = System.getProperty("cf.path", ".");
        String filename = installdir + "/lib/"
                        + "CFServer.properties";
	 	if (direct) {
	 		try {
	 			engine = new CFEngineImpl(filename);
	 		} catch (Exception e) {
	 			System.out.println("Exception " + e);
	 		}
	 		 
	 	} else {     // for RMI
	 	
	 		System.setSecurityManager(new RMISecurityManager());
	 		
			// Load properties            
        	SystemProperties.loadProperties(filename);        
			String url = "rmi://" + SystemProperties.getString("CFServer.rmi.nameserver") + "/";
			String bindName = SystemProperties.getString("CFServer.rmi.objectname");
			
			// Lookup the cfengine server until it boots up
			while (true) {
				try {
					engine = (CFEngine)Naming.lookup(url + bindName);
				} catch (Exception e) {	
					//System.out.println("Exception: " + e);
				}
			
				if (engine != null) {
					try {
						SystemUtils.printDebug(engine.test());
					} catch (Exception e) {
		   				//System.out.println("Error: Cannot locate server on " 
						//	+SystemProperties.getString("CFServer.rmi.nameserver"));
						continue;
					}
					break;
				}
			}
			 	
	 	
	 	}
	 	
	 
	 	get = new GetRatingTest(engine);
		itemArray = new GetItemArrayTest(engine);
		userArray = new GetUserArrayTest(engine);
		getlist = new GetRatingListTest(engine);
	 	
    }
	
	/**
     * Set up the RMI connection.
     */
    public ComponentTest() {
        // First setup the RMI Security manager
		System.setSecurityManager(new RMISecurityManager());
		// Load properties
            String installdir = System.getProperty("cf.path",".");

        SystemProperties.loadProperties(installdir + "/lib/" + "CFServer.properties");
        
		String url = "rmi://" + SystemProperties.getString("CFServer.rmi.nameserver") + "/";
		String bindName = SystemProperties.getString("CFServer.rmi.objectname");
		
		// check server until it boots up
		while (true) {
			try {
				engine = (CFEngine)Naming.lookup(url + bindName);
			} catch (Exception e) {	
				//System.out.println("Error: Cannot locate server on " 
				//		+SystemProperties.getString("CFServer.rmi.nameserver"));
			}
			
			if (engine != null) {
				try {
					SystemUtils.printDebug(engine.test());
				} catch (Exception e) {
		   			//System.out.println("Error: Cannot locate server on " 
					//	+SystemProperties.getString("CFServer.rmi.nameserver"));
					continue;
				}
				break;
			}
		}
		
		get = new GetRatingTest(engine);
		itemArray = new GetItemArrayTest(engine);
		userArray = new GetUserArrayTest(engine);
		getlist = new GetRatingListTest(engine);
		
		//ts = new TestTest(engine);
		//shut = new ShutdownTest(engine);
    }
	
   /**
    * For Direct use only
    */
    public void exit()
    {
    	try {
    		engine.shutdown();
    	} catch (Exception e) {
    		System.out.println("Exception " + e);
    	}
    }
    
	public static void main(String[] args)
	{
		String s = null;
		if (args != null && args.length > 0)
			s = args[0];
			
		ComponentTest test;
		if (s != null && s.equals("rmi")) 
			test = new ComponentTest();
		else
			test = new ComponentTest(true); 
		test.get.testSeq();
		test.itemArray.testSeq();
		test.userArray.testSeq();
		test.getlist.testSeq();
		
		//test.ts.testSeq();
		//test.shut.testSeq();
		if (s == null || !s.equals("rmi")) test.exit();
	}

} // class ComponentTest