/* 
 * CoreComponentTest.java
 * 
 * Version 1.1
 * 
 * 04/10/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */
package org.recommender.test;
 
import java.sql.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.rmi.*;
import java.rmi.server.*;

import org.recommender.server.*;

/**
 * Test for cfengine core components.
 *
 * @author	 <A HREF="mailto:wangyun@cs.orst.edu">Yun Wang</A>
 * @since   JDK 1.2 
 */
public class CoreComponentTest{

	private static CFEngine engine = null;
	
	private static GetPredictedRatingTest pred = null;
	private static GetRecommendationsTest recom = null;
	private static GetRecommendationsByTypeTest type = null;
	private static GetPredictedRatingListTest predlist = null;
	
	/**
	 * Constructor
	 * Either through RMI to locate the CFEngine Server, or directly create a CFEngine.
	 *
	 * @param direct
	 *
	 */
	 public CoreComponentTest(boolean direct) {
	 	
	 	String installdir = System.getProperty("cf.path", ".");
        String filename = installdir + "/lib/"
                        + "CFServer.properties";
	 	if (direct) {
	 		try {
	 			engine = new CFEngineImpl(filename);
	 		} catch (Exception e) {
	 			System.out.println("Exception " + e);
	 		} 
	 	} else {   							// RMI connection
	 		
	 		System.setSecurityManager(new RMISecurityManager());
		    SystemProperties.loadProperties(installdir + "/lib/" + "CFServer.properties");
        
			String url = "rmi://" + SystemProperties.getString("CFServer.rmi.nameserver") + "/";
			String bindName = SystemProperties.getString("CFServer.rmi.objectname");
			
			// Lookup the cfengine server until it boots up
			while (true) {
				try {
					engine = (CFEngine)Naming.lookup(url + bindName);
				} catch (Exception e) {	
									
				}
			
				if (engine != null) {
					try {
						SystemUtils.printDebug(engine.test());
					} catch (Exception e) {
		   				System.out.println("Error: Cannot locate server on " 
							+SystemProperties.getString("CFServer.rmi.nameserver"));
						continue;
					}
					break;
				}
			}
	 	
	 	
	 	}
	 	
	 	pred = new GetPredictedRatingTest(engine);
		recom = new GetRecommendationsTest(engine);
		type = new GetRecommendationsByTypeTest(engine);
		predlist = new GetPredictedRatingListTest(engine);
	 }
	
	
	/**
     * Set up the RMI connection.
     */
    public CoreComponentTest() {

        // First setup the RMI Security manager
		System.setSecurityManager(new RMISecurityManager());
		// Load properties
            String installdir = System.getProperty("cf.path",".");

        SystemProperties.loadProperties(installdir + "/lib/" + "CFServer.properties");
        
		String url = "rmi://" + SystemProperties.getString("CFServer.rmi.nameserver") + "/";
		String bindName = SystemProperties.getString("CFServer.rmi.objectname");
	
		// Lookup the cfengine server until it boots up
		while (true) {
			try {
				engine = (CFEngine)Naming.lookup(url + bindName);
			} catch (Exception e) {	
				
			}
			
			if (engine != null) {
				
				try {
					SystemUtils.printDebug(engine.test());
				} catch (Exception e) {
		   			//System.out.println("Error: Cannot locate server on " 
					//	+ SystemProperties.getString("CFServer.rmi.nameserver"));
					continue;
				}
				break;
			}
		}
		pred = new GetPredictedRatingTest(engine);
		recom = new GetRecommendationsTest(engine);
		type = new GetRecommendationsByTypeTest(engine);
		predlist = new GetPredictedRatingListTest(engine);
		
    }
    
   /**
    * For Direct use only
    */
    public void exit()
    {
    	try {
    		engine.shutdown();
    	} catch (Exception e) {
    		System.out.println("Exception " + e);
    	}
    }
    
    public static void main(String[] args)
	{
		//CoreComponentTest test = new CoreComponentTest();
		String s = null;
		if (args != null && args.length > 0)
			s = args[0];
			
		CoreComponentTest test;
		if (s != null && s.equals("rmi")) {
		
			test = new CoreComponentTest();
		} else {
			test = new CoreComponentTest(true);
		}
		
		test.pred.testSeq();
		test.recom.testSeq();
		test.type.testSeq();
		test.predlist.testSeq();
		
		if (s == null || !s.equals("rmi")) 
			test.exit();		
	}

} // class CoreComponentTest