/* 
 * DBComponentTest.java
 * 
 * Version 1.1
 * 
 * 04/11/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */ 
package org.recommender.test;
 
import java.sql.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.rmi.*;
import java.rmi.server.*;

import org.recommender.server.*;

/**
 * Test for cfengine DB components
 *
 * @author	 <A HREF="mailto:wangyun@cs.orst.edu">Yun Wang</A>
 * @since   JDK 1.2 
 */
public class DBComponentTest{

	private static CFEngine engine = null;
	
	private static RemoveRatingTest remove = null;
	private static AddRatingTest add = null;
	private static AddNewRatingListTest addlist = null;
	
	/**
	 * Constructor
	 * Either through RMI to locate the CFEngine Server, or directly create a CFEngine.
	 *
	 * @param direct
	 *
	 */
	 public DBComponentTest(boolean direct) {
	 	
	 	String installdir = System.getProperty("cf.path", ".");
	 	
	 	if (direct) {
	 		
        	String filename = installdir + "/lib/" + "CFServer.properties";
	 		try {
	 			engine = new CFEngineImpl(filename);
	 		} catch (Exception e) {
	 			System.out.println("Exception " + e);
	 		} 
	 	} else {     // for RMI
	 	
	 		System.setSecurityManager(new RMISecurityManager());
	 		
			// Load properties            
        	SystemProperties.loadProperties(installdir + "/lib/" + "CFServer.properties");        
			String url = "rmi://" + SystemProperties.getString("CFServer.rmi.nameserver") + "/";
			String bindName = SystemProperties.getString("CFServer.rmi.objectname");
			
			// Lookup the cfengine server until it boots up
			while (true) {
				try {
					engine = (CFEngine)Naming.lookup(url + bindName);
				} catch (Exception e) {	
				
				}
			
				if (engine != null) {
					try {
						SystemUtils.printDebug(engine.test());
					} catch (Exception e) {
		   				//System.out.println("Error: Cannot locate server on " 
						//	+SystemProperties.getString("CFServer.rmi.nameserver"));
						continue;
					}
					break;
				}
			}
			 	
	 	
	 	}
	 	
	 	remove = new RemoveRatingTest(engine);
		add = new AddRatingTest(engine);
		addlist = new AddNewRatingListTest(engine);
	 }
	
	/**
     * Set up the RMI connection.
     */
    public DBComponentTest() {
        // First setup the RMI Security manager
		System.setSecurityManager(new RMISecurityManager());
		// Load properties
            String installdir = System.getProperty("cf.path",".");

        SystemProperties.loadProperties(installdir + "/lib/" + "CFServer.properties");
        
		String url = "rmi://" + SystemProperties.getString("CFServer.rmi.nameserver") + "/";
		String bindName = SystemProperties.getString("CFServer.rmi.objectname");
		
		// Lookup the cfengine server until it boots up
		while (true) {
			try {
				engine = (CFEngine)Naming.lookup(url + bindName);
			} catch (Exception e) {	
				
			}
			
			if (engine != null) {
				try {
					SystemUtils.printDebug(engine.test());
				} catch (Exception e) {
		   			//System.out.println("Error: Cannot locate server on " 
					//	+SystemProperties.getString("CFServer.rmi.nameserver"));
					continue;
				}
				break;
			}
		}
		
		remove = new RemoveRatingTest(engine);
		add = new AddRatingTest(engine);
		addlist = new AddNewRatingListTest(engine);
		
    } // DBComponentTest()
    
   /**
    * For Direct use only
    */
    public void exit()
    {
    	try {
    		engine.shutdown();
    	} catch (Exception e) {
    		System.out.println("Exception " + e);
    	}
    }
    
    public static void main(String[] args)
	{
		//DBComponentTest test = new DBComponentTest();
		String testMode = args[0];
		String s = null;
		if (args.length > 1)
			s = args[1];
		
		DBComponentTest test;
		if (s != null && s.equals("rmi"))
			test = new DBComponentTest();
		else
			test = new DBComponentTest(true);
			
		if (testMode.equals("remove"))
			test.remove.testSeq();
		else if (testMode.equals("add"))
			test.add.testSeq();	
		else if (testMode.equals("addlist"))
			test.addlist.testSeq();
			
		if (s == null || !s.equals("rmi")) test.exit();
	}

} // class DBComponentTest