/* 
 * GetItemArrayTest.java
 * 
 * Version 1.1
 * 
 * 03/31/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */

package org.recommender.test;
 
import java.sql.*;
import java.util.*;
import java.io.*;
import org.recommender.server.*;

/**
 * Test for getUserRatingList()
 * @author Yun Wang
 * 
 */
public class GetItemArrayTest{

	private CFEngine engine = null;
	
	private static String ratingTableName;
    private static String userFieldName;
    private static String itemFieldName;
    private static String ratingFieldName;
    
    private float minRating = 0;
    private float maxRating = 0;
    private static Statement stmt = null;
    
    private int maxUser = 0, maxItem = 0;
    
    public GetItemArrayTest(CFEngine s)
    {
    	engine = s;
    	
    	String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                + "CFServer.properties";
        
        SystemProperties.loadProperties(filename);
        
        minRating = SystemProperties.getFloat("CFServer.db.minRating", 0);
        maxRating = SystemProperties.getFloat("CFServer.db.maxRating", 0);
        
        // Load table and field names
        ratingTableName = SystemProperties.getString(
                "CFServer.db.ratingsTable", "");
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
        
        String query = "SELECT MAX(" + userFieldName + "), MAX(" + itemFieldName +
        			") FROM " + ratingTableName; 
	    
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
            conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
        	if (rs != null && rs.next()){ 
               maxUser = rs.getInt(1);
               maxItem = rs.getInt(2);               
           }
           rs.close();
           conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
        } catch (IOException e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
       
    
    } // GetItemArrayTest()
    
    /**
  	 * singleUserTest()
  	 * Get item array:
  	 * 1.get array 
     * 2.rate 5 items and get array 
     * 3.remove first 5 itmes and get array
  	 * @return int
  	 */
  	public int singleUserTest(int user)
  	{
  		System.out.println("Check Item Array for User " + user + " ...");
  		int pass = 1;
  		
  		ItemRating[] itemArr = null;
  		
  		try {
  			itemArr = engine.getUserRatingList(user);
  		} catch (Exception e) {
            pass = -1;
  			System.out.println("Error Testing for getUserRatingList()");
            System.out.println("Exception:" + e);
            e.printStackTrace();
        } 
       
        System.out.println("Originally User " + user + " rated " + itemArr.length + "items");
       /* for ( int i = 0; i < itemArr.length; i++)
        {
        	ItemRating itRating = null;
        	try {
        		itRating = engine.getRating(user, itemArr[i].getItemID());
        	} catch (Exception e) {
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        	System.out.println("\tUser " + user + "\tItem " + itRating.getItemID() 
        					+ "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(user, itemArr[i].getItemID())); 
        }*/
        System.out.println();
        
        // add or change ratings for 5 items
        Random r1 = new Random();
  		Random r2 = new Random();
  		  		
        for (int i = 0; i < 5; i++)
        {
        	int item = (int) (r1.nextFloat() * (maxItem - 1) + 1);
        	ItemRating itRating = null;
        	try {
        		itRating = engine.getRating(user, item);
        	} catch (Exception e) {
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        	float rate = -1;
        	if (itRating.getRating() < minRating || itRating.getRating() > maxRating )
  				rate = r2.nextFloat() * (maxRating - 1) + 1;
  			else
  				rate = 1 + itRating.getRating() / 3;
        	
        	try {
        		engine.setRating(user, item, rate);
        	} catch (Exception e) {
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        }
        
        itemArr = null;
        try {
  			itemArr = engine.getUserRatingList(user);
  		} catch (Exception e) {
            pass = -1;
  			System.out.println("Error Testing for getUserRatingList()");
            System.out.println("Exception:" + e);
            e.printStackTrace();
        } 
       
        System.out.println("After Add, User " + user + " rated " + itemArr.length + " items.");
        /* for ( int i = 0; i < itemArr.length; i++)
        {
        	ItemRating itRating = null;
        	try {
        		itRating = engine.getRating(user, itemArr[i].getItemID());
        	} catch (Exception e) {
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        	System.out.println("\tItem " + itRating.getItemID() + "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(user, itemArr[i].getItemID())); 
        }*/
        
        System.out.println();
        
        // remove first 5 items
        for (int i = 0; i < 5; i++)
        {
        	try {
        		engine.removeRating(user, itemArr[i].getItemID());
        	} catch (Exception e) {
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        }
        
        itemArr = null;
        try {
  			itemArr = engine.getUserRatingList(user);
  		} catch (Exception e) {
            pass = -1;
  			System.out.println("Error Testing for getUserRatingList()");
            System.out.println("Exception:" + e);
            e.printStackTrace();
        } 
        System.out.println("After remove first 5 items, User " + user + " rated "
        					+ itemArr.length + " items.");
        
        /* for ( int i = 0; i < itemArr.length; i++)
        {
        	ItemRating itRating = null;
        	try {
        		itRating = engine.getRating(user, itemArr[i].getItemID());
        	} catch (Exception e) {
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        	System.out.println("\tItem " + itRating.getItemID() + "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(user, itemArr[i].getItemID())); 
        }*/
        System.out.println();
        return pass;
        
  	} // singleUserTest()
    
    /**
  	 * multiUserTest()
  	 * Test for 5 users: 
  	 * 1. get arrays for all users (should only return list for user 0)
     * 2. rate 5 items for all users
     * 3. remove first 5 itmes for all users
     * @return int
  	 */
  	public int multiUserTest()
  	{
  		System.out.println("Check multi users ...");
  		int pass = 1;
  		
  		ItemRating[][] itemArr = new ItemRating[5][];
  		int[] user = new int[5];
  		Random r1 = new Random();
  		
  		for(int i = 0; i < user.length; i++)
  		{
  			user[i] = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  			try {
  				itemArr[i] = engine.getUserRatingList(user[i]);
  			} catch (Exception e) {
            	pass = -1;
  				System.out.println("Error Testing for getUserRatingList()");
            	System.out.println("Exception:" + e);
            	e.printStackTrace();
        	}
        	System.out.println("Originally User " + user[i] + " rated " + itemArr[i].length + " items.");
        	
        	/*for ( int k = 0; k < itemArr[i].length; k++)
        	{
        		ItemRating itRating = null;
        		try {
        			itRating = engine.getRating(user[i], itemArr[i][k]);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        		System.out.println("\tItem " + itRating.getItemID() + "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(user[i], itemArr[i][k])); 
        	}*/
        	
        } // for
        System.out.println();

        // add or change ratings for 5 items
        Random r2 =new Random();
        
        for(int i = 0; i < user.length; i++)
  		{
  			for ( int k = 0; k < 5; k++)
        	{
        		int item = (int) (r1.nextFloat() * (maxItem - 1) + 1);
        		ItemRating itRating = null;
        		
        		try {
        			itRating = engine.getRating(user[i], item);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e + " user "+ user[i] + "item "+ item + "in DB "
                            + getRating(user[i], item));
          	  		e.printStackTrace();
        		}
        		float rate = -1;
        		if (itRating.getRating() < minRating || itRating.getRating() > maxRating )
  					rate = r2.nextFloat() * (maxRating - 1) + 1;
  				else
  					rate = 1 + itRating.getRating() / 3;
        	
        		try {
        			engine.setRating(user[i], item, rate);
        		} catch (Exception e) {
           		 	System.out.println("Exception:" + e);
          		  	e.printStackTrace();
        		}
        	
        	}
        	itemArr[i] = null;
        	try {
        		itemArr[i] = engine.getUserRatingList(user[i]);
  			} catch (Exception e) {
            	pass = -1;
  				System.out.println("Error Testing for getUserRatingList()");
            	System.out.println("Exception:" + e);
            	e.printStackTrace();
        	}
        	System.out.println("After Add 5 items, User " + user[i] + " rated " 
        						+ itemArr[i].length + " items.");
        	
        	/*for ( int k = 0; k < itemArr[i].length; k++)
        	{
        		ItemRating itRating = null;
        		try {
        			itRating = engine.getRating(user[i], itemArr[i][k]);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        		System.out.println("\tItem " + itRating.getItemID() + "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(user[i], itemArr[i][k])); 
        	}*/
        	
  		
  		} // for
  		System.out.println();
  		
  		// remove first 5 items
        for(int i = 0; i < user.length; i++)
  		{
  			int n = 5;
  			if (itemArr[i].length < n)
  				n = itemArr[i].length;
  			for ( int k = 0; k < n; k++)
        	{	
        		try {
        			engine.removeRating(user[i], itemArr[i][k].getItemID());
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        	}
        	itemArr[i] = null;	
        	try {
        		itemArr[i] = engine.getUserRatingList(user[i]);
  			} catch (Exception e) {
            	pass = -1;
  				System.out.println("Error Testing for getUserRatingList()");
            	System.out.println("Exception:" + e);
            	e.printStackTrace();
        	}
        	
        	System.out.println("After remove first 5 items, User "+ user[i] + " rated " 
        						+ itemArr[i].length + " items.");
        	/*for ( int k = 0; k < itemArr[i].length; k++)
        	{
        		ItemRating itRating = null;
        		try {
        			itRating = engine.getRating(user[i], itemArr[i][k]);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        		System.out.println("\tItem " + itRating.getItemID() + "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(user[i], itemArr[i][k])); 
        	}*/
  		
  		} // for
  		System.out.println();
  		return pass;
  		
  	} // multiUserTest()	
    
    
    /**
  	 * illegalParamTest()
  	 * 1. user < 1  
  	 * 2. user > maxUser
  	 * @return int
  	 */
  	public int illegalParamTest()
  	{
  		
  		System.out.println("Checking the illegal elements...");
  		int pass = 1;
  		
  		ItemRating[] itemArr = null;
  		
  		// user < 1
  		for (int user = 0, i = 0; i < 5; i++, user--)
  	 	{
  	 		try {
  	 			itemArr = engine.getUserRatingList(user);
  	 		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
            	pass = -1;
  				System.out.println("Error Testing for getUserRatingList()");
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}        	
        	if (itemArr == null)
        		continue;
        	System.out.println("User " + user + " rated " + itemArr.length + " items.");
        	/*for ( int k = 0; k < itemArr.length; k++)
        	{
        		ItemRating itRating = null;
        		try {
        			itRating = engine.getRating(user, itemArr[k]);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        		System.out.println("\tItem " + itRating.getItemID() + "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(user, itemArr[k])); 
        	} */ 		
        	
  	 	} // for
  	 	System.out.println();
  	 	
  	 	// user > maxUser
  	 	itemArr = null;
  	 	for (int user = maxUser + 1, i = 0; i < 5; i++, user++)
  	    {
  	    	try {
  	 			itemArr = engine.getUserRatingList(user);
  	 		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
            	pass = -1;
  				System.out.println("Error Testing for getUserRatingList()");
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        	
        	if (itemArr == null)
        		continue;
        	System.out.println("User " + user + " rated " + itemArr.length + " items.");
        	/*for ( int k = 0; k < itemArr.length; k++)
        	{
        		ItemRating itRating = null;
        		try {
        			itRating = engine.getRating(user, itemArr[k]);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        		System.out.println("\tItem " + itRating.getItemID() + "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(user, itemArr[k])); 
        	} */ 	
        } // for
        System.out.println();  	    
		return pass;
		
	} // illegalParamTest()
	
    /**
     * Get connection with database. 
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException
    {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");     
         
        return DriverManager.getConnection(url, username, password);
        
        // We now use a connection pooling approach. The real SQL driver
        // is set in the static initializer of this class.
        //return DriverManager.getConnection("jdbc:jdc:jdcpool");
    } // getConnection()
    
  	/**
  	 * getRating()
  	 * Get rating from database.
     * @return rating
     * 
     */
  	private float getRating(int user, int item)
    {
    	float rating = -1;
    	String query = "SELECT " + ratingFieldName + " FROM " + ratingTableName
    				   + " WHERE " + userFieldName + " = " + user + " and "
    				   + itemFieldName + " = " + item; 
   
   		Connection conn = null;	
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
         	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);           
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();    
        } catch (Exception e) {           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           if (rs != null && rs.next()){ 
               rating = rs.getFloat(1);
            }
            rs.close();
            conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
    // System.out.println(""+rating);   
        return rating;
    
    } // getRating()
    
    /**
  	 * testSeq()
  	 */
    public void testSeq()
  	{
  		System.out.println("############################");
  		System.out.println("Test for getUserRatingList() ...");
  		System.out.println("############################");
  		System.out.println("");
  		int pass = 1;
  		
  		Random r = new Random();
  		
  		System.out.println("Start Single User Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = singleUserTest(user);
			if (t < 0 )
				pass = -1;
		}
		
		System.out.println("Start Multi User Test ... ");
    	System.out.println(""); 
    	int t = multiUserTest();
    	if (t < 0 )
			pass = -1;
    	
    	System.out.println("Start illegal param Test ... ");
    	System.out.println("");
    	t = illegalParamTest();
    	if (t < 0 )
			pass = -1;
		
		System.out.println();
		if (pass >= 0) {
			System.out.println("------ Pass! getUserRatingList() Test! ------");
            System.err.println("------ Pass! getUserRatingList() Test! ------");
        } else {
			System.out.println("------ Fail! getUserRatingList() Test! ------");
            System.err.println("------ Fail! getUserRatingList() Test! ------");
        }
        System.out.println();
	}
    
} // class GetItemArrayTest