/* 
 * GetPredictedRatingListTest.java
 * 
 * Version 1.1
 * 
 * 03/23/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */

package org.recommender.test;
 
import java.sql.*;
import java.util.*;
import java.io.*;
import org.recommender.server.*;

/**
 * Test for getPredictedRatingList()
 * @author Yun Wang
 * 
 */
public class GetPredictedRatingListTest{

	private CFEngine engine = null;
	
	private static String ratingTableName;
    private static String userFieldName;
    private static String itemFieldName;
    private static String ratingFieldName;
    
    private float minRating = 0;
    private float maxRating = 0;
  	
    private static Statement stmt = null;
    private int maxUser = 0, maxItem = 0;
    
    
    public GetPredictedRatingListTest(CFEngine s)
    {
  		engine = s;
  		if (engine == null)
  			System.out.println("engine is null");
  			
  		String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                + "CFServer.properties";
        
        SystemProperties.loadProperties(filename);
        
        minRating = SystemProperties.getFloat("CFServer.db.minRating", 0);
        maxRating = SystemProperties.getFloat("CFServer.db.maxRating", 0);
        
        // Load table and field names
        ratingTableName = SystemProperties.getString(
                "CFServer.db.ratingsTable", "");
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
        
    	String query = "SELECT MAX(" + userFieldName + "), MAX(" + itemFieldName +
        			") FROM " + ratingTableName; 
	    
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
           conn = getConnection();
           stmt = conn.createStatement();
           rs = stmt.executeQuery(query); 
           if (rs != null && rs.next()){ 
               maxUser = rs.getInt(1);
               maxItem = rs.getInt(2);
               
           }
           rs.close();           
           conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
        } catch (IOException e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
  	
  	} // GetPredictedRatingListTest()
  	
  	/**
  	 * singleItemTest1()
  	 * item array length is 1(randomly), compare predicted value with the one in DB
  	 * @return int
  	 */
  	public int singleItemTest1(int user)
  	{
  		System.out.println("Check randomly selected Single Item List for User " + user + " ...");
  		int pass = 1;
  		
  		Random r = new Random();
  		int[] item = new int[1];
  		item[0] = (int) (r.nextFloat() * (maxItem - 1) + 1);
  		
  		ItemPrediction[] pred = null;
  		
  		try {
  			pred = engine.getPredictedRatingList(user, item);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getPredictedRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		for ( int i = 0; i < pred.length; i++)	 
  		{
  			System.out.println("\tItem " + pred[i].getItemID() + "\tgetPredictedItemList(): " + pred[i].getPrediction()
  								+ "\tin DB: " + getRating(user, item[0]));
  		}
  		System.out.println("");						
  		return pass;
  		
  	} // singleItemTest1()
  	
  	/**
  	 * singleItemTest2()
  	 * item array length is 1 (existed), compare predicted value with the one in DB
  	 * @return int
  	 */
  	public int singleItemTest2(int user)
  	{
  		System.out.println("Check existed Single Item List for User " + user + " ...");
  		int pass = 1;
  		
  		Random r = new Random();  		
  		int[] item = new int[1];
  		
  		Vector v = getItems(user);
  		int length = v.size();
  		if (length <= 0) 
  		{
  			System.out.println("\t no item available.");
  			return 0;
  		}  	
  		item[0] = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  		
  		ItemPrediction[] pred = null;
  		
  		try {
  			pred = engine.getPredictedRatingList(user, item);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getPredictedRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}	 
  		for ( int i = 0; i < pred.length; i++)	 
  		{
  			System.out.println("\tItem " + pred[i].getItemID() + "\tgetPredictedItemList(): " + pred[i].getPrediction()
  								+ "\tin DB: " + getRating(user, item[0]));
  		}
  		System.out.println("");	
  								
  		return pass;
  		
  	} // singleItemTest2()
  	
  	
  	/**
  	 * fiveItemTest()
  	 * Get 10 items (5 existed, 5 randomly), check values. Test in 3 ways:
  	 * @return int
  	 */
  	public int fiveItemTest(int user)
  	{
  		System.out.println("Check 5 Items for User " + user + " ...");
  		int pass = 1;
  		
  		Random rdm = new Random();
  		int[] item = new int[5];
  		ItemPrediction[] pred = null;
  		
  		// for 5 randomly selected items
  		for (int i = 0; i < 5; i++)
  		{
  			item[i] = (int) (rdm.nextFloat() * (maxItem - 1) + 1);
  		}	
  			
  		System.out.println("  Originally");
  		try {
  			pred = engine.getPredictedRatingList(user, item);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getPredictedRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		
  		// rate first 3 items, then check the predicted rating
  		for (int i = 0; i < 5; i++)
  		{
  			float r = getRating(user, item[i]);
  			System.out.println("\tItem " + item[i] + "\tgetPredictedRatingList(): " 
  								+ pred[i].getPrediction() + "\tin DB: " + r);
  			if (i < 3)
  			{
  				if (r < minRating || r > maxRating)
  					r = rdm.nextFloat() * (maxRating - 1) + 1;
  				else
  					r = 1 + r / 3;
  				try {
  					engine.setRating(user, item[i], r);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  			}
  		}
  		pred = null;
  		try {
  			pred = engine.getPredictedRatingList(user, item);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getPredictedRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		
  		System.out.println("  After rate first 3 items");
  		// rate last 2 items, remove first 2
  		for (int i = 0; i < 5; i++)
  		{
  			float r = getRating(user, item[i]);
  			System.out.println("\tItem " + item[i] + "\tgetPredictedRatingList():: " 
  								+ pred[i].getPrediction() + "\tin DB: " + r);
  			if (i < 2)
  			{
  				try {
  					engine.removeRating(user, item[i]);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  			}
  			if (i > 3)
  			{
  				if (r < minRating || r > maxRating)
  					r = rdm.nextFloat() * (maxRating - 1) + 1;
  				else
  					r = 1 + r / 3;
  				try {
  					engine.setRating(user, item[i], r);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  			}					
  		}
  		
  		pred = null;
  		try {
  			pred = engine.getPredictedRatingList(user, item);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getPredictedRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  			 
  		System.out.println("  After remove first 2 ratings, rate last 2 items");
  		for (int i = 0; i < 5; i++)
  		{
  			System.out.println("\tItem " + item[i] + "\tgetPredictedRatingList(): " 
  								+ pred[i].getPrediction() + "\tin DB: " + getRating(user, item[i]));
  		}
  		
  		// make duplicate entry for item list
  		item[4] = item[0];
  		pred = null;
  		try {
  			pred = engine.getPredictedRatingList(user, item);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getPredictedRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		System.out.println("  Duplicate one item");
  		for (int i = 0; i < 5; i++)
  		{
  			System.out.println("\tItem " + item[i] + "\tgetPredictedRatingList(): " + pred[i].getPrediction() 
  								+ "\tin DB: " + getRating(user, item[i]));
  		}
  		System.out.println();
  		return pass;
  		
  	} // fiveItemTest()
  	
  	/**
  	 * multiUserTest1()
  	 * Test 5 users for 20 items:
  	 * 1. get predictions
  	 * 2. rate the first 3 items, then get predictions
  	 * 3. remove first 2 ratings and rate last 5 itmes, then get predictions
  	 * @return int
  	 */
  	public int multiUserTest1()
  	{
  		System.out.println("Check multi users ...");
  		int pass = 1;
  		
  		Random rdm1 = new Random();
  		int[] item = new int[20];
  		int[] user = new int[5];
  		ItemPrediction[][] pred = new ItemPrediction[5][];
  		
  		for (int i = 0; i < 20; i++)
  		{
  			item[i] = (int) (rdm1.nextFloat() * (maxItem - 1) + 1);
  		}
  		
  		Random rdm2 = new Random();
  		for ( int i = 0; i < 5; i++)
  		{
  			user[i] = (int) (rdm2.nextFloat() * (maxUser - 1) + 1);
  			try {
  				pred[i] = engine.getPredictedRatingList(user[i], item);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getPredictedRatingList()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			
  			System.out.println("  User " + user[i]);
  			System.out.println("\tOriginally");
  			// check value, then rate tirst 3 items
  			for (int k = 0; k < 20; k++)
  			{
  				float r = getRating(user[i], item[k]);
  				System.out.println("\t\tItem " + item[k] + "\tgetPredictedRatingList(): " 
  									+ pred[i][k].getPrediction() + "\tin DB: " + r);
  				if (i < 3) {
  				
  					if (r < minRating || r > maxRating)
  						r = rdm1.nextFloat() * (maxRating - 1) + 1;
  					else
  						r = 1 + r / 3;
  					try {
  						engine.setRating(user[i], item[k], r);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  				}								
  			
  			}
  			
  			pred[i] = null;
  			try {
  				pred[i] = engine.getPredictedRatingList(user[i], item);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getPredictedRatingList()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			// check value, then rate last 5 items, remove first 2 items
  			System.out.println("\tAfter rate first 3 items");
  			for (int k = 0; k < 20; k++)
  			{
  				float r = getRating(user[i], item[k]);
  				System.out.println("\t\tItem " + item[k] + "\tgetPredictedRatingList(): " 
  									+ pred[i][k].getPrediction() + "\tin DB: " + r);
  				
  				if (i < 2)
  				{
  					try {
  						engine.removeRating(user[i], item[k]);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  				}
  				
  				if (i > 14) {
  				
  					if (r < minRating || r > maxRating)
  						r = rdm1.nextFloat() * (maxRating - 1) + 1;
  					else
  						r = 1 + r / 3;
  					try {
  						engine.setRating(user[i], item[k], r);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  				}	
  			}
  			
  			pred[i] = null;
  			try {
  				pred[i] = engine.getPredictedRatingList(user[i], item);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getPredictedRatingList()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			
  			System.out.println("\tAfter rate last 5 items, remove first 2 items");
  			for (int k = 0; k < 20; k++)
  			{
  				float r = getRating(user[i], item[k]);
  				System.out.println("\t\tItem " + item[k] + "\tgetPredictedRatingList(): "  
  									+ pred[i][k].getPrediction() + "\tin DB: " + r);
  			}
  		} // for
  		System.out.println();
  		return pass;
  		
  	} // multiUserTest1()
  	
  	/**
  	 * illegalParamTest()
  	 * 1. item < 1 and item > maxItem
  	 * 2. user < 1 and user > maxUser
  	 * 3. item array is null.
  	 * @ return int
  	 */
  	public int illegalParamTest()
  	{
  		 System.out.println("Checking the illegal elements...");
  		 int pass = 1;
  		 
  		 Random r1 = new Random();
  		   		   		 
  		 // user < 1
  		for (int user = 1, k = 0; k < 5; k++, user--)
  	 	{
  	  		int[] item = new int[5];
  	  		ItemPrediction[] pred = null;
  	  		
  	  		for (int i = 0; i < 5; i++)
  	  		{
  	  			item[i] =(int) (r1.nextFloat() * (maxItem - 1) + 1);
  	  		}  	  		  			
  			
  			try {
  				pred = engine.getPredictedRatingList(user, item);  					
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getPredictedRatingList()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("  For User " + user);
  	  		if (pred == null)
  	  			continue;
  			for (int i = 0; i < pred.length; i++)
  	  		{ 
  				System.out.println("\tItem " + item[i] + "\tgetPredictedRatingList(): " + pred[i].getPrediction()
  								+ "\tin DB " + getRating(user, item[i]));
  			}
  	  	}
  	  	System.out.println();
  	  	
  	  	// user > maxUser 
  	  	for (int user = maxUser + 1, k = 0; k < 5; k++, user++)
  	    {
  	    	int[] item = new int[5];
  	  		ItemPrediction[] pred = null;
  	  		
  	  		for (int i = 0; i < 5; i++)
  	  		{
  	  			item[i] =(int) (r1.nextFloat() * (maxItem - 1) + 1);
  	  		}  	  	
  			try {
  				pred = engine.getPredictedRatingList(user, item);  					
  			
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getPredictedRatingList()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
 
  			System.out.println("  For User " + user);
  	  		if ( pred == null)
  	  			continue;
  			for (int i = 0; i < pred.length; i++)
  	  		{ 
  				System.out.println("\tItem " + item[i] + "\tgetPredictedRatingList(): " + pred[i].getPrediction()
  								+ "\tin DB " + getRating(user, item[i]));
  			}
  	  	}
  	  	System.out.println();
  	  	
  	  	  		
  		// item < 0 in item list
  		Random r2 = new Random();
  		int[] item = new int[5];  		
  		for (int i = 0, k = 2; i < 5; i++, k--)
  	  	{ 
  	  		item[i] = k;
   	  	}
  	  	
  	  	for(int k = 0; k < 3; k++)
  	  	{
  	  		int user = (int) (r2.nextFloat() * (maxUser - 1) + 1);
  	  		ItemPrediction[] pred = null;
  	  		try {
  				pred = engine.getPredictedRatingList(user, item);  					
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getPredictedRatingList()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("  For User " + user);
  	  		if ( pred == null)
  	  			continue;
  			for (int i = 0; i < pred.length; i++)
  	  		{ 
  				System.out.println("\tItem " + item[i] + "\tgetPredictedRatingList(): " 
  								+ pred[i].getPrediction() + "\tin DB " + getRating(user, item[i]));
  			}
  	  	}
  	  	System.out.println();
  	  	
  	  	// item > maxItem in item list
  		for (int i = 0, k = (maxItem - 2); i < 5; i++)
  	  	{ 
  	  		item[i] = k++;
   	  	}
  	  	
  	  	for(int k = 0; k < 3; k++)
  	  	{
  	  		int user = (int) (r2.nextFloat() * (maxUser - 1) + 1);
  	  		ItemPrediction[] pred = null;
  	  		try {
  				pred = engine.getPredictedRatingList(user, item);  					
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getPredictedRatingList()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("  For User " + user);
  	  		if ( pred == null)
  	  			continue;
  			for (int i = 0; i < pred.length; i++)
  	  		{ 
  				System.out.println("\tItem " + item[i] + "\tgetPredictedRatingList(): " 
  							+ pred[i].getPrediction() + "\tin DB " + getRating(user, item[i]));
  			}
  	  	}
  	  	System.out.println();
  	  	
  	  	// item list is null
  	  	int u = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  	  	ItemPrediction[] r = null;
  	  	int[] itm = null;
  	  	try {
  			r = engine.getPredictedRatingList(u, itm);  					
  		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  		} catch (Exception e) {
  			pass = -1;
  				System.out.println("Error Testing for getPredictedRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		
  		/*for (int i = 0; i < r.length; i++)
  	  	{ 
  			System.out.println("User " + u + "\tItem " + itm[i] + "\tpredicted " + r[i]
  								+ "\tin DB " + getRating(u, itm[i]));
  		}*/
  		System.out.println();
  		return pass;
  		
  	} // illegalParamTest
  	
  	/**
     * Get connection with database. 
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException
    {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");
        
        return DriverManager.getConnection(url, username, password);
        
        // We now use a connection pooling approach. The real SQL driver
        // is set in the static initializer of this class.
        //return DriverManager.getConnection("jdbc:jdc:jdcpool");
    } // getConnection()
    
    /**
  	 * getRating()
  	 * Get rating from database.
     * @return rating
     * 
     */
    private float getRating(int user, int item)
    {
    	float rating = -1;
    	String query = "SELECT " + ratingFieldName + " FROM " + ratingTableName
    				   + " WHERE " + userFieldName + " = " + user + " and "
    				   + itemFieldName + " = " + item; 
   
   		Connection conn = null;	
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
          	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
           
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();    
        } catch (Exception e) {           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           if (rs != null && rs.next()){ 
               rating = rs.getFloat(1);
            }
            rs.close();
            conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
    // System.out.println(""+rating);   
        return rating;
    
    } // getRating()
    
     /**
  	 * getItems()
  	 * Get rated items for a user
     * @return Vector
     * 
     */
    private Vector getItems(int user)
    {
    	Vector items = new Vector();
    	String query = "SELECT " + itemFieldName + " FROM " + ratingTableName  + " WHERE " 
    	 				  + userFieldName + " = " + user + " order by " + itemFieldName;
    	
    	Connection conn = null;		    
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
          	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
              // precompileSQLStatements(conn);          
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();        
        } catch (Exception e) {
            System.out.println("Exception:" + e);
            e.printStackTrace();
        }
        
        try {
           while (rs != null && rs.next()) 
           		items.add(new Integer(rs.getInt(1))); 
 			rs.close();
 			conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
        
        return items;
    } // getItems()
    
    /**
  	 * testSeq()
  	 */
    public void testSeq()
    {
    	System.out.println("################################");
  		System.out.println("Test for getPredictedRatingList() ...");
  		System.out.println("################################");
  		System.out.println(""); 
  		int pass = 1;
  		
  		Random r = new Random();
    	
    	System.out.println("  Start Single Item Test ... ");
    	System.out.println("");
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = singleItemTest1(user);
			if (t < 0 )
				pass = -1;
		}
		for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = singleItemTest2(user);
			if (t < 0 )
				pass = -1;
		}
		
		System.out.println("  Start Five Items Test ... ");
    	System.out.println("");
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = fiveItemTest(user);
			if (t < 0 )
				pass = -1;
		}
		
		System.out.println("  Start Multi User Test ... ");
    	System.out.println("");
    	int t = multiUserTest1();
    	if (t <= 0 )
			pass = -1;
    	
    	System.out.println("  Start illegal param Test ... ");
    	System.out.println("");
    	t = illegalParamTest();
    	if (t < 0 )
				pass = -1;
		System.out.println();
		if (pass >= 0) {
			System.out.println("------ Pass! getPredictedRatingList() Test! ------");
            System.err.println("------ Pass! getPredictedRatingList() Test! ------");
        } else {
			System.out.println("------ Fail! getPredictedRatingList() Test! ------");
            System.err.println("------ Fail! getPredictedRatingList() Test! ------");
        }
        System.out.println();
    }
    
} // class GetPredictedRatingListTest