/* 
 * GetPredictedRatingTest.java
 * 
 * Version 1.1
 * 
 * 03/16/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */ 
package org.recommender.test;
 
import java.sql.*;
import java.util.*;
import java.io.*;
import org.recommender.server.*;

/**
 * Test for getPredictedRating()
 * @author Yun Wang
 * 
 */
public class GetPredictedRatingTest{

	private CFEngine engine = null;
	
	private static String ratingTableName;
    private static String userFieldName;
    private static String itemFieldName;
    private static String ratingFieldName;
    
   // private float minRating = 0;
   // private float maxRating = 0;
    private static Statement stmt = null;
    private int maxUser = 0, maxItem = 0;  
    
    public GetPredictedRatingTest(CFEngine s)
    {
  		engine = s;
  		if (engine == null)
  			System.out.println("engine is null");
  			
  		String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                + "CFServer.properties";
        
        SystemProperties.loadProperties(filename);
        
        // Load table and field names
        ratingTableName = SystemProperties.getString(
                "CFServer.db.ratingsTable", "");
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
        
    	String query = "SELECT MAX(" + userFieldName + "), MAX(" + itemFieldName +
        			") FROM " + ratingTableName; 
	    
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
        	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
            if (rs != null && rs.next()){ 
               maxUser = rs.getInt(1);
               maxItem = rs.getInt(2);
               
           	}
           	rs.close();
           	conn.close();       
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
        } catch (IOException e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
       
  	} // GetPredictedRatingTest()
  	
  	/**
  	 * singleItemTest()
  	 * Get 1 item, compare predicted value with the one in DB
  	 * @return int
  	 */
  	public int singleItemTest(int user)
  	{
  		System.out.println("Check Existed single Item for User " + user + " ...");
  		int pass = 1;
  		
  		Random r = new Random();
  		int item = (int) (r.nextFloat() * (maxItem - 1) + 1);
  		ItemPrediction pred = null;
  		
  		try {
  			pred = engine.getPredictedRating(user, item);
  		
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getPredictedRating()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  			
  		}	 
  		System.out.println("\tItem " + item + "\tgetPredictedRating(): " + pred.getPrediction() 
  							+ "\tin DB: " + getRating(user, item));
  		System.out.println("");	
  		
  		return pass;					
  	} // singleItemTest()
    
    /**
  	 * tenItemTest()
  	 * Get 10 items (5 existed, 5 randomly), check values. Test in 3 ways:
  	 * @return int
  	 */
  	public int tenItemTest(int user)
  	{
  		System.out.println("Check 10 Items for User " + user + " ...");
  		int pass = 1;
  		Random r = new Random();
  		
  		// for 5 randomly selected items
  		for (int i = 0; i < 5; i++)
  		{
  			int item = (int) (r.nextFloat() * (maxItem - 1) + 1);
  			
  			ItemPrediction pred = null;
  			try {
  				pred = engine.getPredictedRating(user, item);
  			
      		} catch (Exception e) {
      			pass = -1;
      			System.out.println("Error Testing for getPredictedRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  				
  			}
  			System.out.println("\tItem " + item + "\tgetPredictedRating(): " + pred.getPrediction() 
  						 + "\tin DB: "+ getRating(user, item));
  				 
  		} // for
  		System.out.println("");
  		return pass;
  	} // tenItemTest()
    
    /**
  	 * multiUserTest1()
  	 * Test 5 users for 1 same item
  	 * @return int 
  	 */
  	public int multiUserTest1()
  	{
  		Random r = new Random();
  		int item = (int) (r.nextFloat() * (maxItem - 1) + 1);
  		System.out.println("Check multi users for Item " + item + " ...");
  		int pass = 1;
  		 		
  		for ( int i = 0; i < 5; i++)
  		{
  			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
  			ItemPrediction pred = null;
  			try {
  				pred = engine.getPredictedRating(user, item);
  			
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getPredictedRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("\tUser " + user + "\tgetPredictedRating(): " + pred.getPrediction()
  							 + "\tin DB: " + getRating(user, item));
  		}
  		System.out.println("");
  		return pass;
  		
  	} // multiUserTest1()
  	
  	/**
  	 * multiUserTest2()
  	 * Test 5 users for 10 items.
  	 * @return int
  	 */
  	public int multiUserTest2()
  	{
  		Random r1 = new Random();
  		int pass = 1;  		
  		System.out.println("Check multi users, each for 10 items ...");
  		int[] item = new int[10];
  		
  		for (int i = 0; i < 10; i++)
  		{
  			item[i] = (int) (r1.nextFloat() * (maxItem - 1) + 1);
  		}
  		
  		Random r2 = new Random();
  		for ( int i = 0; i < 5; i++)
  		{
  			int user = (int) (r2.nextFloat() * (maxUser - 1) + 1);
  			System.out.println("  For User " + user);
  			
  			for ( int k = 0; k < 10; k++)
  			{
  				ItemPrediction pred = null;
  				try {
  					pred = engine.getPredictedRating(user, item[k]);
  				
  				} catch (Exception e) {
  					pass = -1;
  					System.out.println("Error Testing for getPredictedRating()");
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  					
  				}
  			System.out.println("\tItem " + item[k] + "\tgetPredictedRating(): " + pred.getPrediction()
  			 				+ "\tin DB: " + getRating(user, item[k]));
  			
  			}
  		}	
  		System.out.println("");
  		return pass;
  	} // multiUserTest2()
  	
  	/**
  	 * illegalParamTest()
  	 * 1. item < 1 and item > maxItem
  	 * 2. user < 1 and user > maxUser
  	 * @ return int 
  	 */
  	public int illegalParamTest() 
  	{
  		 Random r1 = new Random();
  		 int pass = 1;
  		 System.out.println("Checking the illegal elements...");
  	     
  	     // test for itemID < 1
  	  	for (int item = 0, k = 0; k < 5; k++, item--)
  	  	{
  	  		int u = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  	  		
  	  		ItemPrediction pred = null;
  			try {
  				pred = engine.getPredictedRating(u, item);  					
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  				
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getPredictedRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  				
  			}	 
  			System.out.println("User " + u + "\tItem " + item 
  			//					+ "\tgetPredictedRating(): " + pred.getPrediction()
  								+ "\tin DB: " + getRating(u, item));
  	 	}
  	 	
  	 	// itemID > maxItem
  	 	for (int item = maxItem + 1, k = 0; k < 5; k++, item++)
  	 	{
  	 		int u = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  	  		
  	  		ItemPrediction pred = null;
  			try {
  				pred = engine.getPredictedRating(u, item);  					
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  				
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getPredictedRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			System.out.println("User " + u + "\tItem " + item 
  								+ "\tgetPredictedRating(): " + pred.getPrediction()
  								+ "\tin DB: " + getRating(u, item));
  	 	}
  	  
  	 	Random r2 = new Random();
  		 
  	  	// user < 1
  		for (int user = 0, k = 0; k < 5; k++, user--)
  	 	{
  	  		int i = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  	  		  	  		  			
  			ItemPrediction pred = null;
  			try {
  				pred = engine.getPredictedRating(user, i);  					
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  				
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getPredictedRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			System.out.println("User " + user + "\tItem " + i 
  			//					+ "\tgetPredictedRating(): " + pred.getPrediction()
  								+ "\tin DB " + getRating(user, i));
  	  	}
  	  	
  	  	// user > maxUser 
  	  	for (int user = maxUser + 1, k = 0; k < 5; k++, user++)
  	    {
  	    	int i = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  	  		  	  		  			
  			ItemPrediction pred = null;
  			try {
  				pred = engine.getPredictedRating(user, i);  					
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  				
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getPredictedRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			System.out.println("User " + user + "\tItem " + i 
  								+ "\tgetPredictedRating(): " + pred.getPrediction()
  								+ "\tin DB " + getRating(user, i));
  	  	}
  	  	return pass;
  	     
  	 } // illegalParamTest()
  	
   /**
     * Get connection with database. 
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException
    {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");
         
        return DriverManager.getConnection(url, username, password);
        
        // We now use a connection pooling approach. The real SQL driver
        // is set in the static initializer of this class.
        //return DriverManager.getConnection("jdbc:jdc:jdcpool");
        
    } // getConnection()
    
    /**
  	 * getRating()
  	 * Get rating from database.
     * @return rating
     * 
     */
    private float getRating(int user, int item)
    {
    	float rating = -1;
    	String query = "SELECT " + ratingFieldName + " FROM " + ratingTableName
    				   + " WHERE " + userFieldName + " = " + user + " and "
    				   + itemFieldName + " = " + item; 
   
   		Connection conn = null;	
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
          	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);         
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();    
        } catch (Exception e) {           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           if (rs != null && rs.next()){ 
               rating = rs.getFloat(1);
            }
            rs.close();
            conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
    // System.out.println(""+rating);   
        return rating;
    
    } // getRating()
    
    /**
  	 * testSeq()
  	 */
    public void testSeq()
    {
    	System.out.println("###############################");
  		System.out.println("Test for getPredictedRating() ...");
  		System.out.println("###############################");
  		System.out.println(""); 
  		
  		Random r = new Random();
    	int pass = 1;
    	System.out.println("  Start Single Item Test ... ");
    	System.out.println("");
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int single = singleItemTest(user);
			if (single <= 0)
				pass = -1;
		}
		
		System.out.println("Start 10 Items Test ... ");
    	System.out.println("");
    	 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			
			int t =	tenItemTest(user);
			if (t <= 0 )
				pass = -1;
		}	
		
		System.out.println("  Start multiUser Test 1  -- 5 users have 1 same items ... ");
    	System.out.println("");
		int temp = multiUserTest1();
		if (temp <= 0 && pass >= 0)
				pass = -1;
				
		System.out.println("  Start multiUser Test 2 -- 5 users have 10 same items ... ");
    	System.out.println("");
		temp = multiUserTest2();
		if (temp <= 0 && pass >= 0)
				pass = -1;
				
		System.out.println("  Start illegal param Test ... ");
    	System.out.println("");
    	temp = illegalParamTest();
    	if (temp <= 0 && pass >= 0)
				pass = -1;
		
		System.out.println();
		if (pass > 0) {
			System.out.println("------ Pass! getPredictedRating() Test! ------");
            System.err.println("------ Pass! getPredictedRating() Test! ------");
        } else {
			System.out.println("------ Fail! getPredictedRating() Test! ------");
            System.err.println("------ Fail! getPredictedRating() Test! ------");
        }
        System.out.println();
    		
	} // testSeq()
	
} // class GetPredictedRatingTest