/* 
 * GetRatingListTest.java
 * 
 * Version 1.1
 * 
 * 05/06/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */
 
package org.recommender.test;
 
import org.recommender.server.*;

import java.io.IOException;
import java.sql.*;
import java.util.Random;
import java.util.Vector;

/**
 * Test for GetRatingList()
 * @author Yun Wang
 * 
 */
public class GetRatingListTest{

	private CFEngine engine = null;
	
	private static String ratingTableName;
    private static String userFieldName;
    private static String itemFieldName;
    private static String ratingFieldName;
    
    private float minRating = 0;
    private float maxRating = 0;
    private static Statement stmt = null;
    
    private int maxUser = 0, maxItem = 0;
    
    public GetRatingListTest(CFEngine s)
    {
    	engine = s;
  		
  		String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                 + "CFServer.properties";
        
        SystemProperties.loadProperties(filename);
        
        minRating = SystemProperties.getFloat("CFServer.db.minRating", 0);
        maxRating = SystemProperties.getFloat("CFServer.db.maxRating", 0);
        
        // Load table and field names
        ratingTableName = SystemProperties.getString(
                "CFServer.db.ratingsTable", "");
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
        
        String query = "SELECT MAX(" + userFieldName + "), MAX(" + itemFieldName +
        			") FROM " + ratingTableName; 
	    
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
              conn = getConnection();
              stmt = conn.createStatement();
              rs = stmt.executeQuery(query); 
              if (rs != null && rs.next()){ 
               maxUser = rs.getInt(1);
               maxItem = rs.getInt(2);
               
           	}
        	rs.close();
        	conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
        } catch (IOException e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
    } // GetRatingListTest()
    
    /**
  	 * singleItemTest1()
  	 * Get item list (only 1 item rated), check value in 2 ways
  	 * 1. getRating() -- directly from cache
  	 * 2. directly from database
  	 * @return int
  	 */
    private int singleItemTest1(int user)
    {
    	System.out.println("Check Existed single Item for User " + user + " ...");
    	int pass = 1;
  		
  		Vector v = getItems(user);
  		int length = v.size();
  		
  		if (length <= 0) 
  		{
  			System.out.println("\t no item available.");
  			return 0;
  		}
  		int[] item = new int[1]; 
  		Random r = new Random();
  		item[0] = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  		ItemRating[] rating = null;
    	
    	try {
  			rating = engine.getRatingList(user, item);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		
  		for(int i = 0; i < rating.length; i++)
  		{
  			System.out.println("\tItem " + rating[i].getItemID() + "\tin Cache  " + rating[i].getRating()
  							+ "\tin DB " + getDBRating(user, rating[i].getItemID()) );
  		}
  		return pass;
  			 
    } // singleItemTest1()
    
    /**
  	 * singleItemTest2()
  	 * Get item list (only 1 item radomly selected), check value in 2 ways
  	 * 1. getRating() -- directly from cache
  	 * 2. directly from database
  	 * @return int
  	 */
    private int singleItemTest2(int user)
    {
    	System.out.println("Check Randomly Selected Single Item for User " + user + " ...");
    	int pass = 1;
    	
    	int[] item = new int[1];
    	Random r = new Random();
  		item[0] = (int) (r.nextFloat() * (maxItem - 1) + 1);
  		
  		ItemRating[] rating = null;
    	
    	try {
  			rating = engine.getRatingList(user, item);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		
  		for(int i = 0; i < rating.length; i++)
  		{
  			System.out.println("\tItem " + rating[i].getItemID() + "\tin Cache  " + rating[i].getRating()
  							+ "\tin DB " + getDBRating(user, rating[i].getItemID()) );
  		}
  		return pass;
  			 
    } // sigleItemTest2()
    
    /**
  	 * tenItemTest()
  	 * Get 10 items (5 existed, 5 randomly), check values in two ways:
  	 * 1. getRating() -- directly from cache
  	 * 2. directly from database
  	 * @return int
  	 */
  	private int tenItemTest(int user)
  	{
  		System.out.println("Check 10 Items for User " + user + " ...");
  		int pass = 1;
  		
  		int[] item = new int[10];
  		ItemRating[] rating = null;
  		
  		Random r = new Random();
  		Vector v = getItems(user);
  		int length = v.size();
  		
  		for (int i = 0; i < item.length; i++)
  		{
  			if ( i < 5 && length > 5)
  			{	
  				item[i] = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  			} 
  			else if (i < 5 && length <= 5 && length > (i+1))
  			{
  				item[i] = ((Integer) v.get(i)).intValue();  					
  			} 
  			else 
  			{
  				item[i] = (int) (r.nextFloat() * (maxItem - 1) + 1);
  			}
  			
  		} // for
  		
  		try {
  			rating = engine.getRatingList(user, item);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		
  		for(int i = 0; i < rating.length; i++)
  		{
  			System.out.println("\tItem " + rating[i].getItemID() + "\tin Cache  " + rating[i].getRating()
  							+ "\tin DB " + getDBRating(user, rating[i].getItemID()) );
  		}
  		return pass;
  		
  	} // tenItemTest()
  	
  	/**
  	 * multiUserTest1()
  	 * Test for 5 users with 2 items each:
  	 * 1. 1 existed items
  	 * 2. 1 randomly selected items
  	 * @return int 
  	 */
  	private int multiUserTest1()
  	{
  		System.out.println("Check multi users, each for 2 items ...");
  		int pass = 1;
  		int[] user = new int[5];
  		
  		Random r = new Random();
  		for (int i = 0; i < user.length; i++)
  		{
  			user[i] = (int) (r.nextFloat() * (maxUser - 1) + 1);
  			int[] item = new int[2];
  			ItemRating[] rating = null;
  			
  			Vector v = getItems(user[i]);
  			int length = v.size();
  	
  			if (length <= 0) 
  			{
  				System.out.println("\t no item available.");
  				return 0;
  			} 
  			item[0] = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  			item[1] = (int) (r.nextFloat() * (maxItem - 1) + 1);
  			
  			try {
  				rating = engine.getRatingList(user[i], item);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			
  			for(int k = 0; k < rating.length; k++)
  			{
  				System.out.println("User " + user[i] + "\tItem " + rating[k].getItemID()
  								 + "\tin Cache  " + rating[k].getRating()
  								 + "\tin DB " + getDBRating(user[i], rating[k].getItemID()) );
  			}
  		} // for
  		return pass;
  	
  	} // multiUserTest1()
    
    /**
  	 * multiUserTest2()
  	 * Test for 5 users with 10 items each:
  	 * 1. 5 existed items
  	 * 2. 5 randomly selected items
  	 * @return int 
  	 */
  	private int multiUserTest2()
  	{
  		System.out.println("Check multi users, each for 10 items ...");
  		int pass = 1;
  		int[] user = new int[5];
  		
  		Random r1 = new Random();
  		Random r2 = new Random();  		
  		for (int i = 0; i < user.length; i++)
  		{
  			user[i] = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  			int[] item = new int[10];
  			ItemRating[] rating = null;
  			
  			Vector v = getItems(user[i]);
  			int length = v.size();
  	
  			if (length <= 0) 
  			{
  				System.out.println("\t no item available.");
  				return 0;
  			}
  			for (int k = 0; k < item.length; k++)
  			{
  				if ( k < 5 && length > 5)
  				{	
  					item[k] = ((Integer) v.get((int) (r2.nextFloat() * length))).intValue();
  				} 
  				else if (k < 5 && length <= 5 && length > (k+1))
  				{
  					item[k] = ((Integer) v.get(k)).intValue();  					
  				} 
  				else 
  				{
  					item[k] = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  				}
  			} 
  			try {
  				rating = engine.getRatingList(user[i], item);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			
  			for(int k = 0; k < rating.length; k++)
  			{
  				System.out.println("User " + user[i] + "\tItem " + rating[k].getItemID()
  								 + "\tin Cache  " + rating[k].getRating()
  								 + "\tin DB " + getDBRating(user[i], rating[k].getItemID()) );
  			}  			
  			 
  		} // for
  		return pass;
  	
  	} // multiUserTest2()
    
    /**
  	 * illegalParamTest()
  	 * 1. item < 1 and item > maxItem
  	 * 2. user < 1 and user > maxUser
  	 * 3. itemList is null
  	 * @ return int  
  	 */
  	private int illegalParamTest()
  	{
  		System.out.println("Checking the illegal elements...");
  	    int pass = 1;
  	    Random r1 = new Random();
  		  		
  		int[] user = new int[5];
  		int[] item = new int[5];
  		
  		for (int i = 0; i < user.length; i++)
  		{
  			user[i] = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  		}
  		
  		// item < 1
  		item[0] = 2;  		
  		for (int i = 1; i < item.length; i++)
  		{
  			item[i]= item[i-1] - 1;
  		}
  		for (int i = 0; i < user.length; i++)
  		{
  			ItemRating[] rating = null;
  			try {
  				rating = engine.getRatingList(user[i], item);
  			} catch (CFIllegalListParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRatingList()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (rating != null) {
  				for ( int k = 0; k < rating.length; k++)
  				{
  					System.out.println("User " + user[i] + "\tItem " + rating[k].getItemID()
  								 + "\tin Cache  " + rating[k].getRating()
  								 + "\tin DB " + getDBRating(user[i], rating[k].getItemID()) );
  				}
  			}
  				
  		} // outer for
  		 
  		// item > maxItem
  		item[0] = maxItem - 1;
  		for (int i = 1; i < item.length; i++)
  		{
  			item[i]= item[i-1] + 1;
  		}
  		for (int i = 0; i < user.length; i++)
  		{
  			ItemRating[] rating = null;
  			try {
  				rating = engine.getRatingList(user[i], item);
  			} catch (CFIllegalListParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRatingList()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (rating != null) {
  				for ( int k = 0; k < rating.length; k++)
  				{
  					System.out.println("User " + user[i] + "\tItem " + rating[k].getItemID()
  								 + "\tin Cache  " + rating[k].getRating()
  								 + "\tin DB " + getDBRating(user[i], rating[k].getItemID()) );
  				}
  			}	
  		} // outer for
  		
  		Random r2 = new Random();
  		for (int i = 0; i < item.length; i++)
  		{
  			item[i] = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  		}
  		// user < 1
  		user[0] = 2;
  		for (int i = 1; i < user.length; i++)
  		{
  			user[i] = user[i-1] - 1;
  		}
  		for (int i = 0; i < user.length; i++)
  		{
  			ItemRating[] rating = null;
  			try {
  				rating = engine.getRatingList(user[i], item);
  			} catch (CFIllegalListParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRatingList()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if ( rating != null) {
  				for ( int k = 0; k < rating.length; k++)
  				{
  					System.out.println("User " + user[i] + "\tItem " + rating[k].getItemID()
  								 + "\tin Cache  " + rating[k].getRating()
  								 + "\tin DB " + getDBRating(user[i], rating[k].getItemID()) );
  				}
  			}
  				
  		} // outer for
  		
  		// user > maxUser
  		user[0] = maxUser - 1;
  		for (int i = 1; i < user.length; i++)
  		{
  			user[i] = user[i-1] + 1;
  		}
  		for (int i = 0; i < user.length; i++)
  		{
  			ItemRating[] rating = null;
  			try {
  				rating = engine.getRatingList(user[i], item);
  			} catch (CFIllegalListParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for setRatingList()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (rating != null) {
  				for ( int k = 0; k < rating.length; k++)
  				{
  					System.out.println("User " + user[i] + "\tItem " + rating[k].getItemID()
  								 + "\tin Cache  " + rating[k].getRating()
  								 + "\tin DB " + getDBRating(user[i], rating[k].getItemID()) );
  				}
  			}
  				
  		} // outer for
  		
  		// itemList is null
  		item = null;
  		ItemRating[] rating = null;
  		try {
  			rating = engine.getRatingList(user[0], item);
  		} catch (CFIllegalListParameterException e) {
  			System.out.println("Exception: " + e);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for setRatingList()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		  		
  		return pass;
  	} // illegalParamTest()
    
    /**
     * Get connection with database. 
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException
    {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");
         
        return DriverManager.getConnection(url, username, password);
        
    } // getConnection()
    
    /**
  	 * getItems()
  	 * Get rated items for a user
     * @return Vector
     * 
     */
    private Vector getItems(int user)
    {
    	Vector items = new Vector();
    	String query = "SELECT " + itemFieldName + " FROM " + ratingTableName  + " WHERE " 
    	 				  + userFieldName + " = " + user + " order by " + itemFieldName;
    	
    	Connection conn = null;		    
	    ResultSet rs= null; 	                          
	    
	    // Establish connection with databases
        try {
        	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);             
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();
        } catch (Exception e) {           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           while (rs != null && rs.next()) 
           		items.add(new Integer(rs.getInt(1))); 
 			rs.close();
 			conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
        
        return items;
    } // getItems()
    
    /**
  	 * getRating()
  	 * Get rating from database.
     * @return rating
     * 
     */
  	private float getDBRating(int user, int item)
    {
    	Connection conn = null;
    	float rating = -1;
    	String query = "SELECT " + ratingFieldName + " FROM " + ratingTableName
    				   + " WHERE " + userFieldName + " = " + user + " and "
    				   + itemFieldName + " = " + item; 
   	
	    ResultSet rs= null; 	                          
	    
	    // Establish connection with databases
        try {
        	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);           
            
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();
        } catch (Exception e) {           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           if (rs != null && rs.next()){ 
               rating = rs.getFloat(1);
            }
            rs.close();
        	conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        }
           
        return rating;
    
    } // getRating()
    
    /**
  	 * testSeq()
  	 */
  	public void testSeq()
  	{
  		System.out.println("############################");
  		System.out.println("Test for getRatingList() ...");
  		System.out.println("############################");
  		System.out.println("");
  		int pass = 1;
  		
  		Random r = new Random();
  		
  		System.out.println("Start Single Item Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = singleItemTest1(user);
			if (t < 0 )
				pass = -1;
		}
  	
  		for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = singleItemTest2(user);
			if (t < 0 )
				pass = -1;
		}
		
		System.out.println("Start 10 Items Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = tenItemTest(user);
			if (t < 0 )
				pass = -1;
		}	
		
		System.out.println("Start multi user Test ... ");
    	System.out.println("");
    	int t = multiUserTest1();
    	if (t < 0 )
			pass = -1;
				
    	t = multiUserTest2();
    	if (t < 0 )
			pass = -1;
			
		System.out.println("Start illegal param Test ... ");
    	System.out.println("");
    	t = illegalParamTest();
    	if (t < 0 )
			pass = -1;	
			
		System.out.println();
		if (pass >= 0) {
			System.out.println("------ Pass! getRatingList() Test! ------");
            System.err.println("------ Pass! getRatingList() Test! ------");
        } else {
			System.out.println("------ Fail! getRatingList() Test! ------");
            System.err.println("------ Fail! getRatingList() Test! ------");
        }
        System.out.println();
  	} // testSeq()
    
} // class GetRatingListTest