/* 
 * GetRatingTest.java
 * 
 * Version 1.1
 * 
 * 03/14/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */  
package org.recommender.test;
 
import java.sql.*;
import java.util.*;
import java.io.*;
import org.recommender.server.*;

/**
 * Test for getPredictedRating()
 * @author Yun Wang
 * 
 */
public class GetRatingTest{

	private CFEngine engine = null;
	
	private static String ratingTableName;
    private static String userFieldName;
    private static String itemFieldName;
    private static String ratingFieldName;
    
    private float minRating = 0;
    private float maxRating = 0;
    private static Statement stmt = null;
    
    private int maxUser = 0, maxItem = 0;
    
    public GetRatingTest(CFEngine s)
    {
  		engine = s;
  		
  		String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                + "CFServer.properties";
        
        SystemProperties.loadProperties(filename);
        
        minRating = SystemProperties.getFloat("CFServer.db.minRating", 0);
        maxRating = SystemProperties.getFloat("CFServer.db.maxRating", 0);
        
        // Load table and field names
        ratingTableName = SystemProperties.getString(
                "CFServer.db.ratingsTable", "");
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
        
    	String query = "SELECT MAX(" + userFieldName + "), MAX(" + itemFieldName +
        			") FROM " + ratingTableName; 
	    
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
              conn = getConnection();
              stmt = conn.createStatement();
              rs = stmt.executeQuery(query); 
            if (rs != null && rs.next()){ 
               maxUser = rs.getInt(1);
               maxItem = rs.getInt(2);
               
           }
           rs.close();
           conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
        } catch (IOException e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
      
  	} // GetRatingTest()
    
    /**
  	 * singleItemTest1()
  	 * Get 1 item (existed), check value in 2 ways
  	 * 1. getRating() -- directly from cache
  	 * 2. directly from database
  	 * @return int
  	 */
  	private int singleItemTest1(int user)
  	{
  		System.out.println("Check Existed single Item for User " + user + " ...");
  		int pass = 1;
  		
  		Vector v = getItems(user);
  		int length = v.size();
  	
  		if (length <= 0) 
  		{
  			System.out.println("\t no item available.");
  			return 0;
  		} 
  		 				
  		Random r = new Random();
  		int item = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  		ItemRating itRating = null;	
  	
  		try {
  			itRating = engine.getRating(user, item);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRating()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}	 
  		System.out.println("\tItem " + itRating.getItemID() + "\tgetRating(): " + itRating.getRating() 
  							+ "\tin DB: " + getRating(user, item));
  		System.out.println("");					
  		return pass;
  							
  	} // singleItemTest1()
	
	/**
  	 * singleItemTest2()
  	 * Get 1 item (randomly selected), check value in 2 ways
  	 * 1. getRating() -- directly from cache
  	 * 2. directly from database
  	 * @return int
  	 */
  	private int singleItemTest2(int user)
  	{
  		System.out.println("Check Randomly Selected single Item for User " + user + " ...");
  		int pass = 1;
  		  		  		 				
  		Random r = new Random();
  		int item = (int) (r.nextFloat() * (maxItem - 1) + 1);
  		ItemRating itRating = null;		
  	
  		try {
  			itRating = engine.getRating(user, item);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRating()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}	 
  		System.out.println("\tItem " + itRating.getItemID() + "\tgetRating(): " + itRating.getRating() 
  						+ "\tin DB: " + getRating(user, item));
  		System.out.println();
  		return pass;
  							
  	} // singleItemTest2()
  	
  	/**
  	 * tenItemTest()
  	 * Get 10 items (5 existed, 5 randomly), check values in two ways:
  	 * 1. getRating() -- directly from cache
  	 * 2. directly from database
  	 * @return int
  	 */
  	private int tenItemTest(int user)
  	{
  		System.out.println("Check 10 Items for User " + user + " ...");
  		int pass = 1;
  		
  		Random r = new Random();
  		Vector v = getItems(user);
  		int length = v.size();
  	
  		if (length <= 0) 
  		{
  			System.out.println("\t no item available.");
  			return 0;
  		}   		 				
  		
  		// for 5 existed items
  		for (int i = 0; i < 5; i++)
  		{
  			int item;
  			if ( i < 5 && length > 5)
  			{	
  				item = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  			} 
  			else if (i < 5 && length <= 5 && length > (i+1))
  			{
  				item = ((Integer) v.get(i)).intValue();  					
  			} 
  			else 
  			{
  				item = (int) (r.nextFloat() * (maxItem - 1) + 1);
  			}
  		
  			ItemRating itRating = null;	
  			try {
  				itRating = engine.getRating(user, item);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("\tItem " + itRating.getItemID() + "\tgetRating(): " + itRating.getRating() 
  							+ "\tin DB: " + getRating(user, item));
  							
  				 
  		} // for
  		
  		// for 5 randomly selected items
  		for (int i = 0; i < 5; i++)
  		{
  			int item = (int) (r.nextFloat() * (maxItem - 1) + 1);
  			
  			ItemRating itRating = null;	
  			try {
  				itRating = engine.getRating(user, item);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("\tItem " + itRating.getItemID() + "\tgetRating(): " + itRating.getRating() 
  							+ "\tin DB: " + getRating(user, item));
  				 
  		} // for
  		System.out.println("");
  		return pass;
  		
  	} // tenItemTest()
  	
  	/**
  	 * multiUserTest1()
  	 * Test 3 users for 1 same item.
  	 * @return int
  	 */
  	private int multiUserTest1()
  	{
  		System.out.println("Check multi users, for 1 same item ...");
  		int pass = 1;
  		
  		Random r = new Random();
  		int[] user = new int[3];
  		
  		for (int i = 0; i < 3; i++)
  		{
  			user[i] = (int) (r.nextFloat() * (maxUser - 1) + 1);
  		}
  		
  		Vector v = get3UserItems(user[0], user[1], user[2]);
  		int length = v.size();
  	
  		if (length <= 0) 
  		{
  			System.out.println("\t no item available.");
  			return 0;
  		}   		 				
  		
  		int item = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  		
  		System.out.println("Check multi users, each for Item " + item + " ...");
  		for ( int i = 0; i < 3; i++)
  		{
  			ItemRating itRating = null;	
  	
  			try {
  				itRating = engine.getRating(user[i], item);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			System.out.println("\tUser " + itRating.getUserID() + "\tgetRating(): " + itRating.getRating() 
  							+ "\tin DB: " + getRating(user[i], item));
  		} // for
  		System.out.println("");
  		return pass;
  		
  	} // multiUserTest1()
  	
	/**
  	 * multiUserTest2()
  	 * Test 5 users for 10 items:
  	 * @return int 
  	 */
  	private int multiUserTest2()
  	{
  		System.out.println("Check multi users, each for 10 items ...");
  		int pass = 1;
  		
  		Random r1 = new Random();
  		Random r2 = new Random();
  		
  		System.out.println("  existed 5 items and randomly selected 5 items "); 
  		for (int i = 0; i < 5; i++)
  		{
  			int user = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  			System.out.println("\t  For User " + user);
  			
  			// for 5 existed items
  			Vector v = getItems(user);
  			int length = v.size();
  			if (length <= 0) 
  			{
  				System.out.println("\t no item available.");
  				return 0;
  			} 
  			for ( int k = 0; k < 5; k++)
  			{
  				int item;
  				/*if ( length <= 5)
  				{
  					if (length > k )
  						item = ((Integer) v.get(k)).intValue();
  					else
  						item = 0;
  				}
  				else
  				{
  					item = ((Integer) v.get((int) (r2.nextFloat() * length))).intValue();
  				} */
  				if (k < 5 && length > 5)
  				{	
  					item = ((Integer) v.get((int) (r2.nextFloat() * length))).intValue();
  				} 
  				else if (k < 5 && length <= 5 && length > (k+1))
  				{
  					item = ((Integer) v.get(i)).intValue();  					
  				} 
  				else 
  				{
  					item = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  				}
  				
  				ItemRating itRating = null;	
  				try {
  					itRating = engine.getRating(user, item);
  				} catch (Exception e) {
  					pass = -1;
  					System.out.println("Error Testing for getRating()");
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}	 
  				System.out.println("\t\tItem " + itRating.getItemID() + "\tgetRating(): " + itRating.getRating() + "\tin DB: " 
  								+ getRating(user, item));
  			
  			}
  			
  			
  			// for 5 randomly selected items
  			for ( int k = 0; k < 5; k++)
  			{
  				int item = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  				ItemRating itRating = null;	  	
  				try {
  					itRating = engine.getRating(user, item);
  				} catch (Exception e) {
  					pass = -1;
  					System.out.println("Error Testing for getRating()");
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}	 
  				System.out.println("\t\tItem " + itRating.getItemID() + "\tgetRating(): " + itRating.getRating() 
  								+ "\tin DB: " + getRating(user, item));
  			}
  		
  		
  		} // for
  		System.out.println();
  		return pass;
  		
  	} // multiUserTest2()
  	
  	/**
  	 * illegalParamTest()
  	 * 1. item < 1 and item > maxItem
  	 * 2. user < 1 and user > maxUser
  	 * @ return int  
  	 */
  	private int illegalParamTest()
  	{
  		 Random r1 = new Random();
  		 
  		 System.out.println("Checking the illegal elements...");
  	     int pass = 1;
  	     
  	     // test for itemID < 1
  	  	for (int item = 0, k = 0; k < 5; k++, item--)
  	  	{
  	  		int u = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  	  		System.out.println("User " + u + "\tItem " + item
  			//					+ "\tgetRating(): " + itRating.getRating()
  								+ "\tin DB " + getRating(u, item));
  	  		ItemRating itRating = null;	
  			try {
  				itRating = engine.getRating(u, item);  					
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
            if (itRating != null)
  			    System.out.println("User " + itRating.getUserID() + "\tItem " + itRating.getItemID()
  								+ "\tgetRating(): " + itRating.getRating()
  								+ "\tin DB " + getRating(u, item));

  	 	}
  	 	
  	  // itemID > maxItem
  	  for (int item = maxItem + 1, k = 0; k < 5; k++, item++)
  	  {
  	  	   	int u = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  	  		ItemRating itRating = null;	
  			try {
  				itRating = engine.getRating(u, item);  					
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
            if (itRating != null)
  			    System.out.println("User " + itRating.getUserID() + "\tItem " + itRating.getItemID()
  								+ "\tgetRating(): " + itRating.getRating()
  								+ "\tin DB " + getRating(u, item));
  	  }
  	  
  	  Random r2 = new Random();
  		 
  	  // user < 1
  		for (int user = 0, k = 0; k < 5; k++, user--)
  	 	{
  	  		int i = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  	  		System.out.println("User " + user + "\tItem " + i 
  			//					+ "\tgetRating(): " + itRating.getRating()
  								+ "\tin DB " + getRating(user, i));  	  		  			
  			ItemRating itRating = null;	
  			try {
  				itRating = engine.getRating(user, i);  					
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			if (itRating != null)
  			    System.out.println("User " + itRating.getUserID() + "\tItem " + itRating.getItemID()
  								+ "\tgetRating(): " + itRating.getRating()
  								+ "\tin DB " + getRating(user, i));
  	  	} 
  	  	
  	  	// user > maxUser 
  	  	for (int user = maxUser + 1, k = 0; k < 5; k++, user++)
  	    {
  	  		int i = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  	  		System.out.println("User " + user + "\tItem " + i 
  			//					+ "\tgetRating(): " + itRating.getRating()
  								+ "\tin DB " + getRating(user, i));
  	  		ItemRating itRating = null;
  			try {
  				itRating = engine.getRating(user, i);  					
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}	 
  			if (itRating != null)
  			    System.out.println("User " + itRating.getUserID() + "\tItem " + itRating.getItemID()
  								+ "\tgetRating(): " + itRating.getRating()
  								+ "\tin DB " + getRating(user, i));
  	    }
  	    System.out.println();
  	    return pass;	
  	 		  
  	} // illegleParamTest()
  	
	
	/**
     * Get connection with database. 
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException
    {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");
         
        return DriverManager.getConnection(url, username, password);
        
        // We now use a connection pooling approach. The real SQL driver
        // is set in the static initializer of this class.
       // return DriverManager.getConnection("jdbc:jdc:jdcpool");
    } // getConnection()
    
  	/**
  	 * getRating()
  	 * Get rating from database.
     * @return rating
     * 
     */
  	private float getRating(int user, int item)
    {
    	float rating = -1;
    	String query = "SELECT " + ratingFieldName + " FROM " + ratingTableName
    				   + " WHERE " + userFieldName + " = " + user + " and "
    				   + itemFieldName + " = " + item; 
   
   		Connection conn = null;	
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
        	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
              // precompileSQLStatements(conn);          
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();      
        } catch (Exception e) {
           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           if (rs != null && rs.next()){ 
               rating = rs.getFloat(1);
            }
            rs.close();
            conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
    // System.out.println(""+rating);   
        return rating;
    
    } // getRating()
    
    /**
  	 * getItems()
  	 * Get rated items for a user
     * @return Vector
     * 
     */
    private Vector getItems(int user)
    {
    	Vector items = new Vector();
    	String query = "SELECT " + itemFieldName + " FROM " + ratingTableName  + " WHERE " 
    	 				  + userFieldName + " = " + user + " order by " + itemFieldName;
    	
    	Connection conn = null;			    
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
            conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
              // precompileSQLStatements(conn);          
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();      
        } catch (Exception e) {
            System.out.println("Exception:" + e);
            e.printStackTrace();
        }
        
        try {
           while (rs != null && rs.next()) 
           		items.add(new Integer(rs.getInt(1))); 
 			rs.close();
 			conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
        
        return items;
    } // getItems()
    
    /**
  	 * getItems()
  	 * Get rated items for 3 users
     * @return Vector
     * 
     */
    private Vector get3UserItems(int u1, int u2, int u3)
    {
    	Vector items = new Vector();
    	String query = "SELECT " + ratingTableName + "1." + itemFieldName + " FROM " 
    					+ ratingTableName + " " + ratingTableName + "1, "
    					+ ratingTableName + " " + ratingTableName + "2, " 
    					+ ratingTableName + " " + ratingTableName + "3 WHERE "    
    					+ ratingTableName + "1." + userFieldName + " = " + u1 + " and "
    					+ ratingTableName + "2." + userFieldName + " = " + u2 + " and "
    					+ ratingTableName + "3." + userFieldName + " = " + u3 + " and "
    					+ ratingTableName + "1." + itemFieldName + " = " 
    					+ ratingTableName + "2." + itemFieldName + " and " 
    					+ ratingTableName + "1." + itemFieldName + " = " 
    					+ ratingTableName + "3." + itemFieldName + " and "
    					+ ratingTableName + "2." + itemFieldName + " = " 
    					+ ratingTableName + "3." + itemFieldName
    					+ " order by " + ratingTableName + "1." + itemFieldName;
    				    
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
            conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
              // precompileSQLStatements(conn);          
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();    
        } catch (Exception e) {
            System.out.println("Exception:" + e);
            e.printStackTrace();
        }
        
        try {
           while (rs != null && rs.next()) 
           		items.add(new Integer(rs.getInt(1))); 
 			rs.close();
 			conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
        
        return items;
    
    }
    
    /**
  	 * testSeq()
  	 */
    public void testSeq()
    {
    	System.out.println("############################");
  		System.out.println("Test for getRating() ...");
  		System.out.println("############################");
  		System.out.println(""); 
  		int pass = 1;
  		
  		Random r = new Random();
    	
    	System.out.println("  Start Single Item Test ... ");
    	System.out.println("");
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = singleItemTest1(user);
			if (t < 0 )
				pass = -1;
		}
		for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = singleItemTest2(user);
			if (t < 0 )
				pass = -1;
		}
		
		System.out.println("Start 10 Items Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = tenItemTest(user);
			if (t < 0 )
				pass = -1;
		}	
		
		System.out.println("  Start multiUser Test 1  -- 3 users have 5 same items ... ");
    	System.out.println("");
		int t = multiUserTest1();
		if (t < 0 )
			pass = -1;
		
		System.out.println("  Start multiUser Test 2 -- multi users have different items ... ");
    	System.out.println("");
		t = multiUserTest2();
		if (t < 0 )
			pass = -1;
		
		System.out.println("Start illegal param Test ... ");
    	System.out.println("");
    	t = illegalParamTest();
    	if (t < 0 )
			pass = -1;
    	
    	System.out.println();
		if (pass >= 0){
			System.out.println("------ Pass! getRating() Test! ------");
            System.err.println("------ Pass! getRating() Test! ------");
        } else {
			System.out.println("------ Fail! getRating() Test! ------");
            System.err.println("------ Fail! getRating() Test! ------");
        }
        System.out.println();
    } // testSeq()

} // class getRatingTest