/* 
 * GetRecommendationsByTypeTest.java
 * 
 * Version 1.1
 * 
 * 03/19/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */  

package org.recommender.test;
 
import java.sql.*;
import java.util.*;
import java.io.*;
import org.recommender.server.*;

/**
 * Test for getRecommendationsByType()
 * @author Yun Wang
 * 
 */
public class GetRecommendationsByTypeTest{

private CFEngine engine = null;
	
	private static String ratingTableName;
    private static String userFieldName;
    private static String itemFieldName;
    private static String ratingFieldName;
    
    private static String itemTypeTableName; 
    private static String typeFieldName;
    
    private static Statement stmt = null;
    private int maxUser = 0, maxItem = 0, maxType = 0;
    
        
	public GetRecommendationsByTypeTest(CFEngine s)
	{
		engine = s;
  		if (engine == null)
  			System.out.println("engine is null");
  			
  		String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                + "CFServer.properties";
        
        SystemProperties.loadProperties(filename);
        
        // Load table and field names
        itemTypeTableName = SystemProperties.getString("CFServer.db.itemTypeTable", ""); 
        typeFieldName = SystemProperties.getString("CFServer.db.typeField", ""); 
        
        ratingTableName = SystemProperties.getString(
                "CFServer.db.ratingsTable", "");
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
        
    	String query = "SELECT MAX(" + userFieldName + "), MAX(" + itemFieldName +
        			") FROM " + ratingTableName; 
	    
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
            conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
        	if (rs != null && rs.next()){ 
               maxUser = rs.getInt(1);
               maxItem = rs.getInt(2);
               
           }
           rs.close();
         //  conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
        } catch (IOException e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
       /* try {
           if (rs != null && rs.next()){ 
               maxUser = rs.getInt(1);
               maxItem = rs.getInt(2);
               rs.close();
           }           
          
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } */
        
        
        query = "SELECT MAX(" + typeFieldName + ") FROM " + itemTypeTableName; 
	//    rs= null; 	                          
	      
        try {
        	//conn = getConnection();
          	stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
            if (rs != null && rs.next()){ 
               maxType = rs.getInt(1);               
           }
           rs.close();
           conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
        } catch (Exception e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
      /*  try {
           if (rs != null && rs.next()){ 
               maxType = rs.getInt(1);
               rs.close();
           }
           
           conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } */
     //   System.out.println("maxUser = " + maxUser + "\tmaxItem = " + maxItem + "\tmaxType = " + maxType);
             
	} // GetRecommendationsByTypeTest()

	/**
  	 * tenItemTest1()
  	 * Get 10 items for 1 type, 
  	 * compare with predicted value and the one in DB
  	 * @return int
  	 */
  	public int tenItemTest1(int user)
  	{
  		System.out.println("Check 10 Recommendations for User " + user + " ...");
  		int pass = 1;
  		
  		Random rdm = new Random();
  		int type = (int) (rdm.nextFloat() * maxType);
  		System.out.println("  From offset 0 for type " + type); 
  		
  		ItemPrediction[] pred1 = null;
  		try {
  			pred1 = engine.getRecommendationsByType(user, 10, 0, type);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRecommendationsByType()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		if (pred1 == null)
  		{
  			System.out.println("No recommendations available");
  		} 
  		else
  		{ 
  			for (int i = 0; i < pred1.length; i++)
  			{
  				ItemPrediction r = null;
  				int item = pred1[i].getItemID();
  				try {
  					r = engine.getPredictedRating(user, item);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				System.out.println("\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  			}
  		}
  		
  		
  		type =  (int) (rdm.nextFloat() * maxType);
  		
  		System.out.println("  From offset 20 for type " + type);
  		
  		ItemPrediction[] pred2 = null;
  		try {
  			pred2 = engine.getRecommendationsByType(user, 10, 20, type);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRecommendationsByType()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		if (pred2 == null)
  		{
  			System.out.println("No recommendations available");
  		} 
  		else
  		{ 
  			for (int i = 0; i < pred2.length; i++)
  			{
  				ItemPrediction r = null;
  				int item = pred2[i].getItemID();
  				try {
  					r = engine.getPredictedRating(user, item);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				System.out.println("\tItem " + item + "\tgetRecommendationsByType(): " + pred2[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  			}
  		}
  		System.out.println();
  		return pass;
  		
  	} // tenItemTest1()
  	
  	/**
  	 * tenItemTest2()
  	 * Get 10 items for 3 types, 
  	 * compare with predicted value and the one in DB
  	 * @return int
  	 */
  	public int tenItemTest2(int user)
  	{
  		System.out.println("Check 10 Recommendations for User " + user + " ...");
  		int pass = 1;
  		
  		Random rdm = new Random();
  		System.out.println("  From offset 0");
  		
  		for (int k = 0; k < 3; k++)
  		{
  			int type = (int) (rdm.nextFloat() * maxType);
  			System.out.println("\tFor type " + type); 
  		
  			ItemPrediction[] pred1 = null;
  			try {
  				pred1 = engine.getRecommendationsByType(user, 10, 0, type);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\t\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		
  		System.out.println("  From offset 20");
  		
  		for (int k = 0; k < 3; k++)
  		{
  			int type = (int) (rdm.nextFloat() * maxType);
  			System.out.println("\tFor type " + type); 
  		
  			ItemPrediction[] pred1 = null;
  			try {
  				pred1 = engine.getRecommendationsByType(user, 10, 20, type);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\t\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		System.out.println();
  		return pass;
  		
  	} // tenItemTest2()
  	
  	/**
  	 * twentyItemTest1()
  	 * Get 20 items for 1 type, 
  	 * compare with predicted value and the one in DB
  	 * @return int
  	 */
  	public int twentyItemTest1(int user)
  	{
  		System.out.println("Check 20 Recommendations for User " + user + " ...");
  		int pass = 1;
  		
  		Random rdm = new Random();
  		int type = (int) (rdm.nextFloat() * maxType);
  		System.out.println("  From offset 5 for type " + type); 
  		
  		ItemPrediction[] pred1 = null;
  		try {
  			pred1 = engine.getRecommendationsByType(user, 20, 5, type);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRecommendationsByType()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		if (pred1 == null)
  		{
  			System.out.println("No recommendations available");
  		} 
  		else
  		{ 
  			for (int i = 0; i < pred1.length; i++)
  			{
  				ItemPrediction r = null;
  				int item = pred1[i].getItemID();
  				try {
  					r = engine.getPredictedRating(user, item);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				System.out.println("\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  			}
  		}
  		
  		
  		type =  (int) (rdm.nextFloat() * maxType);
  		
  		System.out.println("  From offset 30 for type " + type);
  		
  		ItemPrediction[] pred2 = null;
  		try {
  			pred2 = engine.getRecommendationsByType(user, 20, 30, type);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRecommendationsByType()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		if (pred2 == null)
  		{
  			System.out.println("No recommendations available");
  		} 
  		else
  		{ 
  			for (int i = 0; i < pred2.length; i++)
  			{
  				ItemPrediction r = null;
  				int item = pred2[i].getItemID();
  				try {
  					r = engine.getPredictedRating(user, item);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				System.out.println("\tItem " + item + "\trecommended " + pred2[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  			}
  		}
  		System.out.println();
  		return pass;
  		
  	} //twentyItemTest1()
  	
  	/**
  	 * twentyItemTest2()
  	 * Get 20 items for 3 types, 
  	 * compare with predicted value and the one in DB
  	 * @return int
  	 */
  	public int twentyItemTest2(int user)
  	{
  		System.out.println("Check 20 Recommendations for User " + user + " ...");
  		int pass = 1;
  		
  		Random rdm = new Random();
  		System.out.println("  From offset 5");
  		
  		for (int k = 0; k < 3; k++)
  		{
  			int type = (int) (rdm.nextFloat() * maxType);
  			System.out.println("\tFor type " + type); 
  		
  			ItemPrediction[] pred1 = null;
  			try {
  				pred1 = engine.getRecommendationsByType(user, 20, 5, type);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\t\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		
  		System.out.println("  From offset 30");
  		
  		for (int k = 0; k < 3; k++)
  		{
  			int type = (int) (rdm.nextFloat() * maxType);
  			System.out.println("\tFor type " + type); 
  		
  			ItemPrediction[] pred1 = null;
  			try {
  				pred1 = engine.getRecommendationsByType(user, 20, 30, type);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\t\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		System.out.println();
  		return pass;
  		
  	} //twentyItemTest2()
  	
  	/**
  	 * fiftyItemTest1()
  	 * Get 20 items for 1 type, 
  	 * compare with predicted value and the one in DB
  	 * @return int
  	 */
  	public int fiftyItemTest1(int user)
  	{
  		System.out.println("Check 50 Recommendations for User " + user + " ...");
  		int pass = 1;
  		
  		Random rdm = new Random();
  		int type = (int) (rdm.nextFloat() * maxType);
  		System.out.println("  From offset 10 for type " + type); 
  		
  		ItemPrediction[] pred1 = null;
  		try {
  			pred1 = engine.getRecommendationsByType(user, 50, 10, type);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRecommendationsByType()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		if (pred1 == null)
  		{
  			System.out.println("No recommendations available");
  		} 
  		else
  		{ 
  			for (int i = 0; i < pred1.length; i++)
  			{
  				ItemPrediction r = null;
  				int item = pred1[i].getItemID();
  				try {
  					r = engine.getPredictedRating(user, item);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				System.out.println("\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  			}
  		}
  		
  		
  		type =  (int) (rdm.nextFloat() * maxType);
  		
  		System.out.println("  From offset 50 for type " + type);
  		
  		ItemPrediction[] pred2 = null;
  		try {
  			pred2 = engine.getRecommendationsByType(user, 50, 60, type);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRecommendationsByType()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		if (pred2 == null)
  		{
  			System.out.println("No recommendations available");
  		} 
  		else
  		{ 
  			for (int i = 0; i < pred2.length; i++)
  			{
  				ItemPrediction r = null;
  				int item = pred2[i].getItemID();
  				try {
  					r = engine.getPredictedRating(user, item);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				System.out.println("\tItem " + item + "\tgetRecommendationsByType(): " + pred2[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  			}
  		}
  		System.out.println();
  		return pass;
  		
  	} //fiftyItemTest1()
	
	
	/**
  	 * fiftyItemTest2()
  	 * Get 50 items for 3 types, 
  	 * compare with predicted value and the one in DB
  	 * @return int
  	 */
  	public int fiftyItemTest2(int user)
  	{
  		System.out.println("Check 50 Recommendations for User " + user + " ...");
  		int pass = 1;
  		
  		Random rdm = new Random();
  		System.out.println("  From offset 10");
  		
  		for (int k = 0; k < 3; k++)
  		{
  			int type = (int) (rdm.nextFloat() * maxType);
  			System.out.println("\tFor type " + type); 
  		
  			ItemPrediction[] pred1 = null;
  			try {
  				pred1 = engine.getRecommendationsByType(user, 50, 10, type);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\t\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		
  		System.out.println("  From offset 60");
  		
  		for (int k = 0; k < 3; k++)
  		{
  			int type = (int) (rdm.nextFloat() * maxType);
  			System.out.println("\tFor type " + type); 
  		
  			ItemPrediction[] pred1 = null;
  			try {
  				pred1 = engine.getRecommendationsByType(user, 50, 60, type);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\t\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		System.out.println();
  		return pass;
  		
  	} //fiftyItemTest2()
  	
  	/**
  	 * multiUserTest1()
  	 * Test 5 users for 10 recommendations:
  	 * 1. offset 0
  	 * 2. offset 20
  	 * @return int
  	 */
  	public int multiUserTest1()
  	{
  		System.out.println("Check multi users for 10 Recommendations ...");
  		int pass = 1;
  		
  		Random rdm = new Random();
  		int type = (int) (rdm.nextFloat() * maxType);
  		
  		System.out.println("  From offset 0 for type " + type);
  		
  		for (int k = 0; k < 3; k++)
  		{
  			int user = (int) (rdm.nextFloat() * (maxUser - 1) + 1);
  			System.out.println("\tFor User " + user); 
  		
  			ItemPrediction[] pred1 = null;
  			try {
  				pred1 = engine.getRecommendationsByType(user, 10, 0, type);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\t\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		
  		Random rdm2 = new Random();
  		type = (int) (rdm2.nextFloat() * maxType);
  		
  		System.out.println("  From offset 20 for type " + type);
  		
  		for (int k = 0; k < 3; k++)
  		{
  			int user = (int) (rdm2.nextFloat() * (maxUser - 1) + 1);
  			System.out.println("\tFor User " + user); 
  		
  			ItemPrediction[] pred1 = null;
  			try {
  				pred1 = engine.getRecommendationsByType(user, 10, 20, type);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\t\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		System.out.println();
  		return pass;
  		
  	} // multiUserTest1()
  	
  	/**
  	 * multiUserTest2()
  	 * Test 5 users for 20 recommendations:
  	 * 1. offset 5
  	 * 2. offset 30
  	 * @return int
  	 */
  	public int multiUserTest2()
  	{
  		System.out.println("Check multi users for 20 Recommendations ...");
  		int pass = 1;
  		
  		Random rdm = new Random();
  		int type = (int) (rdm.nextFloat() * maxType);
  		
  		System.out.println("  From offset 5 for type " + type);
  		
  		for (int k = 0; k < 3; k++)
  		{
  			int user = (int) (rdm.nextFloat() * (maxUser - 1) + 1);
  			System.out.println("\tFor User " + user); 
  		
  			ItemPrediction[] pred1 = null;
  			try {
  				pred1 = engine.getRecommendationsByType(user, 20, 5, type);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\t\tItem " + item + "\tgetRecommendationsByType() " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		
  		Random rdm2 = new Random();
  		type = (int) (rdm2.nextFloat() * maxType);
  		
  		System.out.println("  From offset 30 for type " + type);
  		
  		for (int k = 0; k < 3; k++)
  		{
  			int user = (int) (rdm2.nextFloat() * (maxUser - 1) + 1);
  			System.out.println("\tFor User " + user); 
  		
  			ItemPrediction[] pred1 = null;
  			try {
  				pred1 = engine.getRecommendationsByType(user, 20, 30, type);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\t\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		System.out.println();
  		return pass;
  		
  	} // multiUserTest()
  	
  	/**
  	 * multiUserTest3()
  	 * Test 5 users for 50 recommendations.
  	 * 1. offset 10
  	 * 2. offset 60
  	 * @return int
  	 */
  	public int multiUserTest3()
  	{
  		System.out.println("Check multi users for 20 Recommendations ...");
  		int pass = 1;
  		
  		Random rdm = new Random();
  		int type = (int) (rdm.nextFloat() * maxType);
  		
  		System.out.println("  From offset 10 for type " + type);
  		
  		for (int k = 0; k < 3; k++)
  		{
  			int user = (int) (rdm.nextFloat() * (maxUser - 1) + 1);
  			System.out.println("\tFor User " + user); 
  		
  			ItemPrediction[] pred1 = null;
  			try {
  				pred1 = engine.getRecommendationsByType(user, 50, 10, type);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\t\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		
  		Random rdm2 = new Random();
  		type = (int) (rdm2.nextFloat() * maxType);
  		
  		System.out.println("  From offset 60 for type " + type);
  		
  		for (int k = 0; k < 3; k++)
  		{
  			int user = (int) (rdm2.nextFloat() * (maxUser - 1) + 1);
  			System.out.println("\tFor User " + user); 
  		
  			ItemPrediction[] pred1 = null;
  			try {
  				pred1 = engine.getRecommendationsByType(user, 50, 60, type);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\t\tItem " + item + "\tgetRecommendationsByType(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		System.out.println();
  		return pass;
  		
  	} // multiUserTest3()
  	
  	/**
  	 * illegalParamTest()
  	 * 1. item < 1 and item > maxItem
  	 * 2. user < 1 and user > maxUser
  	 * 3. offset < 0
  	 * 4. type < 0 and type > maxType
  	 * @ return int
  	 */
  	public int illegalParamTest()
  	{
  		int pass = 1;
  		System.out.println("Checking the illegal elements...");
  		
  		Random rdm = new Random();
  		// user < 1
  		for (int user = 0, k = 0; k < 5; k++, user--)
  	 	{
  	  		int type = (int) (rdm.nextFloat() * maxType);
  	  		ItemPrediction[] pred1 = null;
  	  		System.out.println("  5 recommendation from offset 0 for User: " + user );
  			try {
  				pred1 = engine.getRecommendationsByType(user, 5, 0, type);
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					int item = pred1[i].getItemID();
  					System.out.println("\tItem " + item + "\tgetRecommendationsByType(): " 
  									+ pred1[i].getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  	  	
  	  	
  	  	// user > maxUser 
  	  	for (int user = maxUser + 1, k = 0; k < 5; k++, user++)
  	    {
  	    	int type = (int) (rdm.nextFloat() * maxType);
  	    	ItemPrediction[] pred1 = null;
  	    	System.out.println("  5 recommendation from offset 0 for User: " + user );
  			try {
  				pred1 = engine.getRecommendationsByType(user, 5, 0, type);
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					int item = pred1[i].getItemID();
  					System.out.println("\tItem " + item + "\tgetRecommendationsByType(): " 
  								+ pred1[i].getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  	  	}
  	  	
  	  	// offset < 0
  	  	Random rdm2 = new Random();
  	  	for (int offset = -1, k = 0; k < 5; k++, offset--)
  	    {
  	    	int user = (int) (rdm2.nextFloat() * (maxUser - 1) + 1);
  	    	int type = (int) (rdm.nextFloat() * maxType);
  	    	ItemPrediction[] pred1 = null;
  	    	System.out.println("  5 recommendation from offset" + offset + " for User: " + user );
  			try {
  				pred1 = engine.getRecommendationsByType(user, 10, offset, type);
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					int item = pred1[i].getItemID();
  					System.out.println("\tItem " + item + "\tgetRecommendationsByType(): " 
  									+ pred1[i].getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  	  	}
  	  	
  	  	// number < 1
  	  	for (int number = 0, k = 0; k < 5; k++, number--)
  	    {
  	    	int user = (int) (rdm2.nextFloat() * (maxUser - 1) + 1);
  	    	int type = (int) (rdm.nextFloat() * maxType);
  	    	
  	    	ItemPrediction[] pred1 = null;
  	    	System.out.println("  " + number + " recommendation from offset 0 for User: " + user );
  			try {
  				pred1 = engine.getRecommendationsByType(user, number, 0, type);
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					int item = pred1[i].getItemID();
  					System.out.println("\tItem " + item + "\tgetRecommendationsByType(): " 
  									+ pred1[i].getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  	  	}
  	  	
  	  	// type < 0
  	  	for (int type = -1, k = 0; k < 5; k++, type--)
  	 	{
  	 		int user = (int) (rdm2.nextFloat() * (maxUser - 1) + 1);
  	 		ItemPrediction[] pred1 = null;
  	    	System.out.println("  5 recommendation of type " + type + " for User: " + user );
  			try {
  				pred1 = engine.getRecommendationsByType(user, 5, 0, type);
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					int item = pred1[i].getItemID();
  					System.out.println("\tItem " + item + "\tgetRecommendationsByType(): " 
  									+ pred1[i].getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  	 	
  	 	
  	 	}
  		
  		// type > maxType
  	  	for (int type = maxType + 1, k = 0; k < 5; k++, type++)
  	 	{
  	 		int user = (int) (rdm2.nextFloat() * (maxUser - 1) + 1);
  	 		ItemPrediction[] pred1 = null;
  	    	System.out.println("  5 recommendation of type " + type + " for User: " + user );
  			try {
  				pred1 = engine.getRecommendationsByType(user, 5, 0, type);
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendationsByType()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					int item = pred1[i].getItemID();
  					System.out.println("\tItem " + item + "\tgetRecommendationsByType(): " 
  									+ pred1[i].getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  	 	
  	 	
  	 	}
  	 	System.out.println();
  	 	return pass;
  		
  	} // illegalParamTest()
	
	/**
     * Get connection with database. 
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException
    {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");
        return DriverManager.getConnection(url, username, password);
        
        // We now use a connection pooling approach. The real SQL driver
        // is set in the static initializer of this class.
       // return DriverManager.getConnection("jdbc:jdc:jdcpool");
    } // getConnection()
    
    /**
  	 * getRating()
  	 * Get rating from database.
     * @return rating
     * 
     */
    private float getRating(int user, int item)
    {
    	float rating = -1;
    	String query = "SELECT " + ratingFieldName + " FROM " + ratingTableName
    				   + " WHERE " + userFieldName + " = " + user + " and "
    				   + itemFieldName + " = " + item; 
      
   		Connection conn = null;	
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
            conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
              // precompileSQLStatements(conn);          
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();     
        } catch (Exception e) {           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           if (rs != null && rs.next()){ 
               rating = rs.getFloat(1);
            }
            rs.close();
            conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
    // System.out.println(""+rating);   
        return rating;
    
    } // getRating()

	/**
  	 * testSeq()
  	 */
	public void testSeq()
    {
    	System.out.println("###################################");
  		System.out.println("Test for getRecommendationsByType() ...");
  		System.out.println("###################################");
  		System.out.println(""); 
  		int pass = 1;
  		
  		Random r = new Random();
    	
    	System.out.println("Start 10 Items Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			
			int t = tenItemTest1(user);
			if (t <= 0)
				pass = -1;
		}
		for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = tenItemTest2(user);
			if (t <= 0)
				pass = -1;
		}
		
		System.out.println("Start 20 Items Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = twentyItemTest1(user);
			if (t <= 0)
				pass = -1;
		}
		for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = twentyItemTest2(user);
			if (t <= 0)
				pass = -1;
		}
		
		System.out.println("Start 50 Items Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = fiftyItemTest1(user);
			if (t <= 0)
				pass = -1;
		}
		
		for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = fiftyItemTest2(user);
			if (t <= 0)
				pass = -1;
		} 
		
		System.out.println("  Start multiUser Test 1  -- 5 users have 10 recommendations ... ");
    	System.out.println("");
		int t = multiUserTest1();
		if (t <= 0)
			pass = -1;
		
		System.out.println("  Start multiUser Test 1  -- 5 users have 20 recommendations ... ");
    	System.out.println("");
		t = multiUserTest2();
		if (t <= 0)
			pass = -1;
		
		System.out.println("  Start multiUser Test 1  -- 5 users have 50 recommendations ... ");
    	System.out.println("");
		t = multiUserTest3();
		if (t <= 0)
			pass = -1;
		
		System.out.println("  Start illegal param Test ... ");
    	System.out.println("");
    	t = illegalParamTest();
    	if (t <= 0)
			pass = -1;
			System.out.println();
		
		if (pass > 0){
			System.out.println("------ Pass! getRecommendationsByType() Test! ------");
            System.err.println("------ Pass! getRecommendationsByType() Test! ------");
        } else {
			System.out.println("------ Fail! getRecommendationsByType() Test! ------");
            System.err.println("------ Fail! getRecommendationsByType() Test! ------");
        }
        System.out.println();
	}
	
} // class GetRecommendationsByTypeTest