/* 
 * GetRecommendationsTest.java
 * 
 * Version 1.1
 * 
 * 03/17/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */  
package org.recommender.test;
 
import java.sql.*;
import java.util.*;
import java.io.*;
import org.recommender.server.*;

/**
 * Test for getRecommendations()
 * @author Yun Wang
 * 
 */
public class GetRecommendationsTest{

	private CFEngine engine = null;
	
	private static String ratingTableName;
    private static String userFieldName;
    private static String itemFieldName;
    private static String ratingFieldName;
    
    private static Statement stmt = null;
    
    private int maxUser = 0, maxItem = 0;
    
    public GetRecommendationsTest(CFEngine s)
    {
    	engine = s;
  		if (engine == null)
  			System.out.println("engine is null");
  			
  		String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                + "CFServer.properties";
        
        SystemProperties.loadProperties(filename);
             
        // Load table and field names
        ratingTableName = SystemProperties.getString(
                "CFServer.db.ratingsTable", "");
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
        
    	String query = "SELECT MAX(" + userFieldName + "), MAX(" + itemFieldName +
        			") FROM " + ratingTableName; 
	    
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
      		conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
            if (rs != null && rs.next()){ 
               maxUser = rs.getInt(1);
               maxItem = rs.getInt(2);
               
           	}
           	rs.close();
           	conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
        } catch (IOException e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
    
    } // GetRecommendationsTest()
    
    /**
  	 * tenItemTest()
  	 * Get 10 item, compare with predicted value and the one in DB
  	 * @return int
  	 */
  	public int tenItemTest(int user)
  	{
  		int pass = 1;
  		System.out.println("Check 10 Recommendations for User " + user + " ...");
  		System.out.println("  From offset 0"); 
  		
  		ItemPrediction[] pred1 = null;
  		try {
  			pred1 = engine.getRecommendations(user, 10, 0);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRecommendations()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		if (pred1 == null)
  		{
  			System.out.println("No recommendations available");
  		} 
  		else
  		{ 
  			for (int i = 0; i < pred1.length; i++)
  			{
  				ItemPrediction r = null;
  				int item = pred1[i].getItemID();
  				try {
  					r = engine.getPredictedRating(user, item);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				System.out.println("\tItem " + item + "\tgetRecommendations() " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  			}
  		}
  		
  		System.out.println("  From offset 50"); 
  		ItemPrediction[] pred2 = null;
  		try {
  			pred2 = engine.getRecommendations(user, 10, 50);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRecommendations()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		if (pred2 == null)
  		{
  			System.out.println("No recommendations available");
  		} 
  		else
  		{ 
  			for (int i = 0; i < pred2.length; i++)
  			{
  				ItemPrediction r = null;
  				int item = pred2[i].getItemID();
  				try {
  					r = engine.getPredictedRating(user, item);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				System.out.println("\tItem " + item + "\tgetRecommendations(): " + pred2[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  			}
  		}
  		System.out.println();
  		return pass;
  	} // tenItemTest()
    
    
    /**
  	 * twentyItemTest()
  	 * Get 20 item, compare with predicted value and the one in DB
  	 * @return int
  	 */
  	public int twentyItemTest(int user)
  	{
  		int pass = 1;
  		System.out.println("Check 20 Recommendations for User " + user + " ...");
  		System.out.println("  From offset 10"); 
  		
  		ItemPrediction[] pred1 = null;
  		try {
  			pred1 = engine.getRecommendations(user, 20, 10);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRecommendations()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		if (pred1 == null)
  		{
  			System.out.println("No recommendations available");
  		} 
  		else
  		{ 
  			for (int i = 0; i < pred1.length; i++)
  			{
  				ItemPrediction r = null;
  				int item = pred1[i].getItemID();
  				try {
  					r = engine.getPredictedRating(user, item);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				System.out.println("\tItem " + item + "\tgetRecommendations(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  			}
  		}
  		
  		System.out.println("  From offset 100"); 
  		ItemPrediction[] pred2 = null;
  		try {
  			pred2 = engine.getRecommendations(user, 20,100);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRecommendations()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		if (pred2 == null)
  		{
  			System.out.println("No recommendations available");
  		} 
  		else
  		{ 
  			for (int i = 0; i < pred2.length; i++)
  			{
  				ItemPrediction r = null;
  				int item = pred2[i].getItemID();
  				try {
  					r = engine.getPredictedRating(user, item);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				System.out.println("\tItem " + item + "\tgetRecommendations(): " + pred2[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  			}
  		}
  		System.out.println();
  		return pass;
  		
  	} // twentyItemTest()
  	
  	/**
  	 * fiftyItemTest()
  	 * Get 50 item, compare with predicted value and the one in DB
  	 * @return int
  	 */
  	public int fiftyItemTest(int user)
  	{
  		int pass = 1;
  		System.out.println("Check 50 Recommendations for User " + user + " ...");
  		System.out.println("  From offset 30"); 
  		
  		ItemPrediction[] pred1 = null;
  		try {
  			pred1 = engine.getRecommendations(user, 50, 30);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRecommendations()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		if (pred1 == null)
  		{
  			System.out.println("No recommendations available");
  		} 
  		else
  		{ 
  			if (pred1.length < 1)
  				System.out.println("prediction length is 0");
  			for (int i = 0; i < pred1.length; i++)
  			{
  				ItemPrediction r = null;
  				int item = pred1[i].getItemID();
  				try {
  					r = engine.getPredictedRating(user, item);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				System.out.println("\tItem " + item + "\tgetRecommendations() " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  			}
  		}
  		
  		System.out.println("  From offset 300"); 
  		ItemPrediction[] pred2 = null;
  		try {
  			pred2 = engine.getRecommendations(user, 50, 300);
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for getRecommendations()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		if (pred2 == null)
  		{
  			System.out.println("No recommendations available");
  		} 
  		else
  		{ 
  			if (pred2.length < 1)
  				System.out.println("prediction length is 0");
  			for (int i = 0; i < pred2.length; i++)
  			{
  				ItemPrediction r = null;
  				int item = pred2[i].getItemID();
  				try {
  					r = engine.getPredictedRating(user, item);
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  				System.out.println("\tItem " + item + "\tgetRecommendations() " + pred2[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  			}
  		}
  		System.out.println();
  		return pass;
  		
  	} // fiftyItemTest()
  	
  	/**
  	 * multiUserTest1()
  	 * Test 5 users for 10 recommendations:
  	 * 1. from offset 0
  	 * 2. from offset 50
  	 * @return int 
  	 */
  	public int multiUserTest1()
  	{
  		System.out.println("Check multi users for 10 recommendations ...");
  		int pass = 1;
  		Random rdm = new Random();
  		
  		System.out.println("  From offset 0"); 
  		
  		for (int k = 0; k < 5; k++)
  		{
  			int user = (int) (rdm.nextFloat() * (maxUser - 1) + 1);
  			ItemPrediction[] pred1 = null;
  			
  			System.out.println("\tFor User: " + user);
  			
  			try {
  				pred1 = engine.getRecommendations(user, 10, 0);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendations()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\tItem " + item + "\tgetRecommendations(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		
  		System.out.println("  From offset 50"); 
  		
  		for (int k = 0; k < 5; k++)
  		{
  			int user = (int) (rdm.nextFloat() * (maxUser - 1) + 1);
  			ItemPrediction[] pred1 = null;
  			
  			System.out.println("\tFor User: " + user);
  			
  			try {
  				pred1 = engine.getRecommendations(user, 10, 50);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendations()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\tItem " + item + "\tgetRecommendations(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		System.out.println();
  		return pass;
  		
  	} // multiUserTest1()
  	
    /**
  	 * multiUserTest2()
  	 * Test 5 users for 20 recommendations:
  	 * 1. from offset 10
  	 * 2. from offset 100
  	 * @return int 
  	 */
  	public int multiUserTest2()
  	{
  		System.out.println("Check multi users for 20 recommendations ...");
  		int pass = 1;
  		Random rdm = new Random();
  		
  		System.out.println("  From offset 10"); 
  		
  		for (int k = 0; k < 5; k++)
  		{
  			int user = (int) (rdm.nextFloat() * (maxUser - 1) + 1);
  			ItemPrediction[] pred1 = null;
  			
  			System.out.println("\tFor User: " + user);
  			
  			try {
  				pred1 = engine.getRecommendations(user, 20, 10);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendations()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\tItem " + item + "\tgetRecommendations(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		
  		System.out.println("  From offset 100"); 
  		
  		for (int k = 0; k < 5; k++)
  		{
  			int user = (int) (rdm.nextFloat() * (maxUser - 1) + 1);
  			ItemPrediction[] pred1 = null;
  			
  			System.out.println("\tFor User: " + user);
  			
  			try {
  				pred1 = engine.getRecommendations(user, 20, 100);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendations()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\tItem " + item + "\tgetRecommendations(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		System.out.println();
  		return pass;
  		
  	} // multiUserTest2()
  	
  	/**
  	 * multiUserTest3()
  	 * Test 5 users for 50 recommendations:
  	 * 1. from offset 50
  	 * 2. from offset 290
  	 * @return int
  	 */
  	public int multiUserTest3()
  	{
  		System.out.println("Check multi users for 50 recommendations ...");
  		int pass = 1;
  		Random rdm = new Random();
  		
  		System.out.println("  From offset 50"); 
  		
  		for (int k = 0; k < 5; k++)
  		{
  			int user = (int) (rdm.nextFloat() * (maxUser - 1) + 1);
  			ItemPrediction[] pred1 = null;
  			System.out.println("\tFor User: " + user);
  			
  			try {
  				pred1 = engine.getRecommendations(user, 50, 50);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendations()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\tItem " + item + "\tgetRecommendations() " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		
  		System.out.println("  From offset 290"); 
  		
  		for (int k = 0; k < 5; k++)
  		{
  			int user = (int) (rdm.nextFloat() * (maxUser - 1) + 1);
  			ItemPrediction[] pred1 = null;
  			System.out.println("\tFor User: " + user);
  			
  			try {
  				pred1 = engine.getRecommendations(user, 50, 290);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendations()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					ItemPrediction r = null;
  					int item = pred1[i].getItemID();
  					try {
  						r = engine.getPredictedRating(user, item);
  					} catch (Exception e) {
  						System.out.println("Exception: " + e);
  						e.printStackTrace();
  					}
  					System.out.println("\tItem " + item + "\tgetRecommendations(): " + pred1[i].getPrediction()
  									+ "\tpredicted: " + r.getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  		System.out.println();
  		return pass;
  		
  	} // multiUserTest3()
    
    /**
  	 * illegalParamTest()
  	 * 1. item < 1 and item > maxItem
  	 * 2. user < 1 and user > maxUser
  	 * 3. offset < 0
  	 * @ return int
  	 */
  	public int illegalParamTest()
  	{
  		int pass = 1;
  		System.out.println("Checking the illegal elements...");
  		// user < 1
  		for (int user = 0, k = 0; k < 5; k++, user--)
  	 	{
  	  		ItemPrediction[] pred1 = null;
  	  		System.out.println("  10 recommendation from offset 0 for User: " + user );
  			try {
  				pred1 = engine.getRecommendations(user, 10, 0);
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendations()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					int item = pred1[i].getItemID();
  					System.out.println("\tItem " + item + "\tgetRecommendations(): " 
  									+ pred1[i].getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  		} // for
  	  	
  	  	
  	  	// user > maxUser 
  	  	for (int user = maxUser + 1, k = 0; k < 5; k++, user++)
  	    {
  	    	ItemPrediction[] pred1 = null;
  	    	System.out.println("  10 recommendation from offset 0 for User: " + user );
  			try {
  				pred1 = engine.getRecommendations(user, 10, 0);
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  				
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendations()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					int item = pred1[i].getItemID();
  					System.out.println("\tItem " + item + "\tgetRecommendations(): " 
  								+ pred1[i].getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  	  	}
  	  	
  	  	// offset < 0
  	  	Random r = new Random();
  	  	for (int offset = -1, k = 0; k < 5; k++, offset--)
  	    {
  	    	int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
  	    	ItemPrediction[] pred1 = null;
  	    	System.out.println("  10 recommendation from offset" + offset + " for User: " + user );
  			try {
  				pred1 = engine.getRecommendations(user, 10, offset);
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendations()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					int item = pred1[i].getItemID();
  					System.out.println("\tItem " + item + "\tgetRecommendations(): " 
  									+ pred1[i].getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  	  	}
  	  	
  	  	// number < 0
  	  	for (int number = 0, k = 0; k < 5; k++, number--)
  	    {
  	    	int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
  	    	ItemPrediction[] pred1 = null;
  	    	System.out.println("  " + number + " recommendation from offset 0 for User: " + user );
  			try {
  				pred1 = engine.getRecommendations(user, number, 0);
  			} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  				
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for getRecommendations()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			if (pred1 == null)
  			{
  				System.out.println("No recommendations available");
  			} 
  			else
  			{ 
  				for (int i = 0; i < pred1.length; i++)
  				{
  					int item = pred1[i].getItemID();
  					System.out.println("\tItem " + item + "\tgetRecommendations(): " 
  									+ pred1[i].getPrediction() + "\tin DB: " + getRating(user, item));
  				}
  			}
  	  	}
  		System.out.println();
  		return pass;
  	
  	} // illegalParamTest()
    
    /**
     * Get connection with database. 
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException
    {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");
        
        return DriverManager.getConnection(url, username, password);
        
        // We now use a connection pooling approach. The real SQL driver
        // is set in the static initializer of this class.
       // return DriverManager.getConnection("jdbc:jdc:jdcpool");
        
    } // getConnection()
    
    /**
  	 * getRating()
  	 * Get rating from database.
     * @return rating
     * 
     */
    private float getRating(int user, int item)
    {
    	float rating = -1;
    	String query = "SELECT " + ratingFieldName + " FROM " + ratingTableName
    				   + " WHERE " + userFieldName + " = " + user + " and "
    				   + itemFieldName + " = " + item; 
   	
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
        	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);                    
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();    
        } catch (Exception e) {           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           if (rs != null && rs.next()){ 
               rating = rs.getFloat(1);
            }
            rs.close();
            conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
    // System.out.println(""+rating);   
        return rating;
    
    } // getRating()
    
    /**
  	 * testSeq()
  	 */
    public void testSeq()
    {
    	System.out.println("###############################");
  		System.out.println("Test for getRecommendations() ...");
  		System.out.println("###############################");
  		System.out.println(""); 
  		int pass = 1;
  		Random r = new Random();
    	
    	System.out.println("Start 10 Items Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = tenItemTest(user);
			if (t <= 0)
				pass = -1;
		}
		
		System.out.println("Start 20 Items Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = twentyItemTest(user);
			if (t <= 0)
				pass = -1;
		}	
		
		System.out.println("Start 50 Items Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = fiftyItemTest(user);
			if (t <= 0)
				pass = -1;
		}	
		
		System.out.println("  Start multiUser Test 1  -- 5 users have 10 recommendations ... ");
    	System.out.println("");
		int t = multiUserTest1();
		if (t <= 0)
			pass = -1;
				
		System.out.println("  Start multiUser Test 1  -- 5 users have 20 recommendations ... ");
    	System.out.println("");
		t = multiUserTest2();
		if (t <= 0)
			pass = -1;
		
		System.out.println("  Start multiUser Test 1  -- 5 users have 50 recommendations ... ");
    	System.out.println("");
		t = multiUserTest3();
		if (t <= 0)
			pass = -1;
				
		System.out.println("  Start illegal param Test ... ");
    	System.out.println("");
    	t = illegalParamTest();
    	if (t <= 0)
			pass = -1;
		
		System.out.println();
		if (pass > 0) {
			System.out.println("------ Pass! getRecommendations() Test! ------");
            System.err.println("------ Pass! getRecommendations() Test! ------");
        } else {
			System.out.println("------ Fail! getRecommendations() Test! ------");
            System.err.println("------ Fail! getRecommendations() Test! ------");
        }
        System.out.println();
  	} // testSeq()
  	
} // class GetRecommendationsTest