/* 
 * GetUserArrayTest.java
 * 
 * Version 1.1
 * 
 * 04/02/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */
package org.recommender.test;
 
import java.sql.*;
import java.util.*;
import java.io.*;
import org.recommender.server.*;

/**
 * Test for getItemRatingList()
 * @author Yun Wang
 * 
 */
public class GetUserArrayTest{

	private CFEngine engine = null;
	
	private static String ratingTableName;
    private static String userFieldName;
    private static String itemFieldName;
    private static String ratingFieldName;
    
    private float minRating = 0;
    private float maxRating = 0;
    private static Statement stmt = null;
    
    private int maxUser = 0, maxItem = 0;
    
    public GetUserArrayTest(CFEngine s)
    {
    	engine = s;
    	
    	String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                + "CFServer.properties";
        
        SystemProperties.loadProperties(filename);
        
        minRating = SystemProperties.getFloat("CFServer.db.minRating", 0);
        maxRating = SystemProperties.getFloat("CFServer.db.maxRating", 0);
        
        // Load table and field names
        ratingTableName = SystemProperties.getString(
                "CFServer.db.ratingsTable", "");
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
        
        String query = "SELECT MAX(" + userFieldName + "), MAX(" + itemFieldName +
        			") FROM " + ratingTableName; 
	    
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
            conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
            if (rs != null && rs.next()){ 
               maxUser = rs.getInt(1);
               maxItem = rs.getInt(2);
               
           }
           rs.close();
           conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
        } catch (IOException e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
     
    } // GetUserArrayTest()
    
    
    /**
  	 * singleItemTest()
  	 * Get user array:
  	 * 1.get array 
     * 2.rate 5 users and get array 
     * 3.remove first 5 users and get array
  	 * @return int 
  	 */
  	public int singleItemTest(int item)
  	{
  		System.out.println("Check User Array for Item " + item + " ...");
  		int pass = 1;
  		
  		ItemRating[] userArr = null;
  		
  		try {
  			userArr = engine.getItemRatingList(item);
  		} catch (Exception e) {
            pass = -1;
  			System.out.println("Error Testing for getItemRatingList()");
            System.out.println("Exception:" + e);
            e.printStackTrace();
        } 
       
        System.out.println("Originally Item " + item + " have been rated by " 
        					+ userArr.length + " users.");
     /*   for ( int i = 0; i < userArr.length; i++)
        {
        	ItemRating itRating = null;
        	try {
        		itRating = engine.getRating(userArr[i].getUserID(), item);
        	} catch (Exception e) {
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        	System.out.println("\tUser " + userArr[i].getUserID() + "\tItem " + item
        					+ "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(userArr[i].getUserID(), item) );
        } */
        System.out.println();
        
        // add or change ratings for 5 users
        Random r1 = new Random();
  		Random r2 = new Random();
  		  		
        for (int i = 0; i < 5; i++)
        {
        	int user = (int) (r1.nextFloat() * (maxUser - 1) + 1);
        	ItemRating itRating = null;
        	try {
        		itRating  = engine.getRating(user, item);
        	} catch (Exception e) {
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        	float rate;
        	if (itRating.getRating() < minRating || itRating.getRating() > maxRating )
  				rate = r2.nextFloat() * (maxRating - 1) + 1;
  			else
  				rate = 1 + itRating.getRating() / 3;
        	
        	try {
        		engine.setRating(user, item, rate);
        	} catch (Exception e) {
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        }
        
        userArr = null;
        try {
  			userArr = engine.getItemRatingList(item);
  		} catch (Exception e) {
            pass = -1;
  			System.out.println("Error Testing for getItemRatingList()");
            System.out.println("Exception:" + e);
            e.printStackTrace();
        } 
        System.out.println("After Add 5 users, Item " + item + " have been rated by " 
        					+ userArr.length + " users.");
        
      /*  for ( int i = 0; i < userArr.length; i++)
        {
        	ItemRating itRating = null;
        	try {
        		itRating = engine.getRating(userArr[i].getUserID(), item);
        	} catch (Exception e) {
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        	System.out.println("\tUser " + userArr[i].getUserID() + "\tItem " + item
        					+ "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(userArr[i].getUserID(), item) );
        	
        } */
        System.out.println();
        
        // remove first 5 users
        for (int i = 0; i < 5; i++)
        {
        	try {
        		engine.removeRating(userArr[i].getUserID(), item);
        	} catch (Exception e) {
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        }
        
        userArr = null;
        try {
  			userArr = engine.getItemRatingList(item);
  		} catch (Exception e) {
            pass = -1;
  			System.out.println("Error Testing for getItemRatingList()");
            System.out.println("Exception:" + e);
            e.printStackTrace();
        } 
        System.out.println("After Remove first 5 users, Item " + item + " have been rated by " 
        					+ userArr.length + " users.");
        
      /*  for ( int i = 0; i < userArr.length; i++)
        {
        	ItemRating itRating = null;
        	try {
        		itRating = engine.getRating(userArr[i].getUserID(), item);
        	} catch (Exception e) {
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        	System.out.println("\tUser " + userArr[i].getUserID() + "\tItem " + item
        					+ "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(userArr[i].getUserID(), item) );
        	
        } */
        System.out.println();
        return pass;
        
  	} // singleItemTest()
    
    /**
  	 * multiItemTest()items 
  	 * Test for 5 items: 
  	 * 1. get arrays for all users (should only return list for user 0)
     * 2. rate 5 users for all users
     * 3. remove first 5 users for all users
     * @return int 
  	 */
  	public int multiItemTest()
  	{
  		System.out.println("Check multi items ...");
  		int pass = 1;
  		
  		ItemRating[][] userArr = new ItemRating[5][];
  		int[] item = new int[5];
  		Random r1 = new Random();
  		
  		for(int i = 0; i < item.length; i++)
  		{
  			item[i] = (int) (r1.nextFloat() * (maxItem - 1) + 1);
  			try {
  				userArr[i] = engine.getItemRatingList(item[i]);
  			} catch (Exception e) {
            	pass = -1;
  				System.out.println("Error Testing for getItemRatingList()");
            	System.out.println("Exception:" + e);
            	e.printStackTrace();
        	}
        	System.out.println("Originally Item " + item[i] + " have been rated by " 
        					+ userArr[i].length + " users.");
        	
        /*	for ( int k = 0; k < userArr[i].length; k++)
        	{
        		ItemRating itRating = null;
        		try {
        			itRating = engine.getRating(userArr[i][k].getUserID(), item[i]);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        		System.out.println("\tUser " + userArr[i][k].getUserID() + "\tItem " + item[i]
        					+ "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(userArr[i][k].getUserID(), item[i]) );
        	}*/
        } // for
        System.out.println();
        
        // add or change ratings for 5 users
        Random r2 =new Random();
        
        for(int i = 0; i < item.length; i++)
  		{
  			for ( int k = 0; k < 5; k++)
        	{
        		int user = (int) (r1.nextFloat() * (maxUser - 1) + 1);
        		ItemRating itRating = null;
        		try {
        			itRating = engine.getRating(user, item[i]);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        		float rate;
        		if (itRating.getRating() < minRating || itRating.getRating() > maxRating )
  					rate = r2.nextFloat() * (maxRating - 1) + 1;
  				else
  					rate = 1 + itRating.getRating() / 3;
        	
        		try {
        			engine.setRating(user, item[i], rate);
        		} catch (Exception e) {
           		 	System.out.println("Exception:" + e);
          		  	e.printStackTrace();
        		}
        	
        	}
        	userArr[i] = null;
        	try {
        		userArr[i] = engine.getItemRatingList(item[i]);
  			} catch (Exception e) {
            	pass = -1;
  				System.out.println("Error Testing for getItemRatingList()");
            	System.out.println("Exception:" + e);
            	e.printStackTrace();
        	}
        	System.out.println("After Add 5 users, Item " + item[i] + " have been rated by " 
        					+ userArr[i].length + " users.");
        	
        /*	for ( int k = 0; k < userArr[i].length; k++)
        	{
        		ItemRating itRating = null;
        		try {
        			itRating = engine.getRating(userArr[i][k].getUserID(), item[i]);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        		System.out.println("\tUser " + userArr[i][k].getUserID() + "\tItem " + item[i]
        					+ "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(userArr[i][k].getUserID(), item[i]) );
        	}   */
  		
  		}// for
  		System.out.println();
  		
  		// remove first 5 items
  		
        for(int i = 0; i < item.length; i++)
  		{
  			int n = 5;
  			
  			if (userArr[i].length < 5) {
  				n = userArr[i].length;
  				System.out.println("" + n);
  			}
  			for ( int k = 0; k < n; k++)
        	{	
           		try {
        			engine.removeRating(userArr[i][k].getUserID(), item[i]);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        	}
        	userArr[i] = null;	
        	try {
        		userArr[i] = engine.getItemRatingList(item[i]);
  			} catch (Exception e) {
            	pass = -1;
  				System.out.println("Error Testing for getItemRatingList()");
  				System.out.println("Exception:" + e);
            	e.printStackTrace();
        	}
        	System.out.println("After Remove first 5 users, Item " + item[i] + " have been rated by " 
        					+ userArr[i].length + " users.");
        	
        /*	for ( int k = 0; k < userArr[i].length; k++)
        	{
        		ItemRating itRating = null;
        		try {
        			itRating = engine.getRating(userArr[i][k].getUserID(), item[i]);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        		System.out.println("\tUser " + userArr[i][k].getUserID() + "\tItem " + item[i]
        					+ "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(userArr[i][k].getUserID(), item[i]) );
        		
        	}  	*/
  		
  		} // for
  		System.out.println();
  		return pass;
  		
  	} // multiItemTest()
    
    /**
  	 * illegalParamTest()
  	 * 1. item < 1  
  	 * 2. item > maxUser
  	 * @return int
  	 */
  	public int illegalParamTest()
  	{
  		
  		System.out.println("Checking the illegal elements...");
  		int pass = 1;
  		
  		ItemRating[] userArr = null;
  		
  		// item < 1
  		for (int item = 1, i = 0; i < 5; i++, item--)
  	 	{
  	 		try {
  	 			userArr = engine.getItemRatingList(item);
  	 		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  	 		} catch (Exception e) {
  	 			pass = -1;
  				System.out.println("Error Testing for getItemRatingList()");
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        	
        	if (userArr == null) {
        		System.out.println("Originally Item " + item + " rating list is null.");
                continue;
            }
        	System.out.println("Originally Item " + item + " have been rated by "
        					+ userArr.length + " users.");
        					
        	/*for ( int k = 0; k < userArr.length; k++)
        	{
        		ItemRating itRating = null;
        		try {
        			itRating = engine.getRating(userArr[k].getUserID(), item);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        		System.out.println("\tUser " + userArr[k].getUserID() + "\tItem " + item
        					+ "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(userArr[k].getUserID(), item) );
        		
        	} */
        	userArr = null;  		
        	
  	 	} // for
  	 	System.out.println();
  	 	
  	 	// item > maxItem
  	 	for (int item = maxItem, i = 0; i < 5; i++, item++)
  	    {
  	    	try {
  	 			userArr = engine.getItemRatingList(item);
  	 		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
            	pass = -1;
  				System.out.println("Error Testing for getItemRatingList()");
            	System.out.println("Exception:" + e);
          	  	e.printStackTrace();
        	}
        	
        	if (userArr == null) {
        		System.out.println("Originally Item " + item + " rating list is null.");
                continue;
            }
        	System.out.println("Originally Item " + item + " have been rated by " 
        					+ userArr.length + " users.");
        	/*for ( int k = 0; k < userArr.length; k++)
        	{
        		ItemRating itRating = null;
        		try {
        			itRating = engine.getRating(userArr[k].getUserID(), item);
        		} catch (Exception e) {
            		System.out.println("Exception:" + e);
          	  		e.printStackTrace();
        		}
        		System.out.println("\tUser " + userArr[k].getUserID() + "\tItem " + item
        					+ "\tin Cache " + itRating.getRating()
        					+ "\tin DB " + getRating(userArr[k].getUserID(), item) );
        	}  */
        	userArr = null;		
        } // for
        System.out.println();  	    
		return pass;
		
	} // illegalParamTest()
	
    
    /**
     * Get connection with database. 
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException
    {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");
    
        return DriverManager.getConnection(url, username, password);
        
        // We now use a connection pooling approach. The real SQL driver
        // is set in the static initializer of this class.
       // return DriverManager.getConnection("jdbc:jdc:jdcpool");
    } // getConnection()
    
  	/**
  	 * getRating()
  	 * Get rating from database.
     * @return rating
     * 
     */
  	private float getRating(int user, int item)
    {
    	float rating = -1;
    	String query = "SELECT " + ratingFieldName + " FROM " + ratingTableName
    				   + " WHERE " + userFieldName + " = " + user + " and "
    				   + itemFieldName + " = " + item; 
   
   		Connection conn = null;	
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
        	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);           
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();    
        } catch (Exception e) {           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           if (rs != null && rs.next()){ 
               rating = rs.getFloat(1);
            }
            rs.close();
            conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
    // System.out.println(""+rating);   
        return rating;
    
    } // getRating()
    
    /**
  	 * testSeq()
  	 */
    public void testSeq()
  	{
  		System.out.println("############################");
  		System.out.println("Test for getUserRatingList() ...");
  		System.out.println("############################");
  		System.out.println("");
  		int pass = 1;
  		
  		Random r = new Random();
  		
  		System.out.println("Start Single Item Test ... ");
    	System.out.println(""); 
    	for (int i = 0; i < 5; i++)
    	{
			int item = (int) (r.nextFloat() * (maxItem - 1) + 1);
			int t = singleItemTest(item);
			if (t < 0 )
				pass = -1;
		}
		
		System.out.println("Start Multi Item Test ... ");
    	System.out.println(""); 
    	int t = multiItemTest();
    	if (t < 0 )
			pass = -1;
    	
    	System.out.println("Start illegal param Test ... ");
    	System.out.println("");
    	t = illegalParamTest();
		if (t < 0 )
			pass = -1;
		
		System.out.println();
		if (pass >= 0) {
			System.out.println("------ Pass! getItemRatingList() Test! ------");
            System.err.println("------ Pass! getItemRatingList() Test! ------");
        } else {
			System.out.println("------ Fail! getItemRatingList() Test! ------");
            System.err.println("------ Fail! getItemRatingList() Test! ------");
        }
        System.out.println();
	}
    
} // class GetUserArrayTest