/* 
 * MultiThreadTest.java
 * 
 * Version 1.1
 * 
 * 04/14/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */
 
package org.recommender.test;
 

import org.recommender.server.CFEngine;
import org.recommender.server.CFEngineImpl;
import org.recommender.server.SystemProperties;
import org.recommender.server.SystemUtils;

import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.util.Random;

/**
 * Test for Multi threads
 *
 * @author	 <A HREF="mailto:wangyun@cs.orst.edu">Yun Wang</A>
 * @since   JDK 1.2 
 */
public class MultiThreadTest
{
	private static CFEngine engine = null;
	
	/**
	 * Constructor
	 * Either through RMI to locate the CFEngine Server, or directly create a CFEngine.
	 *
	 * @param direct
	 *
	 */
	public MultiThreadTest(boolean direct)
	{
		String installdir = System.getProperty("cf.path",".");
		String filename = installdir + "/lib/" + "CFServer.properties";
		if (direct) {
			try {
	 			engine = new CFEngineImpl(filename);
	 		} catch (Exception e) {
	 			System.out.println("Exception " + e);
	 		}
	 		 
	 	} else {     // for RMI
		
			System.setSecurityManager(new RMISecurityManager());
	 		
			// Load properties            
        	SystemProperties.loadProperties(filename);        
			String url = "rmi://" + SystemProperties.getString("CFServer.rmi.nameserver") + "/";
			String bindName = SystemProperties.getString("CFServer.rmi.objectname");
			
			// Lookup the cfengine server until it boots up
			while (true) {
				try {
					engine = (CFEngine)Naming.lookup(url + bindName);
				} catch (Exception e) {	
					//System.out.println("Exception: " + e);
				}
			
				if (engine != null) {
					try {
						SystemUtils.printDebug(engine.test());
					} catch (Exception e) {
		   				//System.out.println("Error: Cannot locate server on " 
						//	+SystemProperties.getString("CFServer.rmi.nameserver"));
						continue;
					}
					break;
				}
			}
			 	
	 	
	 	}
       
        
	}

	
    public static void main(String[] args)
	{
		/*CFEngine engine = null;
		String s = null;
		String installdir = System.getProperty("cf.path",".");
		String filename = installdir + "/lib/"
                        + "CFServer.properties";
                        
		if (args != null  && args.length > 0)
			s = args[0]; 
		
		if (s != null && !s.equals("rmi")) {
			try {
				engine = new CFEngineImpl(filename);
	 		} catch (Exception e) {
	 			System.out.println("Exception " + e);
	 		}
	 	} else if (s != null) {                     // RMI
			// First setup the RMI Security manager
			System.setSecurityManager(new RMISecurityManager());
			
			// Load properties        
        	SystemProperties.loadProperties(filename);
			String url = "rmi://" + SystemProperties.getString("CFServer.rmi.nameserver") + "/";
			String bindName = SystemProperties.getString("CFServer.rmi.objectname");
		
			// Lookup the cfengine server until it boots up
			while (true) {
				try {
					engine = (CFEngine)Naming.lookup(url + bindName);
				} catch (Exception e) {
					//System.out.println("Exception: " + e);
				}

				if (engine != null) {
					try {
						SystemUtils.printDebug(engine.test());
					} catch (Exception e) {
		   				System.out.println("Error: Cannot locate server on "
							+SystemProperties.getString("CFServer.rmi.nameserver"));
						continue;
					}
					break;
				}
			}
		}*/
		
		// get the invoking mode, either rmi or direct
		String s = null;
		if (args != null  && args.length > 0)
			s = args[0]; 
			
		MultiThreadTest test;
		if (s != null && s.equals("rmi")) 
			test = new MultiThreadTest(false);
		else
			test = new MultiThreadTest(true); 
		
		// get the # of thread and execution time	 
        int nThread = 10;
        int minutes = 60000;
        long time = 30 * minutes;
        
        if (args != null && args.length > 1) {
        	try {
            	nThread = Integer.parseInt(args[1]);
            	if (args.length > 2)
        	    	time = Integer.parseInt(args[2]) * minutes;
        	} catch (NumberFormatException nfe){
            	System.out.println("Exception" + nfe);
            	return;
        	}
        }
        
        // generate threads and their operations
		CFOperation[] d = new CFOperation[nThread];
        Thread[] t = new Thread[nThread];
        for (int i = 0; i < d.length; i ++)
        {
            d[i] = new CFOperation(engine, ""+i);
            t[i] = new Thread(d[i]);
            t[i].setDaemon(true);
            t[i].start();
        }
		
		Random rand = new Random();
		Random rand2 = new Random();

		int user;
		int op;
		long start = System.currentTimeMillis();
		
		Runtime rt = Runtime.getRuntime();
        System.out.println("Total memory = " + rt.totalMemory()
                + ", free memory = " + rt.freeMemory());
        System.out.println();        
		System.out.println("   Testing for " + nThread + " thread. And run " + (time/minutes) + " minutes.");
		System.out.println("--------------------------------------------------------");
		
		
		while(System.currentTimeMillis() <= (start + time) )
        {
        	
            for(int k = 0; k < d.length; k++)
            {
               user = 1 + rand.nextInt(1000);
               op = rand2.nextInt(17);
			   // op = rand2.nextInt(2) + 15;
                while (d[k].isBusy()) {
                   try {
					    Thread.sleep(25);
				    } catch (InterruptedException e) {
					    System.out.println(e);
				    }
                }
             //   System.out.println("Thread " + k + " user "+ user+ " op "+ op);
                d[k].doDBop(user, op);

            } // for
           	
           	
        } // while
        
        // wait until all threads are done     
        System.out.println("Finish threading");       
        for (int i = 0 ; i < d.length;)
        {
            if (!d[i].isBusy()) {
                    i++;
                    continue;
           }
           try {
			    Thread.sleep(25);
		   } catch (InterruptedException e) {
			        System.out.println(e);
		   }
       }

		Runtime rt2 = Runtime.getRuntime();
        System.out.println("Total memory = " + rt2.totalMemory()
                + ", free memory = " + rt2.freeMemory());
        
        // if direct test, flush the DB then exit        
        if (s == null || !s.equals("rmi")) {
        	try {
    			engine.shutdown();
    		} catch (Exception e) {
    			System.out.println("Exception " + e);
    		}
        }
		
	} // main()
	
} // class MultiThreadTest