package org.recommender.test;

import org.recommender.server.ItemPrediction;
import org.recommender.server.CFException;
import org.recommender.server.CFEngine;

import java.rmi.RemoteException;

/**
 * Created by 
 * User: herlock
 * Date: May 10, 2003
 * Time: 7:31:51 PM
 */
public class RandomRateThread extends TesterThread {
    private int minItemid;
    private int maxItemid;
    private float minRating;
    private float maxRating;

    RandomRateThread(CFEngine cfengine, String threadName, int minUserid, int maxUserid,
                     int minItemid, int maxItemid, int count) {
        super(cfengine, threadName, minUserid, maxUserid, count);
        this.minItemid = minItemid;
        this.maxItemid = maxItemid;
        try {
            minRating = cfengine.getMinRating();
        } catch (RemoteException e) {
            e.printStackTrace();
            System.exit(1);
        }
        try {
            maxRating = cfengine.getMaxRating();
        } catch (RemoteException e) {
            e.printStackTrace();
            System.exit(1);
        }
    }

    /**
     * Main method for this thread. This will grab count recommendations
     * for users between minUserid and maxUserid, picking a random userid
     * each time.
     */
    public void run() {
        long startTime = System.currentTimeMillis();
        for (int i = 0; i < count; i++) {
            // Pick a random userid between minUserid and maxUserid
            int userid = (int) (minUserid + ((maxUserid - minUserid) * Math.random()));

            // Pick a random movieid
            int itemid = (int) (minItemid + ((maxItemid - minItemid) * Math.random()));

            // Pick a random rating
            float rating = (int) (minRating + ((maxRating- minRating) * Math.random()));

            try {
                cfengine.setRating(userid, itemid, rating);
            } catch (RemoteException e) {
                e.printStackTrace();
                fatalError();
            } catch (CFException e) {
                e.printStackTrace();
                fatalError();
            }
	    //            System.out.print("R");

        }

        System.out.println(" total number of ratings = " + count);
        long stop = System.currentTimeMillis();
        long length = (stop - startTime);
        System.out.println("Total time = " + length + "ms\n\n");
        // preds/s
        float rate = count / ((float) length / 1000);
        System.out.println("Rate: " + rate + " ratings per second");
    }

}
