/* 
 * RemoveRatingTest.java
 * 
 * Version 1.1
 * 
 * 03/10/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */
 
package org.recommender.test;
 
import java.sql.*;
import java.util.*;
import java.io.*;
import org.recommender.server.*;

/**
 * Test for removeRating()
 * @author Yun Wang
 * 
 */
public class RemoveRatingTest{

	private CFEngine engine = null;
	
	private static String ratingTableName;
    private static String userFieldName;
    private static String itemFieldName;
    private static String ratingFieldName;
    
    private float minRating = 0;
    private float maxRating = 0;
    private static Statement stmt = null;
   
    private int maxUser = 0, maxItem = 0;
    
    public RemoveRatingTest(CFEngine s)
    {
  		engine = s;
  		
  		String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                + "CFServer.properties";
        
        SystemProperties.loadProperties(filename);
        
        // Load table and field names
        ratingTableName = SystemProperties.getString(
                "CFServer.db.ratingsTable", "");
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
        
        minRating = SystemProperties.getFloat("CFServer.db.minRating", 0);
        maxRating = SystemProperties.getFloat("CFServer.db.maxRating", 0);
        
        String query = "SELECT MAX(" + userFieldName + "), MAX(" + itemFieldName +
        			") FROM " + ratingTableName; 
	    
	    Connection conn = null;
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
             conn = getConnection();
             stmt = conn.createStatement();
             rs = stmt.executeQuery(query); 
        	if (rs != null && rs.next()){ 
               maxUser = rs.getInt(1);
               maxItem = rs.getInt(2);
            }
            rs.close();           
            conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
        } catch (IOException e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        
  	} // RemoveRatingTest()
  	
  	/**
  	 * singleItemTest1()
  	 * 1. remove an item, check value 
  	 * 1. remove the item again, check value 
  	 * 2. reset item, check value
  	 * @return int
  	 */
  	public int singleItemTest(int user)
  	{
  		System.out.println("Checking Single Item for User " + user +" ...");
  		int pass = 1;
  		  		
  		Vector v = getItems(user);
  		int length = v.size();
  	
  		if (length <= 0) 
  		{
  			System.out.println("\t no item available.");
  			return 0;
  		}  				
  			
  		Random r = new Random();
  		int item = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  		ItemRating itRating = null;
        try {
        	itRating = engine.getRating(user, item);
        } catch (Exception e) {
        	System.out.println("Exception:" + e);
        	e.printStackTrace();
        }
  		System.out.println("\tBefore Remove:\tItem " + item + "\tin Cache " + itRating.getRating());
  		
  		//remove available value
  		try {
  			engine.removeRating(user, item); 
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for removeRating()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		
  		itRating = null;
  		try {
        	itRating = engine.getRating(user, item);
        } catch (Exception e) {
        	System.out.println("Exception:" + e);
        	e.printStackTrace();
        }
  				
  		System.out.println("\tAfter Remove:\tItem " + item + "\t in Cache:" + itRating.getRating() 
  							+"\tin DB: " + getRating(user, item));
  		
  		// remove again
  		// YW: OK for Sql query. If FAIL is wanted, have to handle user exception 
  		try {
  			engine.removeRating(user, item); 
  		} catch (Exception e) {
  			pass = -1;
  			System.out.println("Error Testing for removeRating()");
  			System.out.println("Exception: " + e);
  			e.printStackTrace();
  		}
  		itRating = null;
  		try {
        	itRating = engine.getRating(user, item);
        } catch (Exception e) {
        	System.out.println("Exception:" + e);
        	e.printStackTrace();
        }		
  		System.out.println("\tRemove Again:\tItem " + item + "\t in Cache:" + itRating.getRating() 
  							+"\tin DB: " + getRating(user, item));
  		
  		// reset rating
  		if ( !(itRating.getRating() < minRating || itRating.getRating() > maxRating))
  		{
  			try {
  				engine.setRating(user, item, itRating.getRating());
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  			e.printStackTrace();
  			}
  			itRating = null;
  			try {
        		itRating = engine.getRating(user, item);
        	} catch (Exception e) {
        		System.out.println("Exception:" + e);
        		e.printStackTrace();
       		}		
  			System.out.println("\tReset Rating:\tItem " + item + "\t in Cache:" + itRating.getRating() 
  							+"\tin DB: " + getRating(user, item));
  		}
  		System.out.println("");
  		return pass; 		
  		 
  	} // singleItemTest()
  	
  	/**
  	 * floatItemTest()
  	 * Test for removing 2 items at a time, reset later
  	 * @return int
  	 */
  	public int floatItemTest(int user)
  	{
  		System.out.println("Checking Two Items for User " + user +" ...");
  		int pass = 1;
  		  		
  		int[] item = new int[2];
  		  		
  		Vector v = getItems(user);
  		int length = v.size();
  	
  		System.out.println("Before Remove:");
  		if (length <= 0) 
  		{
  			System.out.println("\t no item available.");
  			return 0;
  		}  				
  		
  		Random r = new Random();
  		for(int i = 0; i < 2; i++)
  		{
  			
  			if ( i < 2 && length > 2)
  			{	
  				item[i] = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  			} 
  			else if (i < 2 && length <= 2 && length > (i+1))
  			{
  				item[i] = ((Integer) v.get(i)).intValue();  					
  			} 
  			else 
  			{
  				item[i] = (int) (r.nextFloat() * (maxItem - 1) + 1);
  			}
  		   	
  		   	ItemRating itRating = null;
  			try {
        		itRating = engine.getRating(user, item[i]);
        	} catch (Exception e) {
        		System.out.println("Exception:" + e);
        		e.printStackTrace();
        	}	
  		    
  			System.out.println("\tItem " + item[i] + "\tin Cache " + itRating.getRating());
  			
  			try {
  				engine.removeRating(user, item[i]);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for removeRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  		} // for
  		
  		System.out.println("After Remove");
  		for(int i = 0; i < 2; i++)
  		{  		
  			ItemRating itRating = null;
  			try {
        		itRating = engine.getRating(user, item[i]);
        	} catch (Exception e) {
        		System.out.println("Exception:" + e);
        		e.printStackTrace();
        	}	
  			System.out.println("\tItem " + item[i] + "\tin Cache " + itRating.getRating()
  							+ "\tin DB " + getRating(user, item[i]));
  			/*try {
  				engine.setRating(user, item[i], rating[i]);
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  			} */  			
  		} 
  		
  		System.out.println(""); 			
  		return pass;
  		
  	} // floatItemTest()
  	
  	/**
  	 * tenItemTest()
  	 * For rated items, remove 10 items, then reset
  	 * @return int 
  	 */
  	public int tenItemTest(int user)
  	{
  		System.out.println("Checking 10 Items for User " + user + " ...");
  		int pass = 1;
  		  		
  		int[] item = new int[10];
  		ItemRating[] rating = new ItemRating[10];
  		
  		Random r = new Random();
  		
  		Vector v = getItems(user);
  		int length = v.size();
	
  		if (length <= 0) 
  		{
  			System.out.println("User " + user + "\t no item available.");
  			return 0;
  		}  				
  			
  		System.out.println("Before Remove");
  		for ( int k = 0; k < 10; k++)
  		{
  			
  			if ( k < 10 && length > 10)
  			{	
  				item[k] = ((Integer) v.get((int) (r.nextFloat() * length))).intValue();
  			} 
  			else if (k < 10 && length <= 10 && length > (k+1))
  			{
  				item[k] = ((Integer) v.get(k)).intValue();  					
  			} 
  			else 
  			{
  				item[k] = (int) (r.nextFloat() * (maxItem - 1) + 1);
  			}
  			try {
        		rating[k] = engine.getRating(user, item[k]);
        	} catch (Exception e) {
        		System.out.println("Exception:" + e);
        		e.printStackTrace();
        	}	
  			
  		    System.out.println("\tItem " + item[k] + "\tRating " + rating[k].getRating());
  		 } 
  		for ( int k = 0; k < 10; k++)
  		{  
  		    try {
  				engine.removeRating(user, item[k]); 
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for removeRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  		} // for
  		
  		System.out.println("After Remove");
  		for ( int k = 0; k < 10; k++)
  		{
  			ItemRating itRating = null;
  			try {
        		itRating = engine.getRating(user, item[k]);
        	} catch (Exception e) {
        		System.out.println("Exception:" + e);
        		e.printStackTrace();
        	}	
  			System.out.println("\tItem " + item[k] + "in Cache: " + itRating.getRating()
  								+ "\tin DB: " + getRating(user, item[k]));
  			if (rating[k].getRating() < minRating || rating[k].getRating() > maxRating)
  				continue;
  			try {
  				engine.setRating(user, item[k], rating[k].getRating());
  			} catch (Exception e) {
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  		}
  		
  		System.out.println("Reset Rating");
  		for ( int k = 0; k < 10; k++)
  		{
  			ItemRating itRating = null;
  			try {
        		itRating = engine.getRating(user, item[k]);
        	} catch (Exception e) {
        		System.out.println("Exception:" + e);
        		e.printStackTrace();
        	}	
  			System.out.println("\tItem " + item[k] + "in Cache: " + itRating.getRating()
  								+ "\tin DB: " + getRating(user, item[k]));
  		}
  	
  		System.out.println("");
  		return pass;
  	}
  	
  	/**
  	 * multiUserTest1()
  	 * Test for 2 pairs of different users for the same 5 items (both rated)
  	 * remove items, then reset
  	 * @return int
  	 */
  	public int multiUserTest1()
  	{
  		System.out.println("Check Same Five Items for Different users ...");
  		int pass = 1;
  		
  		Random r1 = new Random();
  		Random r2 = new Random();
  		  		
  		int[] user = new int[4];
  		int[][] item = new int[2][5];
  		ItemRating[][] rating = new ItemRating[4][5];
  		
  		for (int i = 0 ; i < 4; i++)
  		{
  			user[i] = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  		}  
  		
  		for (int i = 0; i < 2; i++)
  		{
  			Vector v =get2UserItems(user[i], user[i+1]);
  			int length = v.size();
  			System.out.println("  Have " + length +" available Rated Items between user " + user[i] +" and user " + user[i+1] );
  				
  			for (int k = 0; k < 5; k++)
  			{
  				if ( k < 5 && length > 5)
  				{	
  					item[i][k] = ((Integer) v.get((int) (r2.nextFloat() * length))).intValue();
  				} 
  				else if (k < 5 && length <= 5 && length > (k+1))
  				{
  					item[i][k] = ((Integer) v.get(k)).intValue();					
  				} 
  				else 
  				{
  					item[i][k] = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  				} 
  			}
  		}		
  		
  		
  		// remove all the ratings for (4 user X 5 item)
  		for (int i = 0 ; i < 4; i++)
  		{
  			System.out.println(i + ". Before Remove: For user " + user[i]);
  			
  			
  			for (int k = 0; k < 5; k++)
  			{
  				try {
  					rating[i][k] = engine.getRating(user[i], item[i/2][k]);
  				} catch (Exception e) {
        			System.out.println("Exception:" + e);
        			e.printStackTrace();
        		}	
  				System.out.println("\t\tItem " + item[i/2][k] + "\tRating " + rating[i][k].getRating());
  			}
  			for (int k = 0; k < 5; k++)
  			{	
  				try {
  					engine.removeRating(user[i], item[i/2][k]); 
  				} catch (Exception e) {
  					pass = -1;
  					System.out.println("Error Testing for removeRating()");
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  			}
  		}
  		System.out.println(""); 
  		
  		// check the values and reset rating
  		for (int i = 0 ; i < 4; i++)
  		{
  			System.out.println(i + ". After Remove: For user " + user[i] );
  			for (int k = 0; k < 5; k++)
  			{ 
  				ItemRating itRating = null;
  				try {
        			itRating = engine.getRating(user[i], item[i/2][k]);
        		} catch (Exception e) {
        			System.out.println("Exception:" + e);
        			e.printStackTrace();
        		}	
  				System.out.println("\t\tItem " + item[i/2][k] + "\tin Cache: " + itRating.getRating() 
  								+ "\tin DB: " + getRating(user[i], item[i/2][k]));
  				if (rating[i][k].getRating() < minRating || rating[i][k].getRating() > maxRating)
  					continue;		
  					
  				try {
  					engine.setRating(user[i], item[i/2][k], rating[i][k].getRating());
  				} catch (Exception e) {
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  			}
  		}
  		System.out.println("");
  		
  		// check the values after reset
  		for (int i = 0 ; i < 4; i++)
  		{ 
  			System.out.println(i + ". Reset Rating: For user " + user[i] );
  			for (int k = 0; k < 5; k++)
  			{ 
  				ItemRating itRating = null;
  				try {
        			itRating = engine.getRating(user[i], item[i/2][k]);
        		} catch (Exception e) {
        			System.out.println("Exception:" + e);
        			e.printStackTrace();
        		}	
  				System.out.println("\t\tItem " + item[i/2][k] + "\tin Cache: " + itRating.getRating() 
  								+ "\tin DB " + getRating(user[i], item[i/2][k]));
  			}	  			   					
  		
  		}	
  		System.out.println(""); 
  		return pass;
  		
  	} // multiUserTest1()
  	
  	/**
  	 * multiUserTest2()
  	 * Test for different 5 items for multiple users
  	 * remove the items, then remove again
  	 * @ return int
  	 */
  	public int multiUserTest2()
  	{
  		System.out.println("Check Different Items for Multiple Users ...");
  		int pass = 1;
  		  		
  		int[] user = new int[5];
  		int[][] item = new int[5][5];
  		ItemRating[][] rating = new ItemRating[5][5];  		
  		Random r1 = new Random();
  		Random r2 = new Random();
  		
  		// remove rating
  		for (int i = 0; i < 5; i++)
  		{
  			user[i] = (int) (r1.nextFloat() * (maxUser - 1) + 1);
  			System.out.println(i + ". Before Remove : For User " + user[i]);
  			
  			Vector v = getItems(user[i]);
  			int length = v.size();
 
  			if (length <= 0) 
  			{
  				System.out.println("\t no item available.");
  				continue;
  			} 
  			 					
  			
  			for (int k = 0; k < 5; k++)
  			{
  			
  				if ( k < 5 && length > 5)
  				{	
  					item[i][k] = ((Integer) v.get((int) (r2.nextFloat() * length))).intValue();
  				} 
  				else if (k < 5 && length <= 5 && length > (k+1))
  				{
  					item[i][k] = ((Integer) v.get(k)).intValue();					
  				} 
  				else 
  				{
  					item[i][k] = (int) (r2.nextFloat() * (maxItem - 1) + 1);
  				}
  				
  				try {
        			rating[i][k] = engine.getRating(user[i], item[i][k]);
        		} catch (Exception e) {
        			System.out.println("Exception:" + e);
        			e.printStackTrace();
        		}	
  				System.out.println("\t\tItem " + item[i][k] + "\tRating " + rating[i][k].getRating());
  			}	
  			for (int k = 0; k < 5; k++)
  			{
  				try {
  					engine.removeRating(user[i], item[i][k]);
  				} catch (Exception e) {
  					pass = -1;
  					System.out.println("Error Testing for removeRating()");
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  			}
  		}
  		
  		// check removing results and  remove again
  		for (int i = 0; i < 5; i++)
  		{
  			System.out.println(i + ". After Remove: For User " + user[i]);
  			for (int k = 0; k < 5; k++)
  			{ 
  				ItemRating itRating = null;
  				try {
        			itRating = engine.getRating(user[i], item[i][k]);
        		} catch (Exception e) {
        			System.out.println("Exception:" + e);
        			e.printStackTrace();
        		}	  				
  				
  				System.out.println("\t\tItem " + item[i][k] + "\tin Cache: " + itRating.getRating() 
  									+ "\tin DB " + getRating(user[i], item[i][k]));
  			}
  			for (int k = 0; k < 5; k++)
  			{ 
  				try {
  					engine.removeRating(user[i], item[i][k]);
  				} catch (Exception e) {
  					pass = -1;
  					System.out.println("Error Testing for removeRating()");
  					System.out.println("Exception: " + e);
  					e.printStackTrace();
  				}
  			}			
  		}
  		
  		for (int i = 0; i < 5; i++)
  		{
  			System.out.println(i + ". Remove Again: For User " + user[i]);
  			for (int k = 0; k < 5; k++)
  			{ 
  				ItemRating itRating = null;
  				try {
        			itRating = engine.getRating(user[i], item[i][k]);
        		} catch (Exception e) {
        			System.out.println("Exception:" + e);
        			e.printStackTrace();
        		}	  				
  				
  				System.out.println("\t\tItem " + item[i][k] + "\tin Cache: " + itRating.getRating() 
  									+ "\tin DB " + getRating(user[i], item[i][k]));
  			}			
  		}
  		System.out.println("");
  		return pass;
  	
  	} // multiUserTes2()
  	
  	/**
  	 * illegalParamTest()
  	 * Test for illegal parameters
  	 * 1. item < 1 and item > maxItem
  	 * 2. user < 1 and user > maxUser
  	 * @return int 
  	 */
  	public int illegalParamTest()
  	{
  		Random rdm = new Random();
  	    	  
  	  	System.out.println("Checking the illegal elements...");
  	  	int pass = 1;
  	  	
  	 	// test for itemID < 1
  	  	for (int item = 0, k = 0; k < 5; k++, item--)
  	  	{
  	  		int u = (int) (rdm.nextFloat() * (maxUser - 1) + 1);
  	  		
  	  		try {
  	  			engine.removeRating(u, item);
  	  		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  	  		} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for removeRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("User " + u + "\tItem " + item + "\tRating " + getRating(u, item));
  	 	}
  	  
  	  // itemID > maxItem
  	  for (int item = maxItem + 1, k = 0; k < 5; k++, item++)
  	  {
  	  	   int u = (int) (rdm.nextFloat() * (maxUser - 1) + 1);
  	  		
  	  		try {
  	  			engine.removeRating(u, item);
  	  		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for removeRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("User " + u + "\tItem " + item + "\tRating " + getRating(u, item));
  	  } 	
  	
  	  Random rdm2 = new Random();
  	  // user < 1
  	  for (int user = 0, k = 0; k < 5; k++, user--)
  	  {
  	  		int i = (int) (rdm2.nextFloat() * (maxItem - 1) + 1);
  	  		
  	  		try {
  	  			engine.removeRating(user, i);
  	  		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  			} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for removeRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("User " + user + "\tItem " + i + "\tRating " + getRating(user, i));
  	  }
  	 
  	 // user > maxUser 
  	  for (int user = maxUser + 1, k = 0; k < 5; k++, user++)
  	  {
  	  		int i = (int) (rdm2.nextFloat() * (maxItem - 1) + 1);
  	  		
  	  		try {
  	  			engine.removeRating(user, i);
  	  		} catch (CFIllegalParameterException e) {
  				System.out.println("Exception: " + e);
  	  		} catch (Exception e) {
  				pass = -1;
  				System.out.println("Error Testing for removeRating()");
  				System.out.println("Exception: " + e);
  				e.printStackTrace();
  			}
  			System.out.println("User " + user + "\tItem " + i + "\tRating " + getRating(user, i));
  	  }
  	  
  	  System.out.println("");
  	  return pass;
  	  
  	} // illegleParamTest()
  	
  	/**
     * Get connection with database. 
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException
    {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");
     
        return DriverManager.getConnection(url, username, password);
        
        // We now use a connection pooling approach. The real SQL driver
        // is set in the static initializer of this class.
       // return DriverManager.getConnection("jdbc:jdc:jdcpool");
    } // getConnection()
    
    /**
  	 * getRating()
  	 * Get rating from database.
     * @return rating
     * 
     */
    private float getRating(int user, int item)
    {
    	float rating = -1;
    	String query = "SELECT " + ratingFieldName + " FROM " + ratingTableName
    				   + " WHERE " + userFieldName + " = " + user + " and "
    				   + itemFieldName + " = " + item; 
   
   		Connection conn = null;	
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
          	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();   
        } catch (Exception e) {
           
            System.out.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try {
           if (rs != null && rs.next()){ 
               rating = rs.getFloat(1);
            }
            rs.close();
            conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
    // System.out.println(""+rating);   
        return rating;
    
    } // getRating()
    
    /**
  	 * getItems()
  	 * Get rated items for a user
     * @return Vector
     * 
     */
    private Vector getItems(int user)
    {
    	Vector items = new Vector();
    	String query = "SELECT " + itemFieldName + " FROM " + ratingTableName  + " WHERE " 
    	 				  + userFieldName + " = " + user + " order by " + itemFieldName;
    	
    	Connection conn = null;		    
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
          	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
                    
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();    
        } catch (Exception e) {
            System.out.println("Exception:" + e);
            e.printStackTrace();
        }
        
        try {
           while (rs != null && rs.next()) 
           		items.add(new Integer(rs.getInt(1))); 
 			rs.close();
 			conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
        
        return items;
    } // getItems()
    
    /**
  	 * getItems()
  	 * Get rated items for 2 users
     * @return Vector
     * 
     */
    private Vector get2UserItems(int u1, int u2)
    {
    	Vector items = new Vector();
    	String query = "SELECT " + ratingTableName + "1." + itemFieldName + " FROM " 
    					+ ratingTableName + " " + ratingTableName + "1, " 
    					+ ratingTableName + " " + ratingTableName + "2 WHERE "    
    					+ ratingTableName + "1." + userFieldName + " = " + u1 + " and "
    					+ ratingTableName + "2." + userFieldName + " = " + u2 + " and "
    					+ ratingTableName + "1." + itemFieldName + " = " 
    					+ ratingTableName + "2." + itemFieldName
    					+ " order by " + ratingTableName + "1." + itemFieldName;
    	
    	Connection conn = null;			    
	    ResultSet rs= null; 	                          
	      // Establish connection with databases
        try {
         	conn = getConnection();
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query); 
        } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            ex.printStackTrace();   
        } catch (Exception e) {
            System.out.println("Exception:" + e);
            e.printStackTrace();
        }
        
        try {
           while (rs != null && rs.next()) 
           		items.add(new Integer(rs.getInt(1))); 
 			rs.close();
 			conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:");
            e.printStackTrace();
        } 
        
        return items;
    
    }
    
    /**
  	 * testSeq()
  	 */
    public void testSeq()
    {
    	System.out.println("############################");
  		System.out.println("Test for removeRating() ...");
  		System.out.println("############################");
  		System.out.println("");    	
    	int pass = 1;
    	
    	Random r = new Random();
    	
    	System.out.println("  Start Single Item Test ... ");
    	System.out.println("");
    	for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = singleItemTest(user);
			if (t < 0 )
				pass = -1;
		}
		
		System.out.println("  Start 2 Items Test ... ");
    	System.out.println("");
		for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = floatItemTest(user);
			if (t < 0 )
				pass = -1;
		}
		
		System.out.println("  Start 10 Items Test ... ");
    	System.out.println("");
		for (int i = 0; i < 5; i++)
    	{
			int user = (int) (r.nextFloat() * (maxUser - 1) + 1);
			int t = tenItemTest(user);
			if (t < 0 )
				pass = -1;
		}
		
		System.out.println("  Start multiUser Test 1  -- 2 user2 have 5 same items ... ");
    	System.out.println("");
		int t = multiUserTest1();
		if (t < 0 )
			pass = -1;
			
		System.out.println("  Start multiUser Test 2 -- multi users have different items ... ");
    	System.out.println("");
		t = multiUserTest2();
		if (t < 0 )
			pass = -1;
			
		System.out.println("  Start Illegal Param Test ... ");
    	System.out.println("");    	
    	t = illegalParamTest();
		if (t < 0 )
			pass = -1;
			
		System.out.println();
		if (pass >= 0) {
			System.out.println("------ Pass! removeRating() Test! ------");
            System.err.println("------ Pass! removeRating() Test! ------");
        } else {
			System.out.println("------ Fail! removeRating() Test! ------");
            System.err.println("------ Fail! removeRating() Test! ------");
        }
        System.out.println();
    } // testSeq()
        
}