/*
 * MultiThreadTest.java
 *
 * Version 1.1
 *
 * 05/15/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */
package org.recommender.test;

import org.recommender.server.*;

import java.net.*;
import java.rmi.*;
import java.lang.Thread;
import java.util.LinkedList;
import java.util.Iterator;

/**
 * Class to test the resampling thread of the recommender engine
 * * @author Yun Wang
 */
public class ResampleTest {
	private CFEngine cfengine;

	public ResampleTest() {
	System.out.println("start");
		String serviceName = "cfengine";
        String rmiNameserver = "localhost";
        String url = "rmi://" + rmiNameserver + "/" + serviceName;
        try {
            cfengine = (CFEngine) Naming.lookup(url);
        } catch (NotBoundException e) {
            System.err.println("No such service " + serviceName + " registered with RMI on " + rmiNameserver);
            e.printStackTrace();
            fatalError();
        } catch (MalformedURLException e) {
            System.err.println("URL is malformed: " + url);
            e.printStackTrace();
            fatalError();
        } catch (RemoteException e) {
            e.printStackTrace();
            fatalError();
        }
        long startTime = System.currentTimeMillis();
        int numRatingsPerThread = 100;
        int numRatingsThreads = 2;

   		int numRemoveRatingsPerThread = 100;
        int numRemoveRatingsThreads = 2;

   		int numTopNsPerThread = 100;
        int numTopNThreads = 2;

   		int numTopNsByTypePerThread = 100;
        int numTopNByTypeThreads = 4;

        int numPredictPerThread = 100;
        int numPredictThread = 4;

        LinkedList threadList = new LinkedList();

     while ( System.currentTimeMillis() < (startTime + 700000)) {

   		 for (int i = 0; i < numTopNThreads; i++) {
            Thread t = new RandomTopNThread(cfengine, "TopN Thread" + i, 501, 940, numTopNsPerThread);
            threadList.add(t);
        }

    /*    for (int i = 0; i < numTopNByTypeThreads; i++) {
            Thread t = new RandomTopNByTypeThread(cfengine, "TopN Thread" + i, 501, 940, 1, 18, numTopNsByTypePerThread);
            threadList.add(t);
        }

        for (int i = 0; i < numPredictThread; i++) {
            Thread t = new RandomPredictThread(cfengine, "Predict Thread " + i, 1, 940, 1, 1682, numPredictPerThread);
            threadList.add(t);
        }*/

        for (int i = 0; i < numRatingsThreads; i++) {
            Thread t = new RandomRateThread(cfengine, "Rating Thread " + i, 1, 940, 1, 1682, numRatingsPerThread);
            threadList.add(t);
        }

       for (int i = 0; i < numRemoveRatingsThreads; i++) {
           // Thread t = new RandomRemoveRateThread(cfengine, "Rating Thread " + i, 1, 940, 1, 1682, numRemoveRatingsPerThread);
            //threadList.add(t);
        }
		Iterator iter = threadList.iterator();

        // Starts the new thread running
        	while (iter.hasNext()) {
            	Thread thread = (Thread) iter.next();
            	thread.start();
        	}


        // Wait for threads to end, then exit.
        // Probably isn't really necessary now, but later if we want to
        // do some post-processing.
        boolean complete = false;

        do {
            try {
                iter = threadList.iterator();
                while (iter.hasNext()) {
                    Thread thread = (Thread) iter.next();
                    thread.join();
                    System.out.println("Thread " + ((TesterThread) thread).threadName + " joined");
                }
                complete = true;
            } catch (InterruptedException e) {
                e.printStackTrace();
                fatalError();
            }
        } while (! complete);
     }
        long finishTime = System.currentTimeMillis();

        System.out.println("Rating threads: "
                + (1000 * (numRatingsPerThread * numRatingsThreads)) / (finishTime - startTime) + " per second");

        System.out.println("CFEngine Statistics");

        try {
            System.out.println(cfengine.getStatistics());
        } catch (RemoteException e) {
            e.printStackTrace();
            System.exit(1);
        }
    }

    /**
     * Called whenever a fatal error occurs
     */
    private void fatalError() {
        System.exit(1);
    }

    /**
     * The main function of the PerformanceTest class. Starts the testing threads running.
     *
     * @param args Current ignored.
     */
    public static void main(String[] args) {
       ResampleTest pt = new ResampleTest();
    }

}