/* 
 * ComputeEntropy.java
 * 
 * Version 1.1
 * 
 * 05/03/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */ 
package org.recommender.util;

import java.io.*;
import java.sql.*;
import java.util.*;
import java.net.*;
import java.lang.Math.*;
import org.recommender.server.*;
  
/**
 * 
 * This set of scripts will be use to construct a framework
 * for experiments on Recommender Systems. The purpose of these
 * experiments is to determine how sampling user ratings that are
 * cached by the RS system affects system accuracy and item coverage.
 *
 * This class compute the entropy for each item and assigns
 * each user an average entropy value that it loads into our database.
 * @author  (<A HREF="mailto:wangyun@cs.orst.edu">Yun Wang</A>)
 * @since   JDK 1.2
 * @version Version 1.0.0, 05-03-2003
 *
 */
 

public class ComputeEntropy{


	private Statement stmt = null;
	private Connection conn = null;
	
	private static String userTableName;
    private static String ratingsTableName;
	private static String ratingFieldName;
	private static String userFieldName;
	private static String itemFieldName;
	private static String userInfoTable;
	private static String userInfoID;
	private static String userInfoSamplingField;
	private static String userInfoNumRatings;
	private static String installdir;
	private static int maxUsers;
	private static int numUsers;
	private static int maxItems;
	private static int numRating;
	
	private int minRating = 0;
    private int maxRating = 0;
    
    private float[] itemEnt = null;
    
	private PreparedStatement ps1, ps2;
       
	public ComputeEntropy(){
		SystemUtils.printDebug("Starting Compute Entropy ...");
		
		Properties props = new Properties();
        installdir = System.getProperty("cf.path",".");
       
		
		// DB information, contained in the CFServer.properties file.
		// Load properties
        SystemProperties.loadProperties(installdir + "/lib/CFServer.properties");
        
		ratingsTableName = SystemProperties.getString("CFServer.db.ratingsTable", "");     
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
                

        userInfoTable = SystemProperties.getString("CFServer.db.userInfo.table", "");               
        userInfoID = SystemProperties.getString("CFServer.db.userInfo.userField", "");          
        userInfoSamplingField = SystemProperties.getString("CFServer.db.userInfo.samplingField", "");
		userInfoNumRatings = SystemProperties.getString("CFServer.db.userInfo.numRatingsField", "");;
      	
      	minRating = (int) SystemProperties.getFloat("CFServer.db.minRating", 0);
        maxRating = (int) SystemProperties.getFloat("CFServer.db.maxRating", 0);
        
        numRating = maxRating - minRating + 1;
        
        String query = "DELETE FROM " + userInfoTable;
        try {
            conn = getConnection();
            stmt = conn.createStatement();
            stmt.executeUpdate(query);
            precompileSQLStatements(conn);
        
		} catch (SQLException e) {
            System.out.println("SQLException:" + e);
            System.exit(1);
        } catch (Exception e) {
            System.out.println("Exception:" + e);
            System.exit(1);
        }
        
        // cache itemEntopy and userCount
        try {
        	cacheCount();
        } catch (SQLException e) {
            System.out.println("SQLException:" + e);
            System.exit(1);
        }
      	
	}
	
	/**
     * compute the user entropy
     * @throws SQLException
     */
	public void doComp() throws SQLException
	{
		SystemUtils.printDebug("Start Computing IUF ...");
		
		int currentUser= -1;
		int user =0;
		int ratingsCount= 0;
		int item;
		float sum = 0;
		boolean first= true;
		float entropy;
		ResultSet rs = null;
		
		String query = "SELECT " + userFieldName + ", "  + itemFieldName + " FROM  " 
				+ ratingsTableName + " where 1 ORDER BY 1, 2";
		
		try{
			stmt = conn.createStatement();
        	rs= stmt.executeQuery(query);
        } catch(SQLException ex){
        	System.out.println("WARNING: Select failed...");
        	rs= null;
        }
        while (rs.next()){
			user = rs.getInt(userFieldName);
			
			if (currentUser != user){	
				if(!first){
					if(ratingsCount != 0){  //weight by ratingsCount/60 to discard high IUF's with few ratings
						if (ratingsCount > 60)
							entropy = sum/ratingsCount;
						else
							entropy = sum/60; 
					}
				 	else
				 		entropy = 0;
				 	
				 	ps1.setInt(1, currentUser);
					ps1.setInt(2, ratingsCount);
					ps1.setFloat(3, entropy);
					ps1.executeUpdate();
					
				} else {
					first = false;
					
				}
				//initialize for new user
				currentUser = user;
				sum = 0;
				ratingsCount = 0;
			
        	}
        	//increment sum
			item= rs.getInt(itemFieldName);
			sum += itemEnt[item];
			ratingsCount++;
        	
		} // while
		
		// for last record
		if(ratingsCount != 0){  //weight by ratingsCount/60 to discard high IUF's with few ratings
			if (ratingsCount > 60)
				entropy= sum/ratingsCount;
			else
				entropy= sum/60; 
		} else
			entropy= 0;
		ps1.setInt(1, currentUser);
		ps1.setInt(2, ratingsCount);
		ps1.setFloat(3, entropy);
		ps1.executeUpdate();
		
		conn.close();		
		
	} // doComp()
	
	/**
     Precompile SQLStatements.
     @param conn
     @throws SQLException
     */
    private void precompileSQLStatements(Connection conn) throws SQLException 
    {
    	SystemUtils.printDebug("Precompile SQL Statements ... ");
    	
    	ps1 = conn.prepareStatement("INSERT INTO  " + userInfoTable + " (" + userInfoID 
    								+ " , "  + userInfoNumRatings + " , "  
    								+ userInfoSamplingField + ") VALUES (?, ?, ?)" );
    	
    	ps2 = conn.prepareStatement("SELECT " + ratingFieldName + " FROM " + ratingsTableName + " WHERE "
    								+ itemFieldName + "=?");
    								
    }
    
    /**
     * cache some properties
     * @throws SQLException
     */
    private void cacheCount() throws SQLException
    {
    	SystemUtils.printDebug("Cache Count");
    	
    	String query = "SELECT MAX(" + itemFieldName + ") FROM " + ratingsTableName;
    
    	ResultSet rs = null;
    	
    	try {
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);
        } catch (Exception e) {
        
            System.out.println("Fatal Database Error: " + e);
            System.exit(1);
        }
    	if (rs != null && rs.next()) {
            maxItems = rs.getInt(1);
        }
        stmt.close();
        rs.close();
        
        query = "SELECT MAX(" + userFieldName + ") FROM " + ratingsTableName;
        try {         
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);
        } catch (Exception e) {
            System.out.println("Fatal Database Error: " + e);
            System.exit(1);
        }
    	if (rs != null && rs.next()) {
            maxUsers = rs.getInt(1);
        }
        stmt.close();
        rs.close();
        
        query = "SELECT COUNT(DISTINCT " + userFieldName + ") FROM " + ratingsTableName;
        try {         
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);
        } catch (Exception e) {
            System.out.println("Fatal Database Error: " + e);
            System.exit(1);
        }
    	if (rs != null && rs.next()) {
            numUsers = rs.getInt(1);
        }
        stmt.close();
        rs.close(); 
        
        itemEnt = new float[maxItems + 1];
        
    	cacheItemEntropy();
  
    }

    /**
     * cache the item entyopy
     * @throws SQLException
     */
    private void cacheItemEntropy() throws SQLException
    {
    	SystemUtils.printDebug("Cache Item Entropy. Total items " + maxItems);
    	long start = System.currentTimeMillis();
   
    	ResultSet rs = null;    	
    	double[] p = new double[numRating];
    	int nRating = 0;
    	
    	itemEnt[0] = 0;
    	for (int i = 1; i < itemEnt.length; i++)
    	{
    		for (int k = 0; k < p.length; k++)
    		{
    			p[k] = 0.0000001;
    		}	 
    		try {
    			ps2.setInt(1, i);
    			rs = ps2.executeQuery();
    		} catch (Exception e) {
            	System.out.println("Fatal Database Error: " + e);
            	System.exit(1);
        	}
        	int ratingValue;        	
        	while (rs.next()) {
        		
        		ratingValue = (int) rs.getFloat(1);
        		nRating++;
        			
        		int index = ratingValue - minRating;
        		p[index] += 1;       			
        		
        	} // while    		
    		rs.close();
    		
    		itemEnt[i] = 0;
    		if (nRating != 0) {
    			double count = (double) nRating;
    			double e = 0.0;
    			for (int k = 0; k < p.length; k++)
    			{
    				p[k] /= count;
    				e += p[k] * Math.log(1 / p[k]);
    			}
    			itemEnt[i] = (float) e; 
    		}    			
    		nRating = 0;
    	} // for
    	
    	long stop = System.currentTimeMillis();
    	SystemUtils.printDebug("Spend " + (stop - start) + "ms Caching Item Entropy");	
    } // cacheItemEntropy()

   
    
    /**
     * Get connection with database.
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
        	System.setProperty("jdbc.drivers", drivers);
        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");
        return DriverManager.getConnection(url, username, password);        
        
    }
    
}