/* 
 * ComputeIUF.java
 * 
 * Version 1.1
 * 
 * 05/01/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */ 

package org.recommender.util; 

import java.io.*;
import java.sql.*;
import java.util.*;
import java.net.*;
import java.lang.Math.*;
import org.recommender.server.*;

/**
 * 
 * This set of scripts will be use to construct a framework
 * for experiments on Recommender Systems. The purpose of these
 * experiments is to determine how sampling user ratings affects
 * the accuracy and item coverage of a recommender system.
 *
 * This class compute the Inverse User Freqency for each item and assigns
 * each user an average IUF value, which is then loaded into a database.
 *
 * @author 	(<A HREF="mailto:wangyun@cs.orst.edu">Yun Wang</A>)
 * @since   JDK 1.2
 * @version Version 1.0.0, 05-01-2003
 *
 */
 
 public class ComputeIUF{


	private  Statement stmt = null;
	//private static Statement stmt2 = null;
	private Connection conn = null;
	private static String userTableName;
    private static String ratingsTableName;
	private static String ratingFieldName;
	private static String userFieldName;
	private static String itemFieldName;
	private static String userInfoTable;
	private static String userInfoID;
	private static String userInfoSamplingField;
	private static String userInfoNumRatings;
	
	private static String installdir;
	private int maxUsers = 0;
	private int numUsers = 0;
	private int maxItems = 0;
	
	private float[] itemCount = null;
	
	private PreparedStatement ps1, ps2;
        
	public ComputeIUF(){
		SystemUtils.printDebug("Starting Compute IUF ...");
		
		Properties props = new Properties();
        installdir = System.getProperty("cf.path",".");
       
		
		// DB information, contained in the CFServer.properties file.
		// Load properties
        SystemProperties.loadProperties(installdir + "/lib/CFServer.properties");
        
		ratingsTableName = SystemProperties.getString("CFServer.db.ratingsTable", "");     
        userFieldName = SystemProperties.getString("CFServer.db.userField", "");
        itemFieldName = SystemProperties.getString("CFServer.db.itemField", "");
        ratingFieldName = SystemProperties.getString("CFServer.db.ratingField", "");
                

        userInfoTable = SystemProperties.getString("CFServer.db.userInfo.table", "");               
        userInfoID = SystemProperties.getString("CFServer.db.userInfo.userField", "");          
        userInfoSamplingField = SystemProperties.getString("CFServer.db.userInfo.samplingField", "");
		userInfoNumRatings = SystemProperties.getString("CFServer.db.userInfo.numRatingsField", "");;
      
      	String query = "DELETE FROM " + userInfoTable;
		try {
            conn = getConnection();
            stmt = conn.createStatement();
            stmt.executeUpdate(query);
            precompileSQLStatements(conn);
        //    conn.close();
		} catch (SQLException e) {
            System.out.println("SQLException:" + e);
            System.exit(1);
        } catch (Exception e) {
            System.out.println("Exception:" + e);
            System.exit(1);
        }
        
        // cache itemCount and userCount
        try {
        	cacheCount();
        	//conn.close();
        } catch (SQLException e) {
            System.out.println("SQLException:" + e);
            System.exit(1);
        }
        
    }
    
	/**
     * compute the inverse user frequency
     * @throws SQLException
     */
	public void doComp() throws SQLException
	{
		SystemUtils.printDebug("Start Computing IUF ...");
					
		int currentUser= -1;
		int user =0;
		int ratingsCount= 0;
		int item;
		float sum = 0;
		boolean first= true;
		float itemFreq;
		ResultSet rs = null;		
		
		String query = "SELECT " + userFieldName + ", "  + itemFieldName + " FROM  " 
				+ ratingsTableName + " where 1 ORDER BY 1, 2";
		
		try{
			stmt = conn.createStatement();
        	rs= stmt.executeQuery(query);
        } catch(SQLException ex){
        	System.out.println("WARNING: Select failed...");
        	rs= null;
        }
        while (rs.next()){
			user = rs.getInt(userFieldName);
			
			if (currentUser != user){	
				if(!first){
					if(ratingsCount != 0){  //weight by ratingsCount/60 to discard high IUF's with few ratings
						if (ratingsCount > 65)
							itemFreq= sum/ratingsCount;
						else
							itemFreq= sum/65; 
					}
				 	else
				 		itemFreq= 0;
				 	
				 	ps1.setInt(1, currentUser);
					ps1.setInt(2, ratingsCount);
					ps1.setFloat(3, itemFreq);
					ps1.executeUpdate();
					
				} else {
					first = false;
					
				}
				//initialize for new user
				currentUser = user;
				sum = 0;
				ratingsCount = 0;
			
        	}
        	//increment sum
			item= rs.getInt(itemFieldName);
			sum += itemCount[item];
			ratingsCount++;
        	
		} // while
		
		// for last record
		if(ratingsCount != 0){  //weight by ratingsCount/60 to discard high IUF's with few ratings
			if (ratingsCount > 65)
				itemFreq= sum/ratingsCount;
			else
				itemFreq= sum/65; 
		} else
			itemFreq= 0;
		ps1.setInt(1, currentUser);
		ps1.setInt(2, ratingsCount);
		ps1.setFloat(3, itemFreq);
		ps1.executeUpdate();
		
		conn.close();
	} // doComp()
	
	/**
     Precompile SQLStatements.
     @param conn
     @throws SQLException
     */
    private void precompileSQLStatements(Connection conn) throws SQLException 
    {
    	SystemUtils.printDebug("Precompile SQL Statements ... ");
    	
    	ps1 = conn.prepareStatement("INSERT INTO  " + userInfoTable + " (" + userInfoID 
    								+ " , "  + userInfoNumRatings + " , "  
    								+ userInfoSamplingField + ") VALUES (?, ?, ?)" );
    	
    	ps2 = conn.prepareStatement("SELECT COUNT(*) FROM " + ratingsTableName + " WHERE "
    								+ itemFieldName + "=?");    	
    	    								
    																								 
    } // precompileSQLStatements()
    
    /**
     * cache some properties
     * @throws SQLException
     */
    private void cacheCount() throws SQLException
    {
    	SystemUtils.printDebug("Cache Count");
    	
    	String query = "SELECT MAX(" + itemFieldName + ") FROM " + ratingsTableName;
    
    	ResultSet rs = null;
    	
    	try {
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);
        } catch (Exception e) {
        
            System.out.println("Fatal Database Error: " + e);
            System.exit(1);
        }
    	if (rs != null && rs.next()) {
            maxItems = rs.getInt(1);
        }
        stmt.close();
        rs.close();
        
        query = "SELECT MAX(" + userFieldName + ") FROM " + ratingsTableName;
        try {         
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);
        } catch (Exception e) {
            System.out.println("Fatal Database Error: " + e);
            System.exit(1);
        }
    	if (rs != null && rs.next()) {
            maxUsers = rs.getInt(1);
        }
        stmt.close();
        rs.close();
        
        query = "SELECT COUNT(DISTINCT " + userFieldName + ") FROM " + ratingsTableName;
        try {         
            stmt = conn.createStatement();
            rs = stmt.executeQuery(query);
        } catch (Exception e) {
            System.out.println("Fatal Database Error: " + e);
            System.exit(1);
        }
    	if (rs != null && rs.next()) {
            numUsers = rs.getInt(1);
        }
        stmt.close();
        rs.close();
            
        itemCount = new float[maxItems + 1];
        cacheItemCount();
       
    }


    /**
     * cache the inverse item frequency
     * @throws SQLException
     */
    private void cacheItemCount() throws SQLException
    {
    	SystemUtils.printDebug("Cache Item Count. Total items " + maxItems);
    	long start = System.currentTimeMillis();
    
    	ResultSet rs = null;
    	itemCount[0] = 0;
    
    	for (int i = 1; i < itemCount.length; i++)
    	{
    		try {
    			ps2.setInt(1, i);
    			rs = ps2.executeQuery();
    		} catch (Exception e) {
            	System.out.println("Fatal Database Error: " + e);
            	System.exit(1);
        	}
        	if (rs != null && rs.next()) 
            	itemCount[i] = (float) Math.log(numUsers / ((float) rs.getInt(1)));
        	else
        		itemCount[i] = 0;
        	rs.close();
        	
    	} // for
    	
    	long stop = System.currentTimeMillis();
    	SystemUtils.printDebug("Spend " + (stop - start) + "ms Caching Item Count");
    	
    } // cacheItemCount()

   
    
    /**
     * Get connection with database.
     * Get database properties and establish connection to database.
     * @return Connection
     * @throws SQLException
     * @throws IOException
     */
    private Connection getConnection() throws SQLException, IOException {
        String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
        	System.setProperty("jdbc.drivers", drivers);
        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");
        return DriverManager.getConnection(url, username, password);        
        
    }
    
}