/* 
 * Database.java
 * 
 * Version 1.1
 * 
 * 15/08/2002
 *
 * Copyright (c) 2002 Oregon State Universtity. All rights reserved.
 */ 
 
package org.recommender.util;

import java.io.*;
import java.util.*;
import java.net.*;
import java.sql.*;


import org.recommender.server.*;

/**
 * Describes functions to maniplate databases thru JDBC.
 *
 * @author  (<A HREF="mailto:sameerk@cs.orst.edu">Sameer Kadam</A>)
 * @since   JDK 1.2
 * @version Version 1.0.0, 05-13-2001
 */
public class Database {

    /**
     * Load in database properties and create connection with 
     * database using the url, username, password specified in 
     * the properties.
     * @return Connection
     * @throws SQLException
     */
    public static Connection getConnection()     
	    throws SQLException {
	    
	    /* Get access to database from the client side */
	      String installdir = System.getProperty("cf.path",".");
          String filename = installdir + "/lib/" + "CFServer.properties";
	 
	      SystemProperties.loadProperties(filename);
	    
	     String drivers = SystemProperties.getString("CFServer.jdbc.drivers");
        if (drivers != null)
            System.setProperty("jdbc.drivers", drivers);

        String url = SystemProperties.getString("CFServer.jdbc.url");
        String username = SystemProperties.getString("CFServer.jdbc.username");
        String password = SystemProperties.getString("CFServer.jdbc.password");

        
         Connection conn = null;
         try {
               conn = DriverManager.getConnection(url, username, password);           
            
         } catch (SQLException e) {
            System.out.println("SQLException:");
            while (e != null) {
                System.out.println("SQLState: " + e.getSQLState());
                System.out.println("Message: " + e.getMessage());
                System.out.println("Vendor: " + e.getErrorCode());
                e = e.getNextException();
                System.out.println("");
            }
       
          }       
        
        return conn;
        
        // We now use a connection pooling approach. The real SQL driver
        // is set in the static initializer of this class.
      //  return DriverManager.getConnection("jdbc:jdc:jdcpool");
        
    }

    /**
     * Create an SQL table
     * @param tableName
     * @param tableFormat
     * @param stmt
     * @throws SQLException
     */
    public static void createTable(String tableName, String tableFormat, Statement stmt) 
    		throws SQLException 
    {
	    String command = "CREATE TABLE " + tableName + "(" + tableFormat + ")";
	    
	    try {
	    	stmt.executeUpdate(command);
	    } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            System.exit(1);
        }
        
    }

    /**
     * Insert value to a table.
     * @param tableName
     * @param values
     * @param stmt
     * @throws SQLException
     */
    public static void insertIntoTable(String tableName, String values, Statement stmt)
         throws SQLException 
    {
        String command = "INSERT INTO " + tableName + " VALUES (" + values + ")";
        
	    try {
	    	stmt.executeUpdate(command);
	    } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            System.exit(1);
        }
        	
    }

    /**
     * Print out the whole table.
     * @param tableName
     * @param stmt
     * @throws SQLException
     */
    public static void showTable(String tableName, Statement stmt) 
    	throws SQLException 
    {
        String query = "SELECT * FROM " + tableName;
	    ResultSet rs = stmt.executeQuery(query);
        ResultSetMetaData rsmd = rs.getMetaData();
        int colCount = rsmd.getColumnCount();
        while (rs.next()) {
            for (int i = 1; i <= colCount; i++) {
                if (i > 1)
                    System.out.print(", ");
                System.out.print(rs.getString(i));
            }
            System.out.println();
        }
        rs.close();
    }

    
}

