/* 
 * ItemDatabaseCreator.java
 * 
 * Version 1.1
 * 
 * 15/08/2002
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */ 
package org.recommender.util;

import org.recommender.server.*;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.net.*;
import java.sql.*;

/**
 * Describes a tool to read the Item data file in the 
 * dataset and poplulate a database (MySQL).
 * Contains methods for parsing the Item data.
 * This can either be run as a standalone tool or used as a class
 * from another class.
 * To use as standalone tool:
 *	make_user_db "itemDataFile" "itemDBName" 
 * To use from another class:
 * 	call static method createItemDBFromFile with itemDBName
 *	and itemDataFile as args.
 *
 * @author  Sameer Kadam Irwin Yoon
 * @since   JDK 1.2
 * @version Version 1.0.0, 05-13-2001
 * 
 */
public class ItemDatabaseCreator {

    private static Statement stmt = null;
    static final int MAX_TOKENS = 4;

    /**
     * Create Movie database from file.
     * First create a database using JDBC, then file the database
     * with data from movieDataFile.
     * @param itemDataFile
     * @param itemDBName
     */
    public static void createItemDBFromFile(String itemDataFile, String itemDBName) 
	{
	    Connection conn = null;
	    try {
	        // Create DB in using JDBC
	        //makeDB(itemDBName);
			conn = Database.getConnection();
        	stmt = conn.createStatement();

            // Fill DB with data from file
	        populateDB(itemDBName, itemDataFile);
	        conn.close();
	    } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            System.exit(1);
	    } catch (Exception e) {
	        System.out.println("Exception: " + e);
	        e.printStackTrace();
	    }
	    
    }

    /**
     * Private method, create a database using JDBC
     * @param dbName
     * @throws IOException
     * @throws SQLException
     */
  /*  private static void makeDB(String dbName) throws IOException, SQLException {
        Connection conn = Database.getConnection();
        stmt = conn.createStatement();
	    
        String dbFormat = UtilConstants.ITEM_DB_FORMAT;
        //Database.createTable(dbName, dbFormat, stmt);
    }*/

    /**
     * Private method, fill the database with data from file
     * @param dbName
     * @param fileName
     * @throws IOException
     * @throws SQLException
     */
    private static void populateDB(String dbName, String fileName) 
	                                 throws IOException , SQLException{
	    BufferedReader file = new BufferedReader(new FileReader(fileName));
	    String line = file.readLine();

	    while (line != null) {
	        parseString(line, dbName);
	        line = file.readLine();
	    }
    }

    /**
     * Given an entry for an item, parse it.
     * Here is an example entry:
     *
     * <PRE>
     * <Item_ID>    <Item_Name> 
     * </PRE>
     */
    public static void parseString(String str, String table) 
	                                             throws SQLException {
	   
	    StringTokenizer strtok = new StringTokenizer(str, "\t");
        String token;
        String movieID = null;
	    StringBuffer movieName = null;
				
	    String command = "INSERT INTO " + table + " VALUES (";

        try {
                    
             while(strtok.hasMoreTokens())
             { 
               token = strtok.nextToken();
            
               if(strtok.hasMoreTokens())            
	              command = command + token +",";
	           else
	              command = command + "\"" + token + "\"";
	          
	        
	          } 
	   	  

	    } catch (NoSuchElementException e) {  
	             
            if(strtok.hasMoreTokens())            
	            command = command + null +",";
	         else
	            command = command + null;        
	            
	    } catch (Exception e) {
            System.err.println(e);
            System.err.println("Error parsing a movie entry - entry follows");
            System.err.println(str);
        }
        command += ")";

	   // System.out.println(command);
	    stmt.executeUpdate(command);
    }

    /**
     * Main method, command line "Usage: make_item_db " +
	 *	"itemDataFile [itemDBName]"); 
	 */	
    public static void main(String[] args) {
	    if (args.length == 2) {
	        createItemDBFromFile(args[0], args[1]);
	    } else if (args.length == 1) {
	         createItemDBFromFile(args[0], UtilConstants.ITEM_TABLE);
        } else {
	        System.out.println("Usage: make_item_db " +
		    "itemDataFile [itemDBName]"); 
	       
	    }
	    
	     System.exit(0);
    }
}
