/* 
 * ItemTypeDatabaseCreator.java
 * 
 * Version 1.1
 * 
 * 01/27/2003
 *
 * Copyright (c) 2002 Oregon State Universtity. All rights reserved.
 */ 
package org.recommender.util;

import org.recommender.server.*;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.net.*;
import java.sql.*;

/**
 * Describes a tool to read the Item-Type pair file in the 
 * dataset and poplulate a database (MySQL).
 * Contains methods for parsing the  data.
 * This can either be run as a standalone tool or used as a class
 * from another class.
 * To use as standalone tool:
 *	make_itemtype_db "itemtypeFile" "itemtypeDBName" 
 * To use from another class:
 * 	call static method createItemTypeDBFromFile with itemtypeDBName
 *	and itemtypeDataFile as args.
 *
 * @author  Yun Wang
 * @since   JDK 1.2
 * @version Version 1.0.0, 05-13-2001
 *
 */
 public class ItemTypeDatabaseCreator {

    private static Statement stmt = null;
    static final int MAX_TOKENS = 4;
    private static String debugVar = null;

   
    /**
     * Create Rating database from file.
     * First create a database using JDBC, then fill the database
     * with data from itemtypeDataFile.
     * @param itemtypeDataFile
     * @param itemtypeDBName
     */
    public static void createItemTypeDBFromFile(String itemtypeDataFile, String itemtypeDBName) 
	{
	    Connection conn = null;
	    try {
	        // Create DB in using JDBC
	        //makeDB(itemtypeDBName);
	        conn = Database.getConnection();
        	stmt = conn.createStatement();

            // Fill DB with data from file
	        populateDB(itemtypeDBName, itemtypeDataFile);
	        conn.close();
	    } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
        //   ex.printStackTrace();
            System.exit(1);
	    } catch (Exception e) {
	        System.out.println("Exception: " + e);
	        e.printStackTrace();
	    }
	    
    }

    /**
     * Private method, create a database using JDBC
     * @param dbName
     * @throws IOException
     * @throws SQLException
     */
  /*  private static void makeDB(String dbName) throws IOException, SQLException {
        Connection conn = Database.getConnection();
        stmt = conn.createStatement();
	    
       // String dbFormat = UtilConstants.ITEM_TYPE_DB_FORMAT;
        //Database.createTable(dbName, dbFormat, stmt);
    }*/

    /**
     * Private method, fill the database with data from file
     * @param dbName
     * @param fileName
     * @throws IOException
     * @throws SQLException
     */
    private static void populateDB(String dbName, String fileName) 
	                                         throws IOException , SQLException{
	    BufferedReader file = new BufferedReader(new FileReader(fileName));
	    String line = file.readLine();

	    while (line != null) {
	        parseString(line, dbName);
	        line = file.readLine();
	    }
    }

    /**
     * Given an entry for a item-type pair, parse it.
     * Here is an example entry:
     *
     * <PRE>
     * <ITEM_ID>	<TYPE_ID> 
     * 23			10
     * </PRE>
     */
    public static void parseString(String str, String table) 
	                                          throws SQLException {
	
	    StringTokenizer strtok = new StringTokenizer(str, "\t");
        String token = null;
        String userID = null;
        String movieID = null;
        

	    String command = "INSERT INTO " + table + " VALUES (";

        
        StringBuffer s = new StringBuffer (command);
        try {
            while(strtok.hasMoreTokens())
           { 
               token = strtok.nextToken();
            
               if(strtok.hasMoreTokens())            
	              command = command + token +",";
	           else
	              command = command + token;
	          
	         
	        
	       // System.out.println(token +" " + tokenNum );
	        
	       } 
	   
          
	    } catch (NoSuchElementException e) { 
	               
             if(strtok.hasMoreTokens())            
	            command = command + null +",";
	         else
	            command = command + null;        
	            
	    } catch (Exception e) {
            System.err.println(e);
            System.err.println("Error parsing a rating entry - entry follows");
            System.err.println(str);
        }
       
       command += ")";
	
	    stmt.executeUpdate(command);
    }

    /**
     * Main method, command line "Usage: make_itemtype_db " +
	 *	"itemtypeDataFile [itemtypeDBName]"); 
	 */	
    public static void main(String[] args) {
	    String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                + "CFServer.properties";
    	SystemProperties.loadProperties(filename);
	    
	    if (args.length == 2) {
	        createItemTypeDBFromFile(args[0], args[1]);
	    } else if (args.length == 1) {
            createItemTypeDBFromFile(args[0], SystemProperties.getString("CFServer.db.itemTypeTable", ""));
        } else {
	        System.out.println("Usage: make_itemtype_db " +
		    "itemtypeDataFile [itemtypeDBName]"); 
	       
	    }
	    
	     System.exit(0);
    }
}
