/* 
 * LoggingUtil.java
 * 
 * Version 0.9
 * 
 * 05/18/2003
 *
 * Copyright (c) 2003 Oregon State Universtity. All rights reserved.
 */ 
package org.recommender.util;

import java.io.*;
import java.util.logging.*;


//import org.apache.log4j.*;      // commented corrently, we don't need to care about the log4j problem right now

/**
 * This Class is designed to be a initialize jdk1.4.1's Logger for CFEngine
 * need to be replaced be another version when log4j with jdk1.3 or lower is used.
 * <p>
 * Advice on using logger *
 * <p>
 * <ul>
 * <li>Avoid using string add, like logger.log(level.FINE, string1 + string2 + string3)  
 *     it cost more time and no better even when that logging line is disabled
 * <li>Use hierarchical names like org.recommender.server and org.recommender.interface
 * <li>Minimize use of SEVERE, WARNING, INFO
 * <li>Mostly use FINE, FINER, FINEST
 * </ul>
 *
 * @author  Wei Wu
 * @since   JDK 1.3
 * @version Version 0.8.0, 04-15-2003
 * @version Version 0.9.0, 05-01-2003
 *  change from Jakarta Log4J to the JDK1.4.1's logging api
 * 
 */
public class LoggingUtil {

    // constant variables defined
    public static final int JDK141LOGGER = 0;
    public static final int LOG4J = 1;
    public static final int SIMPLEFORMATLOG = 0;
    public static final int XMLFORMATLOG = 1;
    public static final int LOG4JFORMATLOG = 2;

    public static int logFormat = LoggingUtil.SIMPLEFORMATLOG;
    public static String fileLocation = "cfengine.log";

    private static Logger logger = Logger.getLogger("org.recommender.testing");

    //public static int loggingSystem = LoggingUtil.JDK141LOGGER;
    
    /**
     * initialize the logging system for cfengine
     * this function will add a file handler for the logging system by default
     * by default
     * <ul>
     * <li>the format will be set to <code>java.util.logging.SimpleFormatter</code>
     * <li>all the log hierarchy all open by default
     * </ul>
     */
    public static void initLoggingSystem() {

        // init Logging System

        // set the Handler
        Handler fh = null;
        try {
            fh = new FileHandler(LoggingUtil.fileLocation);
        } catch (java.io.IOException ioe) {
            logger.log(Level.WARNING, "logging system dosn't start properly");
            ioe.printStackTrace();
        }

        // set log format
        switch (logFormat) {
            case LoggingUtil.SIMPLEFORMATLOG:
                fh.setFormatter(new SimpleFormatter());
                break;
            case LoggingUtil.XMLFORMATLOG:
                fh.setFormatter(new XMLFormatter());
                break;
            case LoggingUtil.LOG4JFORMATLOG:
                //fh.setFormatter(new Log4JFormatter());
                break;
            default:
        }
        Logger.getLogger("").addHandler(fh);

        // set default log level
        logger.setLevel(Level.WARNING);

    }

    /**
     *  this main method is designed for unit testing
     *  remember the old level setting before test, then open all level and
     *  test all the log level. set back to old level after the test
     */
    public static void main(String[] args) {
        // unit testing
        LoggingUtil.initLoggingSystem();

        // remember the old level
        Level lev = logger.getLevel();

        logger.setLevel(Level.ALL);
        logger.log(Level.FINEST, "old logger level is saved as " + lev);

        logger.log(Level.FINEST, "Finest logging message display ok");
        logger.log(Level.FINER, "Finer logging message display ok");
        logger.log(Level.FINE, "Fine logging message display ok");
        logger.log(Level.CONFIG, "CONFIG logging message display ok");
        logger.log(Level.INFO, "INFO logging message display ok");
        logger.log(Level.WARNING, "WARN logging message display ok");
        logger.log(Level.SEVERE, "SEVERE logging message display ok");

        logger.log(Level.FINE, "UnitTesting Done Successfully");

        // set back to the old level
        logger.setLevel(lev);
    }
}
