/* 
 * RatingDatabaseCreator.java
 * 
 * Version 1.1
 * 
 * 15/08/2002
 *
 * Copyright (c) 2002 Oregon State Universtity. All rights reserved.
 */ 
package org.recommender.util;

import org.recommender.server.*;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.net.*;
import java.sql.*;

/**
 * Describes a tool to read the Ratings data file in the 
 * dataset and poplulate a database (MySQL).
 * Contains methods for parsing the  data.
 * This can either be run as a standalone tool or used as a class
 * from another class.
 * To use as standalone tool:
 *	make_rating_db "ratingDataFile" "ratingDBName" 
 * To use from another class:
 * 	call static method createRatingDBFromFile with ratingDBName
 *	and ratingDataFile as args.
 *
 * @author  Sameer Kadam Irwin Yoon
 * @since   JDK 1.2
 * @version Version 1.0.0, 05-13-2001
 *
 */
public class RatingDatabaseCreator {

    private static Statement stmt = null;
    static final int MAX_TOKENS = 4;
    private static String debugVar = null;

    /**
     * Create Rating database from file.
     * First create a database using JDBC, then fill the database
     * with data from ratingDataFile.
     * @param ratingDataFile
     * @param ratingDBName
     */
    public static void createRatingDBFromFile(String ratingDataFile, String ratingDBName) 
    {
	    Connection conn = null;
	    try {
	        // Create DB in using JDBC
		// We should separate the creation of tables
		// (very DB specific) and the populating of tables
		// This no longer creates the table - CLEAN!
	      //  makeDB(ratingDBName);
			conn = Database.getConnection();
        	stmt = conn.createStatement();
            // Fill DB with data from file
	        populateDB(ratingDBName, ratingDataFile);
	        conn.close();
	    } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
         //  ex.printStackTrace();
            System.exit(1);
	    } catch (Exception e) {
	        System.out.println("Exception: " + e);
	        e.printStackTrace();
	    }
    }

    /**
     * Private method, create a database using JDBC
     * @param dbName
     * @throws IOException
     * @throws SQLException
     */
   /* private static void makeDB(String dbName) throws IOException, SQLException {
        Connection conn = Database.getConnection();
        stmt = conn.createStatement();
	    
       // String dbFormat = UtilConstants.RATING_DB_FORMAT;
        //Database.createTable(dbName, dbFormat, stmt);
    }*/

    /**
     * Private method, fill the database with data from file
     * @param dbName
     * @param fileName
     * @throws IOException
     * @throws SQLException
     */
    private static void populateDB(String dbName, String fileName) 
	                                         throws IOException , SQLException{
	    BufferedReader file = new BufferedReader(new FileReader(fileName));
	    String line = file.readLine();

	    while (line != null) {
	        parseString(line, dbName);
	        line = file.readLine();
	    }
    }

    /**
     * Given an entry for a rating, parse it.
     * Here is an example entry:
     *
     * <PRE>
     * <USER_ID> <ITEM_ID> <RATING>
     * 3226      23       0.80 
     * </PRE>
     */
    public static void parseString(String str, String table) 
	                                          throws SQLException {
	
	    StringTokenizer strtok = new StringTokenizer(str, "\t");
        String token = null;
        String userID = null;
        String movieID = null;
        

	    String command = "INSERT INTO " + table + " VALUES (";

        
        StringBuffer s = new StringBuffer (command);
        try {
            while(strtok.hasMoreTokens())
           { 
               token = strtok.nextToken();
            
               if(strtok.hasMoreTokens())            
	              command = command + token +",";
	           else
	              command = command + token;
	          
	         
	        
	       // System.out.println(token +" " + tokenNum );
	        
	       } 
	   
          
	    } catch (NoSuchElementException e) { 
	               
             if(strtok.hasMoreTokens())            
	            command = command + null +",";
	         else
	            command = command + null;        
	            
	    } catch (Exception e) {
            System.err.println(e);
            System.err.println("Error parsing a rating entry - entry follows");
            System.err.println(str);
        }
       
       command += ")";
	
	    stmt.executeUpdate(command);
    }

    /**
     * Main method, command line "Usage: make_rating_db " +
	 *	"ratingDataFile [ratingDBName]"); 
	 */	
    public static void main(String[] args) {
	    String installdir = System.getProperty("cf.path",".");
        String filename = installdir + "/lib/" 
                + "CFServer.properties";
    	SystemProperties.loadProperties(filename);
    	
	    if (args.length == 2) {
	        createRatingDBFromFile(args[0], args[1]);
	    } else if (args.length == 1) {
            createRatingDBFromFile(args[0], SystemProperties.getString( 
						     "CFServer.db.ratingsTable", ""));
        } else {
	        System.out.println("Usage: make_rating_db " +
		    "ratingDataFile [ratingDBName]"); 
	       
	    }
	    
	     System.exit(0);
    }
}
