/* 
 * UserDatabaseCreator.java
 * 
 * Version 1.1
 * 
 * 15/08/2002
 *
 * Copyright (c) 2002 Oregon State Universtity. All rights reserved.
 */ 
package org.recommender.util;

//import org.recommender.server.*;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.net.*;
import java.sql.*;

/**
 * Describes a tool to read the User data file in
 * dataset and poplulate a database (MySQL).
 * Contains methods for parsing the  data.
 * This can either be run as a standalone tool or used as a class
 * from another class.
 * To use as standalone tool:
 *	java UserDatabaseCreator "userDataFile" "userDBName" 
 * To use from another class:
 * 	call static method createUserDBFromFile with userDBName
 *	and userDataFile as args.
 *
 * @author  Sameer Kadam Irwin Yoon
 * @since   JDK 1.2
 * @version Version 1.0.0, 05-13-2001
 *
 * Revisions:	IY	8/2001	make changes to fit my db
 */

public class UserDatabaseCreator {

    private static Statement stmt = null;
    static final int MAX_TOKENS = 4;

    /**
     * Create Rating database from file.
     * First create a database using JDBC, then fill the database
     * with data from userDataFile.
     * @param userDataFile
     * @param userDBName
     */
    public static void createUserDBFromFile(String userDataFile, String userDBName) 
    {
	    Connection conn = null;
	    try {
	        // Create DB in using JDBC
	        //makeDB(itemDBName);
			conn = Database.getConnection();
        	stmt = conn.createStatement();

            // Fill DB with data from file
	        populateDB(userDBName, userDataFile);
	        conn.close();
	    } catch(SQLException ex){
        	System.out.println("Fatal Database Error: " + ex);
            System.exit(1);
	    } catch (Exception e) {
	        System.out.println("Exception: " + e);
	        e.printStackTrace();
	    }
    }

    /**
     * Private method, create a database using JDBC
     * @param dbName
     * @throws IOException
     * @throws SQLException
     */
  /*  private static void makeDB(String dbName) throws IOException, SQLException {
        Connection conn = Database.getConnection();
        stmt = conn.createStatement();
	    
        String dbFormat = UtilConstants.USER_DB_FORMAT;
        //Database.createTable(dbName, dbFormat, stmt);
    }*/

    /**
     * Private method, fill the database with data from file
     * @param dbName
     * @param fileName
     * @throws IOException
     * @throws SQLException
     */
    private static void populateDB(String dbName, String fileName) 
	                                          throws IOException , SQLException{
	    BufferedReader file = new BufferedReader(new FileReader(fileName));
	    String line = file.readLine();

	    while (line != null) {
	        parseString(line, dbName);
	        line = file.readLine();
	    }
    }

   /**
    * Given an entry for a user, parse it.
    * Here is an example entry:
    *
    * <PRE>
    * <User_ID>    <User_Name> 
    * </PRE>
    */
    public static void parseString(String str, String table) 
	                                               throws SQLException { 
	
	    StringTokenizer strtok = new StringTokenizer(str, "\t");
        String token;
	    int tokenNum = 0;

	    String command = "INSERT INTO " + table + " VALUES (";

        try {
           
          while(strtok.hasMoreTokens())
          { 
            token = strtok.nextToken();
            
            if(strtok.hasMoreTokens())            
	          command = command + token +",";
	        else
	          command = command + "\"" + token + "\"";
	          
	        tokenNum++;
	        
	       // System.out.println(token +" " + tokenNum );
	        
	       } 
	   
          
           
            
	    } catch (NoSuchElementException e) { 
	    
	         if(strtok.hasMoreTokens())            
	            command = command + null +",";
	         else
	            command = command + null;             
           
	    } catch (Exception e) {
            System.err.println(e);
            System.err.println("Error parsing a person entry - entry follows");
            System.err.println(str);
        } 
        
        command += ")";
       // System.out.println(command);
	    stmt.executeUpdate(command);
    }

    /**
     * Main method, command line "Usage: make_user_db " +
	 *	"userDataFile [userDBName]"); 
	 */	
    public static void main(String[] args) {
	    if (args.length == 2) {
	        createUserDBFromFile(args[0], args[1]);
	    } else if (args.length == 1) {
            createUserDBFromFile(args[0], UtilConstants.USER_TABLE);
        } else {
	        System.out.println("Usage: make_user_db " +
		    "userDataFile [userDBName]"); 
	       
	    }
	    
	     System.exit(0);
    }
    
    
}
